package com.gx.obe.server.management.boq.controller;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.boq.entity.ObeBillTable;
import com.gx.obe.server.management.boq.service.ObeBillTableService;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Api(tags = "分部分项")
@RestController
@AllArgsConstructor
@RequestMapping("/obeBillTable")
public class ObeBillTableController extends BaseController<ObeBillTableService, ObeBillTable> {

    @Autowired
    public ObeBillTableService ObeBillTableServices;

    /**
     * @param ObeBillTableList
     * @Description: 批量插入
     * @author mazc
     */
    @PostMapping("/insertByBatch")
    public int insertByBatch(@RequestBody List<ObeBillTable> ObeBillTableList) {
        return ObeBillTableServices.insertByBatch(ObeBillTableList);
    }

    /**
     * @param ObeBillTableList
     * @Description: 批量更新
     * @author mazc
     */
    @PostMapping("/updateByBatch")
    public int updateByBatch(@RequestBody List<ObeBillTable> ObeBillTableList) {
        return ObeBillTableServices.updateByBatch(ObeBillTableList);
    }

    /**
     * @param tenderId
     * @param supplierId
     * @return
     * @Description: 获得供应商全部分布分项的清单列表
     */
    @GetMapping("/getAllBillTableQDList")
    public List<ObeBillTable> getAllBillTableQDList(String tenderId, String supplierId) {
        return ObeBillTableServices.getAllBillTableQDList(tenderId, supplierId);
    }

    /**
     * @param tenderId
     * @param projectCode
     * @return
     * @Description: 获得招标项目的分布分项的清单列表
     * @author guoyr
     */
    @GetMapping("/getTenderProjectBillTableQdList")
    public List<ObeBillTable> getTenderProjectBillTableQdList(String tenderId, String projectCode) {
        return ObeBillTableServices.getTenderProjectBillTableQdList(tenderId, projectCode);
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得分布分项的清单列表
     * @author guoyr
     */
    @GetMapping("/getBillTableQdList")
    public List<ObeBillTable> getBillTableQdList(String projectId) {
        return ObeBillTableServices.getBillTableQdList(projectId);
    }

    /**
     * @param projectId 单位工程编号
     * @return List<BillTable>
     * @Description: 根据工程项目编号"，获得分部分项信息(分布工程、清单、定额、工料机)
     * @author guoyr
     */
    @GetMapping("/getBillTableList")
    public List<ObeBillTable> getBillTableList(String projectId) {
        return ObeBillTableServices.getBillTableList(projectId);
    }

    /**
     * @param constructionIds
     * @return
     * @Description: 获得供应商全部分布分项的清单列表
     * @author mazc
     */
    @PostMapping("/getBillTableBTConstructionIdsList")
    public List<ObeBillTable> getBillTableBTConstructionIdsList(@RequestBody List<String> constructionIds) {
        return ObeBillTableServices.getBillTableBTConstructionIdsList(constructionIds);
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得分布分项的清单和定额列表
     * @author guoyr
     */
    @GetMapping("/getBillTableQdAndDeList")
    public List<ObeBillTable> getBillTableQdAndDeList(String projectId) {
        return ObeBillTableServices.getBillTableQdAndDeList(projectId);
    }

}