package com.gx.obe.server.management.boq.controller;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.boq.entity.ObeMeasureTable;
import com.gx.obe.server.management.boq.service.ObeMeasureTableService;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Api(tags = "措施项目")
@RestController
@AllArgsConstructor
@RequestMapping("/obeMeasureTable")
public class ObeMeasureTableController extends BaseController<ObeMeasureTableService, ObeMeasureTable> {
    @Autowired
    public ObeMeasureTableService ObeMeasureTableServices;

    /**
     * @param ObeMeasureTableList
     * @Description: 批量插入
     * @author mazc
     */
    @PostMapping("/insertByBatch")
    public int insertByBatch(@RequestBody List<ObeMeasureTable> ObeMeasureTableList) {
        return ObeMeasureTableServices.insertByBatch(ObeMeasureTableList);
    }

    /**
     * @param ObeMeasureTableList
     * @Description: 批量更新
     * @author mazc
     */
    @PostMapping("/updateByBatch")
    public int updateByBatch(@RequestBody List<ObeMeasureTable> ObeMeasureTableList) {
        return ObeMeasureTableServices.updateByBatch(ObeMeasureTableList);
    }

    /**
     * @param tenderId
     * @param supplierId
     * @return
     * @Description: 根据措施项目清单列表
     * @author mazc
     */
    @GetMapping("/getAllMeasureTableQDList")
    public List<ObeMeasureTable> getAllMeasureTableQDList(String tenderId, String supplierId, String measureType) {
        return ObeMeasureTableServices.getAllMeasureTableQDList(tenderId, supplierId, measureType);
    }

    /**
     * @param tenderId
     * @param projectCode
     * @return
     * @Description: 获得招标项目的措施清单列表
     * @author guoyr
     */
    @GetMapping("/getTenderProjectMeasureTableQdList")
    public List<ObeMeasureTable> getTenderProjectMeasureTableQdList(String tenderId, String projectCode, String measureType) {
        return ObeMeasureTableServices.getTenderProjectMeasureTableQdList(tenderId, projectCode, measureType);
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得措施的清单列表
     * @author guoyr
     */
    @GetMapping("/getMeasureTableQdList")
    public List<ObeMeasureTable> getMeasureTableQdList(String projectId) {
        return ObeMeasureTableServices.getMeasureTableQdList(projectId);
    }

    /**
     * @param projectId
     * @param measureType
     * @return
     * @Description: 获得措施项目(清单 、 定额 、 工料机汇总)
     * @author guoyr
     */
    @GetMapping("/getMeasureTableListByProjectId")
    public List<ObeMeasureTable> getMeasureTableListByProjectId(String projectId, String measureType) {
        return ObeMeasureTableServices.getMeasureTableListByProjectId(projectId, measureType);
    }

    /**
     * @param requestBodyEntity
     * @return
     * @Description: 根据项目Id，获得措施项目(清单、定额、工料机汇总)
     * @author mazc
     */
    @PostMapping("/getMeasureTableByConstructionIdsList")
    public List<ObeMeasureTable> getMeasureTableByConstructionIdsList(@RequestBody RequestBodyEntity<String> requestBodyEntity) {
        List<String> constructionIds = requestBodyEntity.getEntityList();
        String measureType = (String) requestBodyEntity.getParam("measureType");
        return ObeMeasureTableServices.getMeasureTableByConstructionIdsList(constructionIds, measureType);
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得措施的清单和定额列表
     * @author guoyr
     */
    @GetMapping("/getMeasureTableQdAndDeList")
    public List<ObeMeasureTable> getMeasureTableQdAndDeList(String projectId) {
        return ObeMeasureTableServices.getMeasureTableQdAndDeList(projectId);
    }

}