package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.EngineerConstants;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeBillTableMapper;
import com.gx.obe.server.management.boq.entity.ObeBillTable;
import com.gx.obe.server.management.boq.service.ObeBillTableService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeBillTableServiceImpl extends ServiceImpl<ObeBillTableMapper, ObeBillTable> implements ObeBillTableService {

    @Autowired
    private ObeBillTableMapper obeBillTableMapper;

    /**
     * @param ObeBillTableList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeBillTable> ObeBillTableList) {
        return obeBillTableMapper.updateBatchList(ObeBillTableList);
    }

    /**
     * @param ObeBillTableList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeBillTable> ObeBillTableList) {
        return obeBillTableMapper.insertByBatch(ObeBillTableList);
    }

    /**
     * @param ObeBillTable
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeBillTable ObeBillTable, String[] attributes) {
        return obeBillTableMapper.updateAssignProperty(ObeBillTable, attributes) > 0;
    }

    /**
     * @param ObeBillTableList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeBillTable> ObeBillTableList, String[] attributes) {
        return obeBillTableMapper.batchUpdateProperty(ObeBillTableList, attributes);
    }

    /**
     * @param ObeBillTableList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeBillTable> ObeBillTableList, String[] attributes) {
        List<ObeBillTable> insertObeBillTableList = new ArrayList<>();
        List<ObeBillTable> updateObeBillTableList = new ArrayList<>();
        for (ObeBillTable ObeBillTable : ObeBillTableList) {
            if (StringUtils.isEmpty(ObeBillTable.getId())) {
                ObeBillTable.setId(IDUtils.getId());
                insertObeBillTableList.add(ObeBillTable);
            } else {
                updateObeBillTableList.add(ObeBillTable);
            }
        }
        int count = 0;
        if (insertObeBillTableList.size() > 0) {
            count += obeBillTableMapper.insertByBatch(insertObeBillTableList);
        }
        if (updateObeBillTableList.size() > 0) {
            count += obeBillTableMapper.batchUpdateProperty(updateObeBillTableList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param supplierId
     * @return
     * @Description: 获得供应商全部分布分项的清单列表
     */
    public List<ObeBillTable> getAllBillTableQDList(String tenderId, String supplierId) {
        return obeBillTableMapper.getAllBillTableQDList(tenderId, supplierId);
    }

    /**
     * @param tenderId
     * @param projectCode
     * @return
     * @Description: 获得招标项目的分布分项的清单列表
     * @author guoyr
     */
    public List<ObeBillTable> getTenderProjectBillTableQdList(String tenderId, String projectCode) {
        return obeBillTableMapper.getTenderProjectBillTableQdList(tenderId, projectCode);
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得分布分项的清单列表
     * @author guoyr
     */
    public List<ObeBillTable> getBillTableQdList(String projectId) {
        return obeBillTableMapper.getBillTableQdList(projectId);
    }

    /**
     * @param projectId 单位工程编号
     * @return List<BillTable>
     * @Description: 根据工程项目编号"，获得分部分项信息(分布工程、清单、定额、工料机)
     * @author guoyr
     */
    public List<ObeBillTable> getBillTableList(String projectId) {
        return obeBillTableMapper.getBillTableList(projectId);
    }

    /**
     * @param constructionIds
     * @return
     * @Description: 获得供应商全部分布分项的清单列表
     * @author mazc
     */
    public List<ObeBillTable> getBillTableBTConstructionIdsList(List<String> constructionIds) {
        return obeBillTableMapper.getBillTableBTConstructionIdsList(constructionIds);
    }

    /**
     * @param projectId
     * @return
     * @Description: 获得分布分项的清单和定额列表
     * @author guoyr
     */
    public List<ObeBillTable> getBillTableQdAndDeList(String projectId) {
        List<ObeBillTable> result = new ArrayList<>();
        List<ObeBillTable> billTableList = obeBillTableMapper.getBillTableQdAndDeList(projectId);
        for(ObeBillTable qdBill : billTableList){
            if(ObjectUtils.equals(qdBill.getKind(), EngineerConstants.QD)){
                for(ObeBillTable deBill : billTableList){
                    if(ObjectUtils.equals(deBill.getKind(), EngineerConstants.DE) && StringUtils.equals(qdBill.getId(), deBill.getParentId())){
                        qdBill.addChild(deBill);
                        deBill.setParentBill(qdBill);
                    }
                }
                result.add(qdBill);
            }
        }
        return result;
    }

}
