package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeExtraTableMapper;
import com.gx.obe.server.management.boq.entity.ObeExtraTable;
import com.gx.obe.server.management.boq.service.ObeExtraTableService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeExtraTableServiceImpl extends ServiceImpl<ObeExtraTableMapper, ObeExtraTable> implements ObeExtraTableService {

    @Autowired
    private ObeExtraTableMapper obeExtraTableMapper;

    /**
     * @param ObeExtraTableList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeExtraTable> ObeExtraTableList) {
        return obeExtraTableMapper.updateBatchList(ObeExtraTableList);
    }

    /**
     * @param ObeExtraTableList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeExtraTable> ObeExtraTableList) {
        return obeExtraTableMapper.insertByBatch(ObeExtraTableList);
    }

    /**
     * @param ObeExtraTable
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeExtraTable ObeExtraTable, String[] attributes) {
        return obeExtraTableMapper.updateAssignProperty(ObeExtraTable, attributes) > 0;
    }

    /**
     * @param ObeExtraTableList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeExtraTable> ObeExtraTableList, String[] attributes) {
        return obeExtraTableMapper.batchUpdateProperty(ObeExtraTableList, attributes);
    }

    /**
     * @param ObeExtraTableList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeExtraTable> ObeExtraTableList, String[] attributes) {
        List<ObeExtraTable> insertObeExtraTableList = new ArrayList<>();
        List<ObeExtraTable> updateObeExtraTableList = new ArrayList<>();
        for (ObeExtraTable ObeExtraTable : ObeExtraTableList) {
            if (StringUtils.isEmpty(ObeExtraTable.getId())) {
                ObeExtraTable.setId(IDUtils.getId());
                insertObeExtraTableList.add(ObeExtraTable);
            } else {
                updateObeExtraTableList.add(ObeExtraTable);
            }
        }
        int count = 0;
        if (insertObeExtraTableList.size() > 0) {
            count += obeExtraTableMapper.insertByBatch(insertObeExtraTableList);
        }
        if (updateObeExtraTableList.size() > 0) {
            count += obeExtraTableMapper.batchUpdateProperty(updateObeExtraTableList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param supplierId
     * @return
     * @Description: 获得供应商全部其他项目清单
     * @author mazc
     */
    public List<ObeExtraTable> getAllExtraTableQDList(String tenderId, String supplierId) {
        return obeExtraTableMapper.getAllExtraTableQDList(tenderId, supplierId);
    }

    /**
     * @param tenderId
     * @param projectCode
     * @return
     * @Description: : 获得招标项目的其他项目的清单列表
     * @author mazc
     */
    public List<ObeExtraTable> getTenderProjectExtraTableQdList(String tenderId, String projectCode) {
        return obeExtraTableMapper.getTenderProjectExtraTableQdList(tenderId, projectCode);
    }

    /**
     * @param projectId
     * @return List<ExtraTable>
     * @Description: 根据项目Id， 获得其他项目信息。
     * @author guoyr
     */
    public List<ObeExtraTable> getExtraTableListByProjectId(String projectId) {
        return obeExtraTableMapper.getExtraTableListByProjectId(projectId);
    }

}
