package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeQuotaItemMapper;
import com.gx.obe.server.management.boq.entity.ObeQuotaItem;
import com.gx.obe.server.management.boq.service.ObeQuotaItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeQuotaItemServiceImpl extends ServiceImpl<ObeQuotaItemMapper, ObeQuotaItem> implements ObeQuotaItemService {

    @Autowired
    private ObeQuotaItemMapper obeQuotaItemMapper;

    /**
     * @param ObeQuotaItemList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeQuotaItem> ObeQuotaItemList) {
        return obeQuotaItemMapper.updateBatchList(ObeQuotaItemList);
    }

    /**
     * @param ObeQuotaItemList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeQuotaItem> ObeQuotaItemList) {
        return obeQuotaItemMapper.insertByBatch(ObeQuotaItemList);
    }

    /**
     * @param ObeQuotaItem
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeQuotaItem ObeQuotaItem, String[] attributes) {
        return obeQuotaItemMapper.updateAssignProperty(ObeQuotaItem, attributes) > 0;
    }

    /**
     * @param ObeQuotaItemList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeQuotaItem> ObeQuotaItemList, String[] attributes) {
        return obeQuotaItemMapper.batchUpdateProperty(ObeQuotaItemList, attributes);
    }

    /**
     * @param ObeQuotaItemList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeQuotaItem> ObeQuotaItemList, String[] attributes) {
        List<ObeQuotaItem> insertObeQuotaItemList = new ArrayList<>();
        List<ObeQuotaItem> updateObeQuotaItemList = new ArrayList<>();
        for (ObeQuotaItem ObeQuotaItem : ObeQuotaItemList) {
            if (StringUtils.isEmpty(ObeQuotaItem.getId())) {
                ObeQuotaItem.setId(IDUtils.getId());
                insertObeQuotaItemList.add(ObeQuotaItem);
            } else {
                updateObeQuotaItemList.add(ObeQuotaItem);
            }
        }
        int count = 0;
        if (insertObeQuotaItemList.size() > 0) {
            count += obeQuotaItemMapper.insertByBatch(insertObeQuotaItemList);
        }
        if (updateObeQuotaItemList.size() > 0) {
            count += obeQuotaItemMapper.batchUpdateProperty(updateObeQuotaItemList, attributes);
        }
        return count;
    }

    /**
     * @param billId
     * @return
     * @Description: 获得清单的定额列表
     */
    public List<ObeQuotaItem> getBillQuotaItemList(String billId) {
        return obeQuotaItemMapper.getBillQuotaItemList(billId);
    }

}
