package com.gx.obe.server.management.boq.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.boq.dao.ObeRatesTableMapper;
import com.gx.obe.server.management.boq.entity.ObeRatesTable;
import com.gx.obe.server.management.boq.service.ObeRatesTableService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeRatesTableServiceImpl extends ServiceImpl<ObeRatesTableMapper, ObeRatesTable> implements ObeRatesTableService {

    @Autowired
    private ObeRatesTableMapper obeRatesTableMapper;

    /**
     * @param ObeRatesTableList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeRatesTable> ObeRatesTableList) {
        return obeRatesTableMapper.updateBatchList(ObeRatesTableList);
    }

    /**
     * @param ObeRatesTableList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeRatesTable> ObeRatesTableList) {
        return obeRatesTableMapper.insertByBatch(ObeRatesTableList);
    }

    /**
     * @param ObeRatesTable
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeRatesTable ObeRatesTable, String[] attributes) {
        return obeRatesTableMapper.updateAssignProperty(ObeRatesTable, attributes) > 0;
    }

    /**
     * @param ObeRatesTableList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeRatesTable> ObeRatesTableList, String[] attributes) {
        return obeRatesTableMapper.batchUpdateProperty(ObeRatesTableList, attributes);
    }

    /**
     * @param ObeRatesTableList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeRatesTable> ObeRatesTableList, String[] attributes) {
        List<ObeRatesTable> insertObeRatesTableList = new ArrayList<>();
        List<ObeRatesTable> updateObeRatesTableList = new ArrayList<>();
        for (ObeRatesTable ObeRatesTable : ObeRatesTableList) {
            if (StringUtils.isEmpty(ObeRatesTable.getId())) {
                ObeRatesTable.setId(IDUtils.getId());
                insertObeRatesTableList.add(ObeRatesTable);
            } else {
                updateObeRatesTableList.add(ObeRatesTable);
            }
        }
        int count = 0;
        if (insertObeRatesTableList.size() > 0) {
            count += obeRatesTableMapper.insertByBatch(insertObeRatesTableList);
        }
        if (updateObeRatesTableList.size() > 0) {
            count += obeRatesTableMapper.batchUpdateProperty(updateObeRatesTableList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param supplierId
     * @return
     * @Description: 获得供应商的费率列表
     */
    public List<ObeRatesTable> getRatesTableList(String tenderId, String supplierId, String projectId, String rateType) {
        return obeRatesTableMapper.getRatesTableList(tenderId, supplierId, projectId, rateType);
    }

}
