package com.gx.obe.server.management.config.controller;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.utils.DateUtils;
import com.gx.obe.server.management.config.entity.SysConfigValue;
import com.gx.obe.server.management.config.service.SysConfigValueService;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/sysConfigValue")
public class SysConfigValueController extends BaseController<SysConfigValueService,SysConfigValue>{
	@Autowired
	public SysConfigValueService sysConfigValueService;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param sysConfigValueList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<SysConfigValue> sysConfigValueList) {
		return sysConfigValueService.insertByBatch(sysConfigValueList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param sysConfigValueList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<SysConfigValue> sysConfigValueList) {
		return sysConfigValueService.updateByBatch(sysConfigValueList);
	}
	
	/**
	 * @Description: 指定字段修改
	 * @param vo
	 * @return
	 */
	@PostMapping("/updateAssignProperty")
	public boolean  updateAssignProperty(@RequestBody UpdateAssignPropertyVo<SysConfigValue> vo) {
		return sysConfigValueService.updateAssignProperty(vo.getEntity(), vo.getAttributes());
	}
	
	/**
	 * @Description: 根据配置编号获得数据配置值列表
	 * @author wangxiang
	 * @param configCode
	 * @return 
	 */
	@GetMapping("/getConfigValueListByConfigCode")
	public List<SysConfigValue> getConfigValueListByConfigCode(String configCode){
		return sysConfigValueService.getConfigValueListByConfigCode(configCode);
	}
	
	
	/**
	 * @Description: 验证配置值编码唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param configValueCode
	 * @return 
	 */
	@GetMapping("/isUniqueConfigValueCode")
	public boolean isUniqueConfigValueCode(String configId, String id, String configValueCode){
		return sysConfigValueService.isUniqueConfigValueCode(configId, id, configValueCode);
	}
	
	/**
	 * @Description: 获得数据配置值的最大序号
	 * @author mazc
	 * @return
	 */
	@GetMapping("/getMaxConfigValueSortNo")
	public int getMaxConfigValueSortNo(String configId){
		return sysConfigValueService.getMaxConfigValueSortNo(configId);
	}
	/**
	 * @Description: 获得数据配置值的最大序号
	 * @author mazc
	 * @return
	 */
	@GetMapping("/getLatestModifyTime")
	public String getLatestModifyTime(String configCode){
		Date latestModifyTime = sysConfigValueService.getLatestModifyTime(configCode);
		if(null != latestModifyTime) {
			return DateUtils.getTimestampFormat(latestModifyTime);
		}
		return null;
	}
	
	
	/**
	 * @Description: 交换数据配置值的排序
	 * @author mazc
	 * @param sourceSysConfigValue
	 * @param targetSysConfigValue
	 */
	@GetMapping("/changeSysConfigValueSortNo")
	public boolean changeConfigValueSortNo(Integer sourceSortNo, String sourceId, Integer targetSortNo, String targetId){
		return sysConfigValueService.changeConfigValueSortNo(sourceSortNo, sourceId, targetSortNo, targetId);
	}
	
	
	/**
	 * @Description: 获得配置ID下的配置值集合
	 * @author mazc
	 * @return List<SysConfigValue>
	 */
	@GetMapping("/getConfigValueListByConfigId")
	public List<SysConfigValue> getConfigValueListByConfigId(String configId, String useStatus){
		return sysConfigValueService.getConfigValueListByConfigId(configId, useStatus);
	}


}