package com.gx.obe.server.management.dictionary.controller;

import java.util.List;
import java.util.Map;

import com.gx.obe.server.common.utils.IDUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;
import com.gx.obe.server.management.dictionary.service.BaseDictionaryValueService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

 /**  
 * 
 * @author 
 * @Title: BaseDictionaryValueController  
 * @Package com.gx.obe.server.management.dictionary.controller   
 * @Description:   数据字典值表
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-13   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Api(tags = "数据字典")
@RestController
@AllArgsConstructor
@RequestMapping("/baseDictionaryValue")
public class BaseDictionaryValueController {
	
	@Autowired
	private BaseDictionaryValueService baseDictionaryValueService;
	
	@GetMapping("/dictionarylist")
	public List<BaseDictionaryValue> getDictionaryValueList(String dictCode, String useStatus){
		return baseDictionaryValueService.getDictionaryValueList(dictCode,useStatus);
	}
	
	/**
	 * @Description: 根据字典编号获得数据字典值列表
	 * @author wangxiang
	 * @param dicCode
	 * @return 
	 */
	@GetMapping("/getDictValueListByDicCode")
	public List<BaseDictionaryValue> getDictValueListByDicCode(String dicCode){
		return baseDictionaryValueService.getDictValueListByDicCode(dicCode);
	}
	
	/**
	 * @Description: 根据字典编码，获得有效字典值集合。
	 * @author mazc
	 * @param dictCode
	 * @return 
	 */
	@GetMapping("/getDictionaryValueListBtDictCode")
	public List<BaseDictionaryValue> getDictionaryValueListBtDictCode(String dictCode){
		return baseDictionaryValueService.getDictionaryValueListByDictCode(dictCode);
	}
	
	/**
	 * @Description: 验证字典值名称唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param dictValueName
	 * @return 
	 */
	@GetMapping("/isUniqueDictValueName")
	public boolean isUniqueDictValueName(String dictId, String id, String dictValueName){
		return baseDictionaryValueService.isUniqueDictValueName(dictId, id, dictValueName);
	}
	
	/**
	 * @Description: 验证字典值编码唯一性
	 * @author mazc
	 * @param dictId
	 * @param id
	 * @param dictValueCode
	 * @return 
	 */
	@GetMapping("/isUniqueDictValueCode")
	public boolean isUniqueDictValueCode(String dictId, String id, String dictValueCode){
		return baseDictionaryValueService.isUniqueDictValueCode(dictId, id, dictValueCode);
	}
	
	/**
	 * @Description: 获得数据字典值的最大序号
	 * @author mazc
	 * @return
	 */
	@GetMapping("/getMaxDictValueSortNo")
	public int getMaxDictValueSortNo(){
		return baseDictionaryValueService.getMaxDictValueSortNo();
	}
	
	
	/**
	 * @Description: 交换数据字典值的排序
	 * @author mazc
	 * @param sourceDictValue
	 * @param targetDictValue
	 */
	@GetMapping("/changeDictValueSortNo")
	public boolean changeDictValueSortNo(Integer sourceSortNo, String sourceid, Integer targetSortNo, String targetId){
		return baseDictionaryValueService.changeDictValueSortNo(sourceSortNo, sourceid, targetSortNo, targetId);
	}
	
	
	/**
	 * @Description: 获得字典值列表
	 * @author mazc
	 * @param objIds
	 * @return
	 */
	@PostMapping("/getDictValueList")
	public List<BaseDictionaryValue> getDictValueList(List<String> objIds){
		return baseDictionaryValueService.getDictValueList(objIds);
	}
	
	
	/**
	 * @Description: 获得字典ID下的字典值集合
	 * @author mazc
	 * @return List<DictionaryValue>
	 */
	@Deprecated
	@GetMapping("/getDictionaryValueListByDictIdAnduseStatus")
	public List<BaseDictionaryValue> getDictionaryValueListByDictIdAnduseStatus(String dictId, String useStatus){
		return baseDictionaryValueService.getDictionaryValueListByDicId(dictId, useStatus);
	}
	/**
	 * @Description: 获得字典ID下的字典值集合
	 * @author mazc
	 * @return List<DictionaryValue>
	 */
	@GetMapping("/getDictionaryValueListByDicId")
	public List<BaseDictionaryValue> getDictionaryValueListByDicId(String dictId, String useStatus){
		return baseDictionaryValueService.getDictionaryValueListByDicId(dictId, useStatus);
	}

	/**
	 * @Description: 根据字典编号获得数据字典值
	 * @author mazc
	 * @param dicCode
	 * @return 
	 */
	@GetMapping("/getDictMapValueByDicCode")
	public List<Map<String, Object>> getDictMapValueByDicCode(String dicCode){
		return baseDictionaryValueService.getDictMapValueByDicCode(dicCode);
	}
	
	@GetMapping("/getDictValueBydictCodeAndValue")
	public BaseDictionaryValue getDictValueBydictCodeAndValue(String dictCode, String valueCode){
		return baseDictionaryValueService.getDictValue(dictCode,valueCode);
				
	}
	
	/**
	 * @Description: 获得特征码检查项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getFeatureCheckItemList")
	public List<BaseDictionaryValue> getFeatureCheckItemList(String tenderId){
		return baseDictionaryValueService.getFeatureCheckItemList(tenderId);
	}
	
	/**
	 * @Description: 获得清标检查项
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getCleanTenderItemList")
	public List<BaseDictionaryValue> getCleanTenderItemList(String tenderId){
		return baseDictionaryValueService.getCleanTenderItemList(tenderId);
	}
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param dictValueName
	 * @return 
	 */
	@GetMapping("/hasDictValueName")
	public boolean hasDictValueName(String dictValueName){
		return baseDictionaryValueService.hasDictValueName(dictValueName);
	}
	
	
	@GetMapping("/getDictValueByDictValueName")
	public BaseDictionaryValue getDictValueByDictValueName(String dictValueName){
		return baseDictionaryValueService.getDictValue(dictValueName);
	}
	
	@PostMapping("/batchDelete")
	public int batchDelete(@RequestBody List<BaseDictionaryValue> list) {
		return baseDictionaryValueService.batchDelete(list);
	}
	
	
	@PostMapping("/updateAssignProperty")
	public boolean updateAssignProperty(@RequestBody BaseDictionaryValue baseDictionaryValue) {
		return baseDictionaryValueService.updateAssignProperty(baseDictionaryValue);
		
	}
	
	@PostMapping("/saveOrUpdateReturn")
	public BaseDictionaryValue saveOrUpdateReturn(@RequestBody BaseDictionaryValue dictionaryValue){
		if(null == dictionaryValue) {
			return dictionaryValue;
		}
		if(StringUtils.isNotEmpty(dictionaryValue.getId())) {
			if(baseDictionaryValueService.updateById(dictionaryValue)) {
				return baseDictionaryValueService.getById(dictionaryValue.getId());				
			}else {
				return dictionaryValue;
			}
		}else {
			dictionaryValue.setId(IDUtils.getId());
			baseDictionaryValueService.save(dictionaryValue);
			return dictionaryValue;
		}
	}
}