package com.gx.obe.server.management.dictionary.service;

import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;
 
 /**  
 * 
 * @author 
 * @Title: BaseDictionaryValueService
 * @Package com.gx.obe.server.management.dictionary.service  
 * @Description:   数据字典值表 
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-05-13   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
public interface BaseDictionaryValueService extends IService<BaseDictionaryValue> {
 
	 /**   
	 * @Title: getDictionaryValueList   
	 * @Description: 查询数据字典
	 * @param: @param dictCode
	 * @param: @param useStatus
	 * @param: @return      
	 * @return: List<BaseDictionaryValue>      
	 * @throws   
	 */  
	List<BaseDictionaryValue> getDictionaryValueList(String dictCode, String useStatus);
	
	/**
	 * @Description: 根据字典编号获得数据字典值列表
	 * @author wangxiang
	 * @param dicCode
	 * @return 
	 */
	List<BaseDictionaryValue> getDictValueListByDicCode(String dicCode);

	 /**
	 * @Description:根据字典编码，获得有效字典值集合。 
	 * @author mazc
	 * @param dictCode
	 * @return 
	 */
	List<BaseDictionaryValue> getDictionaryValueListByDictCode(String dictCode);
	
	/**
	 * @Description: 验证字典值名称唯一性
	 * @author mazc
	 * @param id
	 * @param dictName
	 * @return boolean true：是唯一；false:不是唯一
	 */
	 boolean isUniqueDictValueName(String dictId, String id, String dictValueName);
	
	/**
	 * @Description: 验证字典值编码唯一性
	 * @author mazc
	 * @param id
	 * @param dictCode
	 * @return
	 */
	 boolean isUniqueDictValueCode(String dictId, String id, String dictValueCode);
	
	/**
	 * @Description: 获得数据字典值的最大序号
	 * @author mazc
	 * @return
	 */
	 int getMaxDictValueSortNo();
	
	
	/**
	 * @Description: 交换数据字典值的排序
	 * @author mazc
	 * @param sourceSortNo
	 * @param sourceid
	 * @param targetSortNo
	 * @param targetId
	 * @return 
	 */
	boolean changeDictValueSortNo(Integer sourceSortNo, String sourceid, Integer targetSortNo, String targetId);
	
	/**
	 * @Description: 获得字典值列表
	 * @author mazc
	 * @param objIds
	 * @return
	 */
	 List<BaseDictionaryValue> getDictValueList(List<String> objIds);
	

	/**
	 * @Description: 根据字典编号获得数据字典值
	 * @author mazc
	 * @param dicCode
	 * @return
	 */
	 List<Map<String, Object>> getDictMapValueByDicCode(String dicCode);
	
	/**
	 * @Description: 获得指定数据字典的值
	 * @author mazc
	 * @param dictCode
	 * @param valueCode
	 * @return 
	 */
	 BaseDictionaryValue getDictValue(String dictCode, String valueCode);
	
	/**
	 * @Description: 获得特征码检查项
	 * @author mazc
	 * @param tenderId
	 * @return
	 */
	 List<BaseDictionaryValue> getFeatureCheckItemList(String tenderId);
	
	/**
	 * @Description: 获得清标检查项
	 * @author mazc
	 * @param tenderId
	 * @return
	 */
	 List<BaseDictionaryValue> getCleanTenderItemList(String tenderId);
	
	/**
	 * @Description: 判断字典值是否存在
	 * @author mazc
	 * @param dictValueName
	 * @return 
	 */
	 boolean hasDictValueName(String dictValueName);
	
	/**
	 * @Description: 获得指定数据字典的值
	 * @author mazc
	 * @return 
	 */
	 BaseDictionaryValue getDictValue(String dictValueName);
	
	
	List<BaseDictionaryValue> getDictionaryValueListByDicId(String dictId, String useStatus);
	
	
	
	 int batchDelete(List<BaseDictionaryValue> list);
	 
	 
	 boolean updateAssignProperty(BaseDictionaryValue baseDictionaryValue);
}

