package com.gx.obe.server.management.dictionary.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.management.dictionary.dao.BaseDictionaryMapper;
import com.gx.obe.server.management.dictionary.dao.BaseDictionaryValueMapper;
import com.gx.obe.server.management.dictionary.entity.BaseDictionary;
import com.gx.obe.server.management.dictionary.entity.BaseDictionaryValue;
import com.gx.obe.server.management.dictionary.service.BaseDictionaryService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class BaseDictionaryServiceImpl extends ServiceImpl<BaseDictionaryMapper, BaseDictionary> implements BaseDictionaryService {

	@Autowired
	private BaseDictionaryMapper baseDictionaryMapper;
	
	@Autowired
	private BaseDictionaryValueMapper baseDictionaryValueMapper;
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param BaseDictionaryList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<BaseDictionary> BaseDictionaryList){
		return baseDictionaryMapper.updateBatchList(BaseDictionaryList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param BaseDictionaryList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<BaseDictionary> BaseDictionaryList){
		return baseDictionaryMapper.insertByBatch(BaseDictionaryList);
	}

	/** 
	 * @Description: 验证字典名称唯一性
	 * @author mazc
	 * @param id
	 * @param dictName
	 * @return 
	 */
	@Override
	public boolean isUniqueDictName(String id, String dictName) {
		return baseDictionaryMapper.isUniqueDictName(id, dictName) > 0 ? false : true;
	}

	@Override
	public boolean isUniqueDictCode(String id, String dictCode) {
 		return baseDictionaryMapper.isUniqueDictCode(id, dictCode) > 0 ? false : true;
	}

	@Override
	public BaseDictionary getDictionaryByCode(String dictCode) {
		return baseDictionaryMapper.getDictionaryByCode(dictCode);
	}

	@Override
	public BaseDictionary getDictionaryAndValuesByCode(String dictCode) {
		BaseDictionary dictionary = baseDictionaryMapper.getDictionaryByCode(dictCode);
		if(null != dictionary && null != dictionary.getId()){
			List<BaseDictionaryValue> dictValueList = baseDictionaryValueMapper.getDictionaryValuesByCode(dictionary.getId());
			dictionary.setDictValueList(dictValueList);
		}
		return dictionary;
	}

	@Override
	public List<BaseDictionaryValue> getDictionaryValueList(String dictCode) {
		return baseDictionaryValueMapper.getDictionaryValueList(dictCode);
	}

	@Override
	public List<BaseDictionary> getDictionaryListOrderByCreateTime() {
		QueryWrapper<BaseDictionary> query = new QueryWrapper<BaseDictionary>();
		query.orderByAsc(BaseDictionary.CREATE_TIME);
		return baseDictionaryMapper.selectList(query);
	}
	

}
