package com.gx.obe.server.management.dictionary.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.dao.MachineMapper;
import com.gx.obe.server.management.dictionary.entity.Machine;
import com.gx.obe.server.management.dictionary.service.MachineService;
import com.gx.obe.server.management.vo.Condition;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class MachineServiceImpl extends ServiceImpl<MachineMapper, Machine> implements MachineService {

	@Autowired
	private MachineMapper machineMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeMachineList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<Machine> ObeMachineList){
		return machineMapper.updateBatchList(ObeMachineList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeMachineList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<Machine> ObeMachineList){
		return machineMapper.insertByBatch(ObeMachineList);
	}

	/** 
	 * @Description: 获得评标机位集合
	 * @author mazc
	 * @param tenderId
	 * @param isUsed
	 * @param machineType
	 * @return 
	 */
	@Override
	public List<Machine> getProjectMachineList(String tenderId, String isUsed, String machineType) {
		return machineMapper.getProjectMachineList(tenderId, isUsed, machineType);
	}

	/** 
	 * @Description: 获得当前项目下已登录的机位
	 * @author mazc
	 * @param tenderId
	 * @param loginTime
	 * @return 
	 */
	@Override
	public List<Machine> getProjectLoginedMachineList(String tenderId, Date loginTime) {
		return machineMapper.getProjectLoginedMachineList(tenderId, loginTime);
	}

	/** 
	 * @Description: 获得指定评标室内的机位列表
	 * @author mazc
	 * @param roomeId
	 * @return 
	 */
	@Override
	public List<Machine> getEvaluationRoomMachineList(String roomeId) {
		return machineMapper.getEvaluationRoomMachineList(roomeId);
	}

	@Override
	public int saveMachineList(List<Machine> list) {
		int iCount = 0;
		List<Machine> insertMachineList = new ArrayList<Machine>();
		List<Machine> UpdateMachineList = new ArrayList<Machine>();
		for(Machine supplierEvaluationFactor : list) {
			if(null == supplierEvaluationFactor.getId()) {
				supplierEvaluationFactor.setId(IDUtils.getId());
				insertMachineList.add(supplierEvaluationFactor);
			}else {
				UpdateMachineList.add(supplierEvaluationFactor);
			}
		}
		iCount += machineMapper.updateBatchList(UpdateMachineList);
		iCount += machineMapper.insertByBatch(insertMachineList);
		return iCount;	
	}

	@Override
	public int deleteEvaluationRoomMachine(String roomId) {
		QueryWrapper<Machine> queryWrapper = new QueryWrapper<Machine>();
		queryWrapper.lambda().eq(Machine::getEvaluationRoomId, roomId);
		return machineMapper.delete(queryWrapper);
	}

	/** 
	 * @Description:  验证是否已经申请过该 机位
	 * @author mazc
	 * @param machineMac
	 * @param machineIp
	 * @param useStatus
	 * @return 
	 */
	@Override
	public List<Object> isExist(String machineMac, String machineIp, String useStatus) {
		QueryWrapper<Machine> queryWrapper = new QueryWrapper<Machine>();
		queryWrapper.select(Machine.MACHINE_ID).lambda().eq(Machine::getMachineMac, machineMac).eq(Machine::getMachineIp, machineIp)
		.eq(Machine::getUseStatus, useStatus);
		return machineMapper.selectObjs(queryWrapper);
	}

	@Override
	public Machine saveOrUpdateReturn(Machine machine) {
		int size = 0;
		if(machine == null) {
			return null;
		}
		if(null == machine.getId()) {
			machine.setId(IDUtils.getId());
			size = machineMapper.insert(machine);
			if(size > 0) {
				return machine;
			}
		}else {
			size =  machineMapper.updateById(machine);
			if(size > 0) {
				return machineMapper.selectById(machine.getId());
			}
			return machine;
		}
 		return null;
	}

	@Override
	public List<Object> getEvalRoomIdByMachineMacAndUseStatus(String machineMac, String useStatus) {
		QueryWrapper<Machine> queryWrapper = new QueryWrapper<Machine>();
		queryWrapper.select(Machine.EVALUATION_ROOM_ID).eq(Machine.MACHINE_MAC, machineMac).eq(Machine.USE_STATUS, useStatus);
		return machineMapper.selectObjs(queryWrapper);
	}

	@Override
	public List<Machine> getMachineByUseStatusAndOrderby(String useStatus) {
		QueryWrapper<Machine> queryWrapper = new QueryWrapper<Machine>();
		queryWrapper.lambda().eq(Machine::getUseStatus, useStatus).orderByAsc(Machine::getId);
		return machineMapper.selectList(queryWrapper);
	}

	@Override
	public Machine getMachineByMacOrNotEQ(String machineMac, String id) {
		QueryWrapper<Machine> queryWrapper = new QueryWrapper<Machine>();
		queryWrapper.lambda().eq(Machine::getMachineMac, machineMac);
		if(StringUtils.isNotEmpty(id)) {
			queryWrapper.lambda().ne(Machine::getId, id);
		}
		return machineMapper.selectOne(queryWrapper);
	}

	@Override
	public List<Object> getEvalRoomIdByMachineNameAndEvalRoomId(String machineName, String evalRoomId, String id) {
		QueryWrapper<Machine> queryWrapper = new QueryWrapper<Machine>();
		queryWrapper.lambda().eq(Machine::getMachineName, machineName).eq(Machine::getEvaluationRoomId, evalRoomId);
		if(StringUtils.isNotEmpty(id)) {
			queryWrapper.lambda().ne(Machine::getId, id);
		}
		return machineMapper.selectObjs(queryWrapper);
	}

	@Override
	public int updateCondition(Condition condition) {
		UpdateWrapper<Machine> update = new UpdateWrapper<Machine>();
		Map<String, Object> setParm = condition.getSetParm();
		for (Map.Entry<String, Object> entry : setParm.entrySet()) {
			update.set(entry.getKey(), entry.getValue());
		}
		Map<String, Object> setWhere = condition.getSetWhereParam();
		for (Map.Entry<String, Object> entry : setWhere.entrySet()) {
			update.eq(entry.getKey(), entry.getValue());
		}
		 
		return machineMapper.update(null, update);
 	}
	

}
