package com.gx.obe.server.management.dictionary.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.dictionary.dao.TorrentFileMapper;
import com.gx.obe.server.management.dictionary.entity.TorrentFile;
import com.gx.obe.server.management.dictionary.service.TorrentFileService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class TorrentFileServiceImpl extends ServiceImpl<TorrentFileMapper, TorrentFile> implements TorrentFileService {

	@Autowired
	private TorrentFileMapper torrentFileMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeTorrentFileList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<TorrentFile> ObeTorrentFileList){
		return torrentFileMapper.updateBatchList(ObeTorrentFileList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeTorrentFileList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<TorrentFile> ObeTorrentFileList){
		return torrentFileMapper.insertByBatch(ObeTorrentFileList);
	}

	/** 
	 * @Description: 通过主键删除
	 * @author mazc
	 * @param uTorrentFileId
	 * @return 
	 */
	@Override
	public int deleteTorrentFileById(List<String> uTorrentFileId) {
		return torrentFileMapper.deleteBatchIds(uTorrentFileId);
	}

	/** 
	 * @Description: 获取项目经理所拥有的项目的种子文件
	 * @author mazc
	 * @param userId
	 * @return 
	 */
	@Override
	public List<TorrentFile> getTorrentFileListByManager(String userId) {
		return torrentFileMapper.getTorrentFileListByManager(userId);
	}

	@Override
	public TorrentFile getTorrentFileByTenderId(String tenderId) {
		QueryWrapper<TorrentFile> query = new QueryWrapper<TorrentFile>();
		query.lambda().eq(TorrentFile::getTenderId, tenderId);
		return torrentFileMapper.selectOne(query);
	}

	@Override
	public List<TorrentFile> getTorrentFileAllOrderByFileName() {
		QueryWrapper<TorrentFile> query = new QueryWrapper<TorrentFile>();
		query.orderByDesc(TorrentFile.FILE_NAME);
		return torrentFileMapper.selectList(query);
	}

	@Override
	public int batchSaveOrUpdate(List<TorrentFile> torrentFileList) {
		int iCount = 0;
		List<TorrentFile> insertTorrentFile = new ArrayList<TorrentFile>();
		List<TorrentFile> UpdateTorrentFile= new ArrayList<TorrentFile>();
		for(TorrentFile torrentFile : torrentFileList) {
			if(null == torrentFile.getId()) {
				torrentFile.setId(IDUtils.getId());
				insertTorrentFile.add(torrentFile);
			}else {
				UpdateTorrentFile.add(torrentFile);
			}
		}
		iCount += torrentFileMapper.updateBatchList(UpdateTorrentFile);
		iCount += torrentFileMapper.insertByBatch(insertTorrentFile);
		return iCount;	
 	}
	

}
