package com.gx.obe.server.management.dictionary.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.enumeration.CommonEnum;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.dictionary.dao.TransactionServerMapper;
import com.gx.obe.server.management.dictionary.entity.TransactionServerEntity;
import com.gx.obe.server.management.dictionary.service.TransactionServerService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class TransactionServerServiceImpl extends ServiceImpl<TransactionServerMapper, TransactionServerEntity> implements TransactionServerService {

	@Autowired
	private TransactionServerMapper transactionServerMapper;

	@Override
	public boolean changeTransactionServerSortNo(String sourceSortNo, String sourceid, String targetSortNo,String targetId) {
		int count = 0;
		UpdateWrapper<TransactionServerEntity> sourceUpdateWrapper = new UpdateWrapper<>();
		sourceUpdateWrapper.set(TransactionServerEntity.SORT_NO,targetSortNo);
		sourceUpdateWrapper.lambda().eq(TransactionServerEntity::getId, sourceid);
		count += transactionServerMapper.update(null, sourceUpdateWrapper);
		
		UpdateWrapper<TransactionServerEntity> targetUpdateWrapper = new UpdateWrapper<>();
		targetUpdateWrapper.set(TransactionServerEntity.SORT_NO,sourceSortNo);
		targetUpdateWrapper.lambda().eq(TransactionServerEntity::getId, targetId);
		count +=  transactionServerMapper.update(null, targetUpdateWrapper); 
		
		return count == 2;
 	}

	/** 
	 * @Description: 获得执行交易系统的最大序号
	 * @author mazc
	 * @return 
	 */
	@Override
	public int getMaxTransactionServerSortNo() {
		Integer maxSortNo =  transactionServerMapper.getMaxTransactionServerSortNo();
		return null != maxSortNo ? maxSortNo.intValue() : 0;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public TransactionServerEntity getCurPlatformTransactionServer(String serName, String serverAddress) {
		TransactionServerEntity transactionServerEntity = null;
		transactionServerEntity = transactionServerMapper.getCurPlatformTransactionServer(serName, serverAddress);
		if(null == transactionServerEntity){
			transactionServerEntity = new TransactionServerEntity();
			transactionServerEntity.setServerName(serName);
			transactionServerEntity.setId(IDUtils.getId());
			transactionServerEntity.setServerAddress(serverAddress);
			transactionServerEntity.setUseStatus(CommonEnum.REAL_STATUS);
			transactionServerMapper.insert(transactionServerEntity);
		}
		
		return transactionServerEntity;
	}

	/**
	 * @Description: 获得指定状态的服务
	 * @author wangxiang
	 * @param realStatus
	 * @return 
	 */
	@Override
	public List<TransactionServerEntity> getStatusServerOrderBySortNo(String realStatus) {
		return transactionServerMapper.getStatusServer(realStatus);
	}

	@Override
	public TransactionServerEntity getTransactionServerByServerAddress(String serverAddress) {
		QueryWrapper<TransactionServerEntity> quwery = new QueryWrapper<TransactionServerEntity>();
		quwery.lambda().eq(TransactionServerEntity::getServerAddress, serverAddress);
		return transactionServerMapper.selectOne(quwery);
	}

	@Override
	public boolean updateAssignProperty(TransactionServerEntity transactionServerEntity) {
		return transactionServerMapper.updateById(transactionServerEntity) > 0;
	}

	@Override
	public List<TransactionServerEntity> getTransactionServerEntityAllOrderBySortNo() {
		QueryWrapper<TransactionServerEntity> quwery = new QueryWrapper<TransactionServerEntity>();
		quwery.orderByAsc(TransactionServerEntity.SORT_NO);
		return transactionServerMapper.selectList(quwery);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public TransactionServerEntity saveOrUpdateReturn(TransactionServerEntity transactionServerEntity) {
		if(StringUtils.isNotEmpty(transactionServerEntity.getId())) {
			transactionServerMapper.updateById(transactionServerEntity);
			return transactionServerMapper.selectById(transactionServerEntity.getId());
		}else {
			transactionServerEntity.setId(IDUtils.getId());
			  transactionServerMapper.insert(transactionServerEntity);
			  return transactionServerEntity;
		}		 
 	}

	@Override
	public List<Object> getIdList(String serverName, String id) {
		QueryWrapper<TransactionServerEntity> quwery = new QueryWrapper<TransactionServerEntity>();
		quwery.select("TRANSACTION_SERVER_ID")
		.lambda().eq(TransactionServerEntity::getServerName, serverName);
		if(StringUtils.isNotEmpty(id)) {
			quwery.lambda().ne(TransactionServerEntity::getId, id);
		}
		return transactionServerMapper.selectObjs(quwery);
	}
	
}
