package com.gx.obe.server.management.dictionary.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.management.dictionary.dao.WorkMemberMapper;
import com.gx.obe.server.management.dictionary.entity.WorkMember;
import com.gx.obe.server.management.dictionary.service.WorkMemberService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class WorkMemberServiceImpl extends ServiceImpl<WorkMemberMapper, WorkMember> implements WorkMemberService {

	@Autowired
	private WorkMemberMapper workMemberMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param WorkMemberList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<WorkMember> WorkMemberList){
		return workMemberMapper.updateBatchList(WorkMemberList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param WorkMemberList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<WorkMember> WorkMemberList){
		return workMemberMapper.insertByBatch(WorkMemberList);
	}

	/** 
	 * @Description: 返回指定类型的工作人员
	 * @author mazc
	 * @param tenderId
	 * @param workerType
	 * @return 
	 */
	@Override
	public List<WorkMember> getWorkerList(boolean isParentProject, String tenderId, String workerType) {
		return workMemberMapper.getWorkerList(isParentProject,tenderId, workerType);
	}

	/** 
	 * @Description: 返回指定项目的工作人员
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	@Override
	public WorkMember getWorker(String tenderId, String userId) {
		return workMemberMapper.getWorker(tenderId, userId);
	}

	/** 
	 * @Description: 删除工作人员
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param wokerType
	 * @return 
	 */
	@Override
	public boolean deleteWorker(String tenderId, String userId, String wokerType) {
		return workMemberMapper.deleteWorker(tenderId, userId, wokerType) > 0;
	}

	/** 
	 * @Description: 删除所有工作人员
	 * @author mazc
	 * @param tenderId
	 * @param wokerType
	 * @return 
	 */
	@Override
	public boolean delAllWorkers(String tenderId, String wokerType,String monitorType) {
		if(null != monitorType) {
			return workMemberMapper.delAllWorkersByMonitorType(tenderId, wokerType,monitorType) > 0;
		}else {
			return workMemberMapper.delAllWorkers(tenderId, wokerType) > 0;
		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public int batchSaveOrUpdate(List<WorkMember> workMemberList) {
		int iCount = 0;
		List<WorkMember> insertWorkMember = new ArrayList<WorkMember>();
		List<WorkMember> UpdateWorkMember = new ArrayList<WorkMember>();
		for(WorkMember workMember : workMemberList) {
			if(null == workMember.getId()) {
				workMember.setId(IDUtils.getId());
				insertWorkMember.add(workMember);
			}else {
				UpdateWorkMember.add(workMember);
			}
		}
		if(UpdateWorkMember.size() > 0) {
			iCount += workMemberMapper.updateBatchList(UpdateWorkMember);
		}
		if(insertWorkMember.size() > 0) {
			iCount += workMemberMapper.insertByBatch(insertWorkMember);
		}
		return iCount;	
 	}

	@Override
	public boolean updateAssignProperty(WorkMember workMember) {
		return workMemberMapper.updateById(workMember) > 0;
	}

	@Override
	public WorkMember getWorkerByUserIdAndTenderIdOrMemberType(String userId, String tenderId, String memberType) {
//		QueryWrapper<WorkMember> querWrapper = new QueryWrapper<WorkMember>();
//		querWrapper.lambda().eq(WorkMember::getUser, userId).eq(WorkMember::getTenderId, tenderId);
//		if(StringUtils.isNotEmpty(memberType)) {
//			querWrapper.lambda().eq(WorkMember::getMemberType, memberType);
//		}
		return workMemberMapper.getWorkerByUserIdAndTenderIdOrMemberType(userId,tenderId,memberType);
	}
	

}
