package com.gx.obe.server.management.evaluation.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.enumeration.BidPriceEnum;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.vo.Result;
import com.gx.obe.server.management.evaluation.entity.BidPrice;
import com.gx.obe.server.management.evaluation.service.BidPriceService;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "投标报价")
@RestController
@AllArgsConstructor
@RequestMapping("/bidPrice")
public class BidPriceController extends BaseController<BidPriceService,BidPrice>{
	
	@Autowired
	private BidPriceService bidPriceService;
	
	/**
	 * @Description: 获取当前项目的开标一览表
	 * @author guoyr
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getTenderBidPriceTableList")
	public List<BidPrice> getTenderBidPriceTableList(String tenderId){
		QueryWrapper<BidPrice> QueryWrapper = new QueryWrapper<BidPrice>();
		QueryWrapper.lambda().eq(BidPrice::getTenderId, tenderId);
		QueryWrapper.orderByAsc("SORT_NO");
		return bidPriceService.list(QueryWrapper);
	}
	
	/**
	 * @Description: 获取当前项目的开标一览表报价项
	 * @author guoyr
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getTenderBidPriceList")
	public List<BidPrice> getTenderBidPriceList(String tenderId){
		QueryWrapper<BidPrice> QueryWrapper = new QueryWrapper<BidPrice>();
		QueryWrapper.lambda().eq(BidPrice::getTenderId, tenderId);
		QueryWrapper.lambda().eq(BidPrice::getBidPriceType, BidPriceEnum.IS_PRICE_ITEM);
		QueryWrapper.orderByAsc("SORT_NO");
		return bidPriceService.list(QueryWrapper);
	}
	
	@GetMapping("/findTenderIdList")
	public List<BidPrice> findTenderIdList(String tenderId){
		QueryWrapper<BidPrice> QueryWrapper = new QueryWrapper<BidPrice>();
		QueryWrapper.lambda().eq(BidPrice::getTenderId, tenderId);
		return bidPriceService.list(QueryWrapper);
	}
	
	@GetMapping("/getBidPriceFactorList")
	public List<BidPrice> getBidPriceFactorList(String tenderId){
		return bidPriceService.getBidPriceFactorList(tenderId);
	}
	
	/**
	 * @Description: 批量保存已解析项目的开标一览表
	 * @author guoyr
	 * @param bidPriceList
	 * @return 
	 */
	@PostMapping("/batchSaveResolverBidPriceTable")
	@SysLogAnnotation(detail="批量保存开标一览表项",level = 0,operationType = OperationType.INSERT)
	public boolean batchSaveResolverBidPriceTable(@RequestBody List<BidPrice> bidPriceList) {
		return bidPriceService.batchSaveResolverBidPriceTable(bidPriceList);
	}
	
	/**
	 * @Description: 删除分项报价
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@GetMapping("/deleteBidPrice")
	@SysLogAnnotation(detail="删除开标一览表项",level = 0,operationType = OperationType.DELETE)
	public boolean deleteBidPrice(String tenderId, String bidPriceId ,String joinTotal){
		return bidPriceService.deleteBidPrice(tenderId, bidPriceId, joinTotal);
	}
	
	@GetMapping("/deleteProjectBidPrice")
	@SysLogAnnotation(detail="删除开标一览表",level = 0,operationType = OperationType.DELETE)
	public boolean deleteProjectBidPrice(String tenderId){
		return bidPriceService.deleteProjectBidPrice(tenderId);
	}
	
	
	/**
	 * @Description:交换两个分项报价的排序
	 * @author mazc
	 * @param sourceSortNo
	 * @param sourceid
	 * @param targetSortNo
	 * @param targetId
	 * @return 
	 */
	@GetMapping("/changeBidPriceSortNo")
	public boolean changeBidPriceSortNo(String sourceSortNo, String sourceid, String targetSortNo, String targetId){
		return bidPriceService.changeBidPriceSortNo(sourceSortNo, sourceid, targetSortNo, targetId);
	}
	
	/**
	 * @Description: 获得分项报价计入总价的其中一个
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getBidPriceByJoinTotal")
	public BidPrice getBidPriceByJoinTotal(String tenderId){
		return bidPriceService.getBidPriceByJoinTotal(tenderId);
	}
	
	/** 
	 * @Description: 获取项目中开标一览表的数量
	 * @author guoyr
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getBidPriceTableCount")
	public int getBidPriceTableCount(String tenderId) {
		return bidPriceService.getBidPriceTableCount(tenderId);
	}
	
	@PostMapping("/saveOrUpdate")
	public boolean saveOrUpdate(@RequestBody BidPrice bidPrice){
		return bidPriceService.saveOrUpdate(bidPrice);
	}

	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param updateAssignProperty
	 */
	@PostMapping("/updateAssignProperty")
	public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<BidPrice> updateAssignProperty) {
		BidPrice bidPrice = updateAssignProperty.getEntity();
		String[] attributes = updateAssignProperty.getAttributes();
		return bidPriceService.updateAssignProperty(bidPrice, attributes);
	}
	/**
	 * @Description: 修改报价项
	 * @author chenxw
	 * @param updateAssignProperty
	 */
	@PostMapping("/updateBidPrice")
	public boolean updateBidPrice(@RequestBody UpdateAssignPropertyVo<BidPrice> updateAssignProperty) {
		BidPrice bidPrice = updateAssignProperty.getEntity();
		String[] attributes = updateAssignProperty.getAttributes();
		return bidPriceService.updateBidPrice(bidPrice, attributes);
	}

	/**
	 * @Description: 保存报价项
	 * @author chenxw
	 * @param bidPrice
	 */
	@PostMapping("/saveBidPrice")
	@SysLogAnnotation(detail="新增开标一览表项",level = 0,operationType = OperationType.INSERT)
	public Result<Integer> saveBidPrice(@RequestBody BidPrice bidPrice) {
		return bidPriceService.saveBidPrice(bidPrice);
	}

	
	/** 
	 * @Description: 获取项目中开标一览表的数量
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getMaxBidPriceSortNo")
	public int getMaxBidPriceSortNo(String tenderId) {
		return bidPriceService.getMaxBidPriceSortNo(tenderId);
	}
	
}