package com.gx.obe.server.management.evaluation.controller;

import java.math.BigDecimal;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.enumeration.EvaluationStepEnum;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactor;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.evaluation.entity.EvaluationStepFactor;
import com.gx.obe.server.management.evaluation.service.EvaluationStepService;
import com.gx.obe.server.management.project.entity.TenderProject;
import com.gx.obe.server.management.supplier.entity.Supplier;
import com.gx.obe.server.management.vo.BatchUpdatePropertyVo;
import com.gx.obe.server.management.vo.CreateEvaluationStepsVo;
import com.gx.obe.server.management.vo.SaveEvaluationStepAndFactorsVo;
import com.gx.obe.server.management.vo.SaveEvaluationStepVo;
import com.gx.obe.server.management.vo.SaveWasteBidStepVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;

/**
 * @Description: 评标步骤管理
 * @author mazc
 */
@Api(tags = "评标步骤管理")
@RestController
@AllArgsConstructor
@RequestMapping("/evaluationStep")
public class EvaluationStepController extends BaseController<EvaluationStepService, EvaluationStep> {
    
    @Autowired
    EvaluationStepService evaluationStepService;
    
    @GetMapping("/stepByTenderIdCount")
    public List<Object> getIdByTenderIdCount(String tenderId) {
        QueryWrapper<EvaluationStep> wrapper = new QueryWrapper<EvaluationStep>();
        wrapper.select("STEP_ID").lambda().eq(EvaluationStep::getTenderId, tenderId);
        return evaluationStepService.listObjs(wrapper);
    }
    
    /**
     * @Description: 通过指标获得该指标所在的评标步骤
     * @author mazc
     * @param tenderId
     * @param factorId
     * @return
     */
    @GetMapping("/findByTenderIdAndFactorId")
    public EvaluationStep getEvaluationStepByFactor(String tenderId, String factorId) {
        return evaluationStepService.selectEvaluationStepByFactor(tenderId, factorId);
    }
    
    /**
     * @Description: 查询指定评审类型的评标步骤列表
     * @author mazc
     * @param tenderId
     * @param stepType
     * @return
     */
    @GetMapping("/findEvaluationStepListByType")
    public List<EvaluationStep> getEvaluationStepListByType(String tenderId, String stepType) {
        return evaluationStepService.getEvaluationStepListByType(tenderId, stepType);
    }
    
    /**
     * @Description: 查询评标步骤列表
     * @author mazc
     * @param tenderId
     * @return
     */
    @GetMapping("/findEvaluationStepList")
    public List<EvaluationStep> findEvaluationStepList(String tenderId) {
        return evaluationStepService.getEvaluationStepList(tenderId);
    }
    
    /**
     * @Description: 获得评审要素下可评审的评标步骤
     * @author mazc
     * @param tenderId
     * @param evalGroup
     * @return
     */
    @GetMapping("/findEvalGroupEvalStepList")
    public List<String> findEvalGroupEvalStepList(String tenderId, String evalGroup) {
        return evaluationStepService.getEvalGroupEvalStepList(tenderId, evalGroup);
    }
    
    /**
     * @Description: 获得指定评标步骤以前未评审的节点
     * @author mazc
     * @param tenderId
     * @param sortNo
     * @return
     */
    @GetMapping("/findPreNoStarEvaluationStep")
    public EvaluationStep findPreNoStarEvaluationStep(String tenderId, int sortNo) {
        return evaluationStepService.getPreNoStarEvaluationStep(tenderId, sortNo);
    }
    
    
    /**
     * @Description: 获得指定序号的评标步骤
     * @author guoyr
     * @param tenderId
     * @param sortNo
     * @return
     */
    @GetMapping("/getEvaluationStepBySortNo")
    public EvaluationStep getEvaluationStepBySortNo(String tenderId, int sortNo) {
        return evaluationStepService.getEvaluationStepBySortNo(tenderId, sortNo);
    }
    
    /**
     * @Description: 获得评标步骤的最大序号
     * @author guoyr
     * @param tenderId
     * @return
     */
    @GetMapping("/getMaxEvaluationStepSortNo")
    public int getMaxEvaluationStepSortNo(String tenderId) {
        return evaluationStepService.getMaxEvaluationStepSortNo(tenderId);
    }
    
    /**
     * @Description: 交换评审结点的顺序
     * @author guoyr
     * @param source
     * @param target
     */
    @GetMapping("/changeEvaluationStepSortNo")
    @SysLogAnnotation(detail="交换两个步骤的顺序",level = 0,operationType = OperationType.DELETE)
    public boolean changeEvaluationStepSortNo(String source, String target) {
        return evaluationStepService.changeEvaluationStepSortNo(source, target);
    }
    
    /**
     * @Description: 删除评标步骤
     * @author guoyr
     * @param EvaluationStep
     * @return
     */
    @PostMapping("/deleteEvaluationStep")
    @SysLogAnnotation(detail="删除评标步骤",level = 0,operationType = OperationType.DELETE)
    public boolean deleteEvaluationStep(@RequestBody EvaluationStep EvaluationStep) {
        return evaluationStepService.deleteEvaluationStep(EvaluationStep);
    }
    
    /**
     * @Description: 保存签到评标步骤
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    @PostMapping("/saveSignEvaluationStep")
    @SysLogAnnotation(detail="保存签到评标步骤",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveSignEvaluationStep(@RequestBody EvaluationStep evaluationStep) {
        return evaluationStepService.saveSignEvaluationStep(evaluationStep);
    }
    
    /**
     * @Description: 保存特征码检查评标步骤
     * @author guoyr
     * @param evaluationStep
     * @param stepExpertUserList
     * @param featureItemObjIdList
     * @return
     */
    @PostMapping("/saveFeatureEvaluationStep")
    @SysLogAnnotation(detail="保存特征码检查评标步骤",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveFeatureEvaluationStep(@RequestBody SaveEvaluationStepVo featureEvaluationStepVo) {
        return evaluationStepService.saveFeatureEvaluationStep(featureEvaluationStepVo.getEvaluationStep(), featureEvaluationStepVo.getStepExpertUserList(), featureEvaluationStepVo.getItemObjIdList());
    }
    
    /**
     * @Description: 保存清标检查评标步骤
     * @author guoyr
     * @param evaluationStep
     * @param stepExpertUserList
     * @param cleanTenderItemObjIdList
     * @return
     */
    @PostMapping("/saveCleanTenderEvaluationStep")
    @SysLogAnnotation(detail="保存清标检查评标步骤",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveCleanTenderEvaluationStep(@RequestBody SaveEvaluationStepVo featureEvaluationStepVo) {
        return evaluationStepService.saveCleanTenderEvaluationStep(featureEvaluationStepVo.getEvaluationStep(), featureEvaluationStepVo.getStepExpertUserList(), featureEvaluationStepVo.getItemObjIdList());
    }
    
    /**
     * @Description: 保存评标步骤和评审指标
     * @author guoyr
     * @param evaluationStep
     * @param evaluationFactorList
     * @return
     */
    @PostMapping("/saveEvaluationStepAndFactors")
    @SysLogAnnotation(detail="保存评标步骤和评审指标",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveEvaluationStepAndFactors(@RequestBody SaveEvaluationStepAndFactorsVo saveEvaluationStepAndFactorsVo) {
        EvaluationStep evaluationStep = saveEvaluationStepAndFactorsVo.getEvaluationStep();
        List<EvaluationFactor> evaluationFactorList = saveEvaluationStepAndFactorsVo.getEvaluationFactorList();
        List<String> factorWeightList = saveEvaluationStepAndFactorsVo.getFactorWeightList();
        List<String> stepExpertUserList = saveEvaluationStepAndFactorsVo.getStepExpertUserList();
        return evaluationStepService.saveEvaluationStepAndFactors(evaluationStep, evaluationFactorList, factorWeightList, stepExpertUserList);
    }
    
    @PostMapping("/saveTianYanEvaluationStep")
    @SysLogAnnotation(detail="保存评标步骤和评审指标",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveTianYanEvaluationStep(@RequestBody SaveEvaluationStepAndFactorsVo saveEvaluationStepAndFactorsVo) {
        EvaluationStep evaluationStep = saveEvaluationStepAndFactorsVo.getEvaluationStep();
        List<String> stepExpertUserList = saveEvaluationStepAndFactorsVo.getStepExpertUserList();
        return evaluationStepService.saveTianYanEvaluationStep(evaluationStep, stepExpertUserList);
    }
    
    /**
     * @Description: 保存评标步骤
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    @PostMapping("/saveEvaluationStep")
    @SysLogAnnotation(detail="保存评标步骤",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveEvaluationStep(@RequestBody EvaluationStep evaluationStep) {
        return evaluationStepService.saveEvaluationStep(evaluationStep);
    }
    
    /**
     * @Description: 创建废标步骤
     * @author guoyr
     * @param tenderProjectEntity
     * @param evaluationStep
     */
    @PostMapping("/saveWasteBidStep")
    @SysLogAnnotation(detail="创建废标步骤",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveWasteBidStep(@RequestBody SaveWasteBidStepVo saveWasteBidStepVo) {
        TenderProject tenderProjectEntity = saveWasteBidStepVo.getTenderProject();
        EvaluationStep evaluationStep = saveWasteBidStepVo.getEvaluationStep();
        return evaluationStepService.saveWasteBidStep(tenderProjectEntity, evaluationStep);
    }
    
    /**
     * @Description: 验证评标步骤的名称是否已经存在
     * @author guoyr
     * @param tenderId
     * @param id
     * @param stepName
     * @return
     */
    @GetMapping("/valEvaluationStepNameExist")
    public boolean valEvaluationStepNameExist(String tenderId, String id, String stepName) {
        return evaluationStepService.valEvaluationStepNameExist(tenderId, id, stepName);
    }
    
    /**
     * @Description: 设置默认评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    @GetMapping("/createDefaultEvaluationSteps")
    @SysLogAnnotation(detail="设置默认评标步骤",level = 0,operationType = OperationType.UPDATE)
    public boolean createDefaultEvaluationSteps(String tenderId, String siginStepName, String complianceStepName, String detailStepName, String priceStepName, String recommonedStepName, String SHOW_SUMMARY_COMPLIANCE, String SHOW_SUMMARY_SCORE) {
        return evaluationStepService.createDefaultEvaluationSteps(tenderId, siginStepName, complianceStepName, detailStepName, priceStepName, recommonedStepName, SHOW_SUMMARY_COMPLIANCE, SHOW_SUMMARY_SCORE);
    }
    
    /**
     * @Description: 设置默认评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    @PostMapping("/createEvaluationSteps")
    @SysLogAnnotation(detail="设置默认评标步骤",level = 0,operationType = OperationType.UPDATE)
    public boolean createEvaluationSteps(@RequestBody CreateEvaluationStepsVo createEvaluationStepsVo) {
        String tenderId = createEvaluationStepsVo.getTenderId();
        List<EvaluationStep> nodeList = createEvaluationStepsVo.getNodeList();
        List<EvaluationStepFactor> evaluationStepFactorList = createEvaluationStepsVo.getEvaluationStepFactorList();
        return evaluationStepService.createDefaultEvaluationSteps(tenderId, nodeList, evaluationStepFactorList);
    }
    
    /**
     * @Description: 清空项目的评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    @GetMapping("/deleteAllEvaluationSteps")
    @SysLogAnnotation(detail="清空项目的评标步骤",level = 0,operationType = OperationType.DELETE)
    public boolean deleteAllEvaluationSteps(String tenderId) {
        return evaluationStepService.deleteAllEvaluationSteps(tenderId);
    }
    
    /**
     * @Description: 重新授权评标步骤，并重新授权推荐
     * @author guoyr
     * @param tenderId
     * @param nodeSortNo
     * @param isFilter 是否筛选节点
     */
    @PostMapping("/reBackEvaluationStep")
	@SysLogAnnotation(detail="重新授权评标步骤",level = 0,operationType = OperationType.UPDATE)
    public void reBackEvaluationStep(@RequestBody RequestBodyEntity<EvaluationStep> bodyEntity) {
        evaluationStepService.reBackEvaluationStep(bodyEntity.getEntity(), (String) bodyEntity.getParam("userId"));
    }
    
    /**
     * @Description: 获得最后一个评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    @GetMapping("/getLastEvaluationStep")
    public EvaluationStep getLastEvaluationStep(String tenderId) {
        return evaluationStepService.getLastEvaluationStep(tenderId);
    }
    
    /**
     * @Description: 获得存在评审不通过投标人的评标步骤
     * @author guoyr
     * @param tenderId
     * @return
     */
    @GetMapping("/getDisAccessmentNode")
    public List<EvaluationStep> getDisAccessmentNode(String tenderId) {
        return evaluationStepService.getDisAccessmentNode(tenderId);
    }
    
    /**
     * @Description: 验证评标步骤是否正在评标步骤中被退回
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    @PostMapping("/isEvaluationStepRebackedInProgress")
    public boolean isEvaluationStepRebackedInProgress(@RequestBody EvaluationStep evaluationStep) {
        return evaluationStepService.isEvaluationStepRebackedInProgress(evaluationStep);
    }
    
    /**
     * @Description: 获得评标步骤的权重
     * @author guoyr
     * @param evaluationStep
     * @return
     */
    @PostMapping("/getEvaluationStepWeight")
    public BigDecimal getEvaluationStepWeight(@RequestBody EvaluationStep evaluationStep) {
        return evaluationStepService.getEvaluationStepWeight(evaluationStep);
    }
    
    /**
     * @Description: 获得评标步骤的分值
     * @author guoyr
     * @param evaluationStep
     * @param weightScore 权重分值
     * @return
     */
    @PostMapping("/getEvaluationStepScore")
    public BigDecimal getEvaluationStepScore(@RequestBody RequestBodyEntity<EvaluationStep> bodyEntity) {
        boolean weightScore = (boolean) bodyEntity.getParam("weightScore");
        return evaluationStepService.getEvaluationStepScore(bodyEntity.getEntity(), weightScore);
    }
    
    /**
     * @Description: 根据项目id,专家用户id(userId)查找当前专家所拥有的评标步骤
     * @author guoyr
     * @param tendertId
     * @param userId
     * @return
     */
    @GetMapping("/getEvaluationStepByUserId")
    public List<EvaluationStep> getEvaluationStepByUserId(String tenderId, String userId) {
        return evaluationStepService.getEvaluationStepByUserId(tenderId, userId);
    }
    
    /**
     * @Description: 根据项目评审点id查询当前评审点所属评标步骤
     * @author guoyr
     * @param tendertId
     * @param factorId
     * @return
     */
    @GetMapping("/getEvaluationStepByFactorId")
    public EvaluationStep getEvaluationStepByFactorId(String tenderId, String factorId) {
        return evaluationStepService.getEvaluationStepByFactorId(tenderId, factorId);
    }
    
    /**
     * @Description: 获得已完成步骤最大编号
     * @author wangxiang
     * @param tenderId
     * @param stepStatus
     * @return
     */
    @GetMapping("/getMAXStepCount")
    public int getMAXStepCount(String tenderId, String stepStatus) {
        return evaluationStepService.getMAXStepCount(tenderId, stepStatus);
    }
    
    /**
     * @Description: 更新添加步骤后后面步骤+1
     * @author wangxiang
     * @param tenderId
     * @param maxComplateIndex
     */
    @GetMapping("/updateSortNo")
    public void updateSortNo(String tenderId, int maxComplateIndex) {
        evaluationStepService.updateSortNo(tenderId, maxComplateIndex);
    }
    
    /**
     * @Description:获取未完成的评审步骤列表
     * @author chenxw
     * @param tenderId
     * @param stepSortNo
     * @return
     */
    @GetMapping("/getNotComplateStepList")
    public List<EvaluationStep> getNotComplateStepList(String tenderId, int stepSortNo) {
        return evaluationStepService.getNotComplateStepList(tenderId, stepSortNo);
    }
    
    /**
     * @Description: 获取存在废标的未完成评审步骤列表
     * @author chenxw
     * @param tenderId
     * @param stepSortNo
     * @return
     */
    @GetMapping("/getNotComplateFilterStepList")
    public List<EvaluationStep> getNotComplateFilterStepList(String tenderId, int stepSortNo) {
        return evaluationStepService.getNotComplateFilterStepList(tenderId, stepSortNo);
    }
    
    /**
     * @Description: 当前步骤是否在评审中
     * @author guoyr
     * @param tenderId
     * @param stepId
     * @return
     */
    @GetMapping("/isInEvaluationStep")
    public boolean isInEvaluationStep(String tenderId, String stepId) {
        return evaluationStepService.isInEvaluationStep(tenderId, stepId);
    }
    
    /**
     * @Description: 删除专家组长评标结果
     * @author chenxw
     * @param tenderId
     */
    @GetMapping("/deleteExpertLeaderEvaluationStepResult")
    @SysLogAnnotation(detail="删除专家组长评标结果",level = 0,operationType = OperationType.DELETE)
    public void deleteExpertLeaderEvaluationStepResult(String tenderId) {
        evaluationStepService.deleteExpertLeaderEvaluationStepResult(tenderId);
    }
    
    /**
     * @Description:是否完成商务技术评审
     * @author guoyr
     * @param tenderId
     * @return
     */
    @GetMapping("/isComplateBusAndTecEvaluation")
    public boolean isComplateBusAndTecEvaluation(String tenderId) {
        return evaluationStepService.isComplateBusAndTecEvaluation(tenderId);
    }
    
    /**
     * @Description: 保存评标步骤
     * @author wangx
     * @param tenderId
     * @param evaluationStep
     * @param supplierList
     * @return
     */
    @GetMapping("/saveReEvaluationStep")
    @SysLogAnnotation(detail="保存评标步骤",level = 0,operationType = OperationType.UPDATE)
    public EvaluationStep saveReEvaluationStep(TenderProject tender, EvaluationStep evaluationStep, List<Supplier> supplierList) {
        return evaluationStepService.saveReEvaluationStep(tender, evaluationStep, supplierList);
    }
    
    /**
     * @Description: 获得价格评审和推荐的步骤完成步骤
     * @author wangxiang
     * @param tenderId
     * @return
     */
    @GetMapping("/getPriceOrRecommEvaluationStepList")
    public List<EvaluationStep> getPriceOrRecommEvaluationStepList(String tenderId) {
        QueryWrapper<EvaluationStep> wrapper = new QueryWrapper<EvaluationStep>();
        wrapper.lambda().eq(EvaluationStep::getTenderId, tenderId).in(EvaluationStep::getStepType, new Object[] { EvaluationStepEnum.PRICE + "," + EvaluationStepEnum.RECOMMONED });
        return evaluationStepService.list(wrapper);
    }
    
    /**
     * @Description: 是否含有评标步骤
     * @author chenxw
     * @param tenderId
     * @return 
     */
    @GetMapping("/isHasEvaluationStep")
    public boolean isHasEvaluationStep(String tenderId) {
        return evaluationStepService.isHasEvaluationStep(tenderId);
    }
    
    /**
     * @Description: 根据id更新指定字段
     * @author chenxw
     * @return 
     */
    @PostMapping("/updateAssignProperty")
    @SysLogAnnotation(detail="指定字段修改评标步骤信息",level = 0,operationType = OperationType.UPDATE)
    public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<EvaluationStep> updateAssignPropertyVo) {
        EvaluationStep evaluationStep = updateAssignPropertyVo.getEntity();
        String[] attributes = updateAssignPropertyVo.getAttributes();
        return evaluationStepService.updateAssignProperty(evaluationStep, attributes);
    }
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param evaluationStepList
     * @param attributes
     * @return 
     */
    @PostMapping("/batchUpdateProperty")
    @SysLogAnnotation(detail="批量指定字段修改评标步骤信息",level = 0,operationType = OperationType.UPDATE)
    public int batchUpdateProperty(@RequestBody BatchUpdatePropertyVo<EvaluationStep> batchUpdatePropertyVo) {
        List<EvaluationStep> evaluationStepList = batchUpdatePropertyVo.getEntityList();
        String[] attributes = batchUpdatePropertyVo.getAttributes();
        return evaluationStepService.batchUpdateProperty(evaluationStepList, attributes);
    }
    
}