package com.gx.obe.server.management.evaluation.service;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import com.baomidou.mybatisplus.extension.service.IService;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactorResult;
 
/** 
 * @Description: 
 * @author mazc 
 */
public interface EvaluationFactorResultService extends IService<EvaluationFactorResult> {
    
 	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeEvaluationFactorResultList 
	 */
 	Integer updateByBatch (List<EvaluationFactorResult> ObeEvaluationFactorResultList);
 	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeEvaluationFactorResultList 
	 */
	Integer insertByBatch (List<EvaluationFactorResult> ObeEvaluationFactorResultList);
	
	/**
	 * @Description:  删除所有专家为投标人指标的的评标结果
	 * @author mazc
	 * @param tenderId 项目id
	 * @return 
	 */
	public int deleteStepByTenderId(String tenderId);
	
	/**
	 * @Description: 获得评标专家的评标结果列表
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @param supplierId
	 * @return 
	 */
	public List<EvaluationFactorResult> getExpertEvaluationFactorResult(String tenderId, String stepId, String userId, String supplierId);
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param isScoreFactor
	 * @return 
	 */
	public List<Map<String, Object>> getSupplierDistinctFactorResultList(String tenderId, String stepId,  boolean isScoreFactor);
	
    /**
     * @Description: 保存按供应商打分结果
     * @author guoyr
     * @param supplierEvaluationFactorResultList 
     */
    public void saveEvaluationResultBySupplier(String projectId, String groupCode, String userId, List<Map<String, Object>> supplierEvaluationFactorResultList);
    
    /**
     * @Description: 获得指标的平均分
     * @author guoyr
     * @param tenderId
     * @param stepId
     * @param userId
     * @return
     */
    public List<Map<String, Object>> getEvaluationFactorAvgScoreList(String tenderId, String stepId, String supplierId, boolean removeMaxMin);

    /**
     * @Description: 根据指标Id获取当前专家对应的指标评审结果
     * @author chenxw
     * @param factorId
     * @return 
     */
    public List<EvaluationFactorResult> getEvaluationFactorResultList(String tenderId, String stepId, String factorId, String userId);

    /**
     * @Description: 获得评标专家的评标结果列表
     * @author guoyr
     * @param factorType
     */
    public List<EvaluationFactorResult> getResultListByFactorType(String tenderId, String factorType);
    
    /**
     * @Description: 获取指定指标不同评审结果的个数
     * @author guoyr
     * @param tenderId
     * @param stepId
     * @param factorId
     * @param userId
     * @param result
     * @param score
     * @return 
     */
    public int getSupplierDistinctFactorCount(String tenderId, String stepId, String factorId, String userId, String result, BigDecimal score);
    
    /**
     * @Description: 获得按供就商打分结果列表
     * @author guoyr
     * @param projectId
     * @param supplierName
     * @param groupCode
     * @return 
     */
    public List<Map<String, Object>> getSupplierEvaluationFactorResult(String projectId, String groupCode, String userId, String supplierName);

    /**
     * @Description: 获取得分列表
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @param supplierId
     * @param factorId
     * @return 
     */
    public List<Object> getEvaluationScoreList(String tenderId, String stepId, String supplierId, String factorId);
    
}





