package com.gx.obe.server.management.evaluation.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.entity.ReflecUtils;
import com.gx.obe.server.common.enumeration.BidPriceEnum;
import com.gx.obe.server.common.enumeration.EvaluationFactorEnum;
import com.gx.obe.server.common.enumeration.EvaluationStepEnum;
import com.gx.obe.server.common.type.EvaluationFactorType;
import com.gx.obe.server.common.utils.BigDecimalUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.ObjectUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.evaluation.dao.BidPriceMapper;
import com.gx.obe.server.management.evaluation.dao.EvaluationFactorMapper;
import com.gx.obe.server.management.evaluation.dao.EvaluationStepFactorMapper;
import com.gx.obe.server.management.evaluation.entity.BidPrice;
import com.gx.obe.server.management.evaluation.entity.EvaluationFactor;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.evaluation.entity.EvaluationStepFactor;
import com.gx.obe.server.management.evaluation.service.EvaluationFactorService;
import com.gx.obe.server.management.project.dao.TenderProjectMapper;
import com.gx.obe.server.management.project.entity.TenderProject;
import com.gx.obe.server.management.struct_new.dao.ObeEvaluationContentMapper;
import com.gx.obe.server.management.struct_new.entity.ObeEvaluationContent;

/**
 * 
 * @author
 * @Title: EvaluationFactorService
 * @Package com.gx.obe.server.management.evaluation.service
 * @Description:
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date: 2019-05-17
 * @version V1.0
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved.
 * 
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class EvaluationFactorServiceImpl extends ServiceImpl<EvaluationFactorMapper, EvaluationFactor> implements EvaluationFactorService {
    
    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(EvaluationFactorServiceImpl.class);
    
    @Autowired
    private EvaluationFactorMapper evaluationFactorMapper;
    
    @Autowired
    private EvaluationStepFactorMapper evaluationStepFactorMapper;
    
    @Autowired
    private BidPriceMapper bidPriceMapper;

    @Autowired
    private TenderProjectMapper tenderProjectMapper;
    
    @Autowired
    private ObeEvaluationContentMapper evaluationContentMapper;
    
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void delAllEvaluationFactor(String tenderId) {
        QueryWrapper<EvaluationFactor> evaluationFactorWrapper = new QueryWrapper<EvaluationFactor>();
        evaluationFactorWrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId);
        evaluationFactorMapper.delete(evaluationFactorWrapper);
        
        QueryWrapper<EvaluationStepFactor> evaluationStepFactorWrapper = new QueryWrapper<EvaluationStepFactor>();
        evaluationStepFactorWrapper.lambda().eq(EvaluationStepFactor::getTenderId, tenderId);
        evaluationStepFactorMapper.delete(evaluationStepFactorWrapper);
        
        QueryWrapper<ObeEvaluationContent> evaluationContentWrapper = new QueryWrapper<>();
        evaluationContentWrapper.lambda().eq(ObeEvaluationContent::getTenderId, tenderId);
        evaluationContentMapper.delete(evaluationContentWrapper);
    }
    
    @Override
    @Transactional(rollbackFor = Exception.class)
    public EvaluationFactor saveOrUpdateComplianceEvaluationFactor(EvaluationFactor evaluationFactor) {
        EvaluationFactor parentFactor = null;
        if (StringUtils.isNotEmpty(evaluationFactor.getParentId())) {
            parentFactor = evaluationFactorMapper.selectById(evaluationFactor.getParentId());
        }
        // 修改
        if (StringUtils.isNotEmpty(evaluationFactor.getId())) {
            evaluationFactorMapper.updateById(evaluationFactor);
        } else {
            evaluationFactor.setId(IDUtils.getId());
            evaluationFactorMapper.insert(evaluationFactor);
        }
        evaluationFactor.setParentFactor(parentFactor);
        
        return evaluationFactor;
    }
    
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteEvaluationFactor(EvaluationFactor factor) {
        int delCount = 0;
        if (StringUtils.isNotEmpty(factor.getContainId())) {
            QueryWrapper<EvaluationFactor> evaluationFactorWrapper = new QueryWrapper<EvaluationFactor>();
            evaluationFactorWrapper.lambda().eq(EvaluationFactor::getId, factor.getContainId());
            delCount = evaluationFactorMapper.delete(evaluationFactorWrapper);
        }
        
        QueryWrapper<EvaluationFactor> evaluationFactorWrapper = new QueryWrapper<EvaluationFactor>();
        evaluationFactorWrapper.lambda().eq(EvaluationFactor::getId, factor.getId());
        delCount += evaluationFactorMapper.delete(evaluationFactorWrapper);
        
        QueryWrapper<EvaluationStepFactor> evaluationStepFactorWrapper = new QueryWrapper<EvaluationStepFactor>();
        evaluationStepFactorWrapper.lambda().eq(EvaluationStepFactor::getFactorId, factor.getId());
        delCount += evaluationStepFactorMapper.delete(evaluationStepFactorWrapper);
        
        return delCount;
    }
    
    /**
     * @Description: 按分类删除指标
     * @author mazc
     * @param tenderId
     * @param auditType
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int delEvaluationFactorByType(String tenderId, String auditType) {
        int delCount = 0;
        delCount = evaluationFactorMapper.delEvaluationFactorByType(tenderId, auditType);
        
        QueryWrapper<EvaluationFactor> evaluationFactorWrapper = new QueryWrapper<EvaluationFactor>();
        evaluationFactorWrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId).eq(EvaluationFactor::getAuditType, auditType);
        
        return delCount += evaluationFactorMapper.delete(evaluationFactorWrapper);
    }
    
    @Override
    public List<EvaluationFactor> getAllEvaluationFactorList(String tenderId, String auditType) {
        QueryWrapper<EvaluationFactor> wrapper = new QueryWrapper<EvaluationFactor>();
        wrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId);
        if (null != auditType && !auditType.isEmpty()) {
            wrapper.lambda().eq(EvaluationFactor::getAuditType, auditType);
        }
        wrapper.orderByAsc(EvaluationFactor.SORT_NO);
        return initChildFactor(evaluationFactorMapper.selectList(wrapper), null);
    }
    
    @Override
    @Transactional(rollbackFor = Exception.class)
    public EvaluationFactor saveEvaluationFactor(EvaluationFactor factor, boolean changeWeigth) {
        EvaluationFactor parentFactor = null;
        // 先记录最原始的浮动指标
        EvaluationFactor sourceFloatFactor = factor.getContainFactor();
        // 如果存在父级指标
        if (StringUtils.isNotEmpty(factor.getParentId())) {
            parentFactor = evaluationFactorMapper.selectById(factor.getParentId());
            if (null != parentFactor) {
                boolean updateParentFlag = false;
                // 如果父级指标存在浮动指标 则将父级指标的浮动指标删除 {父级指标不进行打分 所以不需要浮动指标}
                if (StringUtils.isNotEmpty(parentFactor.getContainId())) {
                    evaluationFactorMapper.deleteById(parentFactor.getContainId());
                    updateParentFlag = true;
                }
                // 如果父级指标是客观项 则修改为非客观项[父级指标不进行打分 所以不能设置为客观项]
                if (StringUtils.isNotEmpty(parentFactor.getObjective()) && parentFactor.getObjective().equals(EvaluationFactorType.OBJECTIVE_YES)) {
                    updateParentFlag = true;
                }
                
                if (null != parentFactor.getFactorType() && !parentFactor.getFactorType().equals(EvaluationFactorType.FACTOR_DEFAULT)) {
                    updateParentFlag = true;
                    parentFactor.setBidPriceCode(null);
                }
                
                if (null != parentFactor.getEvalGroup()) {
                    updateParentFlag = true;
                    parentFactor.setEvalGroup(null);
                }
                parentFactor.setContainId(null);
                // 如果父级指标是客观项 则修改为非客观项[父级指标不进行打分 所以不能设置为客观项]
                if (null == parentFactor.getAuditType() || !parentFactor.getAuditType().equals(EvaluationFactorType.AUDIT_BILL)) {
                    updateParentFlag = true;
                    parentFactor.setFactorType(EvaluationFactorType.FACTOR_DEFAULT);
                }
                
                // 修改父级指标不能拥有的属性 主要是由在当前独立指标下添加子指标问题造成的
                if (updateParentFlag) {
                    EvaluationFactor parentFactorTemp = ReflecUtils.reflex(parentFactor, new String[] { "id", "factorType", "bidPriceCode", "objective", "containId", "evalGroup" }, EvaluationFactor.class);
                    evaluationFactorMapper.updateById(parentFactorTemp);
                }
            }
            factor.setParentFactor(parentFactor);
        }
        // 查询后需重新放置父对象
        factor.setParentFactor(parentFactor);
        boolean isNew = false;
        // 修改
        if (StringUtils.isEmpty(factor.getId())) {
            isNew = true;
            factor.setId(IDUtils.getId());
        }
        
        if (factor.getAuditType().equals(EvaluationFactorType.AUDIT_SCORE)) {
            
            // 如果当前指标存浮动指标
            if (StringUtils.isNotEmpty(factor.getContainId()) && null != factor.getContainFactor()) {
                // 设置当前指标为基础指标
                factor.setFactorType(EvaluationFactorType.FACTOR_BASE);
                EvaluationFactor floatFactor = initFloatFactor(factor.getTenderId(), sourceFloatFactor);
                floatFactor.setId(factor.getContainId());
                floatFactor.setContainId(factor.getId());
                floatFactor.setParentId(factor.getParentId());
                floatFactor.setSortNo(factor.getSortNo() + 1);
                evaluationFactorMapper.updateById(floatFactor);
//				evaluationFactorDao.saveOrUpdateNotNull(floatFactor);
                floatFactor.setContainFactor(factor);
                factor.setContainFactor(floatFactor);
                
                // 如果存在浮动指标id，但不存在浮动指标，说明浮动指标被删除
            } else if (StringUtils.isNotEmpty(factor.getContainId()) && null == sourceFloatFactor) {
                factor.setFactorType(EvaluationFactorType.FACTOR_DEFAULT);
                evaluationFactorMapper.deleteById(factor.getContainId());
//				evaluationFactorDao.deleteById(factor.getContainId(), EvaluationFactor.class);
                factor.setContainId(null);
                factor.setContainFactor(null);
                
                // 如果不存在浮动指标id，但存在指标浮动对象，说明新添加浮动指标
            } else if (StringUtils.isEmpty(factor.getContainId()) && null != sourceFloatFactor) {
                EvaluationFactor floatFactor = initFloatFactor(factor.getTenderId(), sourceFloatFactor);
                floatFactor.setParentId(factor.getParentId());
                floatFactor.setContainId(factor.getId());
                floatFactor.setSortNo(factor.getSortNo() + 1);
                if (StringUtils.isEmpty(floatFactor.getId())) {
                    floatFactor.setId(IDUtils.getId());
                }
                evaluationFactorMapper.updateById(floatFactor);
                floatFactor = evaluationFactorMapper.selectById(floatFactor.getId());
//				floatFactor = (EvaluationFactor)evaluationFactorDao.saveOrUpdateReturn(floatFactor);
                factor.setContainId(floatFactor.getId());
                // 设置当前指标为基础指标
                factor.setFactorType(EvaluationFactorType.FACTOR_BASE);
                floatFactor.setContainFactor(factor);
                factor.setContainFactor(floatFactor);
            } else {
                factor.setContainId(null);
                // 设置指标为默认指标
                factor.setFactorType(EvaluationFactorType.FACTOR_DEFAULT);
            }
        } else if (factor.getAuditType().equals(EvaluationFactorType.AUDIT_PRICE)) {
            
        }
        
        // 修改
        if (isNew) {
            // 必须先设置当前的权重 否如果刷新其他权重后数据将会不一致
            factor = setEvaluationFactorWeight(factor);
            factor.setId(IDUtils.getID(factor.getId()));
            evaluationFactorMapper.insert(factor);
//			evaluationFactorDao.insert(factor);
            // 新增时只需要设置当前新增的权重，不需要刷新其他指标的权重
            changeWeigth = false;
        } else {
            factor.setId(IDUtils.getID(factor.getId()));
            evaluationFactorMapper.updateById(factor);
        }
        
        // 如果修改了权重 为了保正所有指标权重的完整性刷新全部指标的权重
        if (changeWeigth) {
            List<EvaluationFactor> initWeightFactorList = updateEvaluationFactorWeight(factor.getTenderId(), factor.getAuditType());
            if (initWeightFactorList.size() > 0) {
				batchUpdateProperty(initWeightFactorList, new String[]{"factorWeight","factorFinalWeight"});
                LOG.debug("为" + initWeightFactorList.size() + "个指标刷新权重");
            }
        }
        
        return factor;
    }
    
    /**
     * @Description: 初始化浮动指标
     * @author mazc
     * @param tenderId
     * @param factor
     * @return
     */
    private EvaluationFactor initFloatFactor(String tenderId, EvaluationFactor factor) {
        EvaluationFactor floatFactor = new EvaluationFactor();
        floatFactor.setFactorName(factor.getFactorName());
        floatFactor.setFactorMinScore(factor.getFactorMinScore());
        floatFactor.setFactorMaxScore(factor.getFactorMaxScore());
        floatFactor.setFactorScore(factor.getFactorMaxScore());
        floatFactor.setTenderId(tenderId);
        floatFactor.setAuditType(EvaluationFactorType.AUDIT_SCORE);
        // 设置为浮动指标
        floatFactor.setFactorType(EvaluationFactorType.FACTOR_FLOAT);
        floatFactor.setMemo(factor.getMemo());
        // 设置为非客观项
        floatFactor.setObjective(EvaluationFactorType.OBJECTIVE_NO);
        return floatFactor;
    }
    
    /**
     * @Description: 设置权重
     * @author mazc
     * @param factor
     * @return
     */
    private EvaluationFactor setEvaluationFactorWeight(EvaluationFactor factor) {
        // 如果父级指标有权重
        if (null != factor.getParentFactor() && null != factor.getParentFactor().getFactorFinalWeight()) {
            if (null != factor.getFactorWeight()) {
                // 如果当前指标也有权重 则当前指标的实际权重应该为当前权重*父级指标的权重
                factor.setFactorFinalWeight(BigDecimalUtils.div(BigDecimalUtils.mul(factor.getFactorWeight(), factor.getParentFactor().getFactorFinalWeight()), new BigDecimal(100), 2));
            } else {
                // 如果当前指标没有权重，则当前指标的实际权重为父级指标的实际权重
                factor.setFactorFinalWeight(factor.getParentFactor().getFactorFinalWeight());
            }
        } else {
            factor.setFactorFinalWeight(factor.getFactorWeight());
        }
        return factor;
    }
    
    /**
     * @Description: 刷新指标的权重
     * @author guoyr
     */
    public List<EvaluationFactor> updateEvaluationFactorWeight(String tenderId, String auditType) {
        QueryWrapper<EvaluationFactor> evaluationFactorWrapper = new QueryWrapper<EvaluationFactor>();
        evaluationFactorWrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId).eq(EvaluationFactor::getAuditType, auditType).orderByAsc(EvaluationFactor::getSortNo);
//		queryWrapper.setWhereParam("tenderId", tenderId);
//		queryWrapper.setWhereParam("auditType", auditType);
//		queryWrapper.setOrderParam("sortNo", false);
//		List<EvaluationFactor> evaluationFactorList = evaluationFactorDao.queryList(queryWrapper, EvaluationFactor.cla);
        List<EvaluationFactor> evaluationFactorList = evaluationFactorMapper.selectList(evaluationFactorWrapper);
        evaluationFactorList = initChildFactor(evaluationFactorList, null);
        
        List<EvaluationFactor> initWeightFactorList = new ArrayList<EvaluationFactor>();
        if (null != evaluationFactorList && evaluationFactorList.size() > 0) {
            for (EvaluationFactor topFactor : evaluationFactorList) {
                initWeightFactorList.addAll(initChangeEvaluationFactorWeight(topFactor));
            }
        }
        return initWeightFactorList;
    }
    
    /**
     * @Description: 递归修改指标的权重
     * @author guoyr
     * @param factor
     */
    private List<EvaluationFactor> initChangeEvaluationFactorWeight(EvaluationFactor factor) {
        
        List<EvaluationFactor> list = new ArrayList<EvaluationFactor>();
        
        // 先设置权重
        list.add(setEvaluationFactorWeight(factor));
        
        // 再设置子级的权重
        if (null != factor.getChildFactorList() && factor.getChildFactorList().size() > 0) {
            for (EvaluationFactor childFactor : factor.getChildFactorList()) {
                list.addAll(initChangeEvaluationFactorWeight(childFactor));
            }
        }
        
        return list;
    }
    
    /**
     * @Description: 指标信息封装树节点(双重循环实现拼接)
     * @author guoyr
     * @param list
     * @return
     */
    private List<EvaluationFactor> initChildFactor(List<EvaluationFactor> list, List<String> containsFactorList) {
        List<EvaluationFactor> factorList = null;
        if (null != list && list.size() > 0) {
            factorList = new ArrayList<EvaluationFactor>();
            int topCount = 1;
            for (EvaluationFactor parentFactor : list) {
                if (ObjectUtils.equals(parentFactor.getFactorType(), EvaluationFactorType.FACTOR_FLOAT)) {
                    continue;
                }
                boolean isParentFactor = false;
                int childCount = 0;
                for (EvaluationFactor subFactor : list) {
                    // 如果存在包含指标，即存在浮动指标
                    if (StringUtils.isNotEmpty(parentFactor.getContainId()) && subFactor.getId().equals(parentFactor.getContainId())) {
                        parentFactor.setContainFactor(subFactor);
                        subFactor.setContainFactor(parentFactor);
                    }
                    if (StringUtils.isNotEmpty(subFactor.getParentId()) && parentFactor.getId().equals(subFactor.getParentId())) {
                        isParentFactor = true;
                        if (null != containsFactorList && !containsFactorList.contains(subFactor.getId())) {
                            continue;
                        }
                        parentFactor.addChild(subFactor);
                        subFactor.setSortNo(childCount + 1);
                        subFactor.setEvalBidType(parentFactor.getEvalBidType());
                        subFactor.setParentFactor(parentFactor);
                        childCount++;
                    }
                    
                }
                
                // 如果是顶级指标
                if (StringUtils.isEmpty(parentFactor.getParentId())) {
                    // 如果没有子节点，且不是父级指标
                    if (childCount == 0) {
                        if (isParentFactor) {
                            continue;
                        } else {
                            if (null != containsFactorList && !containsFactorList.contains(parentFactor.getId())) {
                                continue;
                            }
                        }
                    }
                    parentFactor.setSortNo(topCount++);
                    factorList.add(parentFactor);
                    parentFactor.setChildCount(childCount);
                }
            }
        }
        return factorList;
    }
    
    /**
     * @Description: 验证是否已经存在投标总报价价格指标
     * @author mazc
     * @param tenderId
     * @param factorId
     * @return
     */
    @Override
    public boolean hasTotalBidPriceFactor(String tenderId, String factorId) {
        return evaluationFactorMapper.hasTotalBidPriceFactor(tenderId, factorId) > 0;
    }
    
    /**
     * @Description: 导入投标报价指标
     * @author mazc
     * @param tenderId
     * @param parentFactor
     * @return
     */
    @Override
    public int importBidPriceFactor(String tenderId, EvaluationFactor parentFactor) {
        //通过tenderid查询价格列表
        QueryWrapper<BidPrice> queryWrapper = new QueryWrapper<BidPrice>();
        queryWrapper.lambda().eq(BidPrice::getTenderId, tenderId);
        queryWrapper.lambda().orderByAsc(BidPrice::getSortNo);
        List<BidPrice> bidPriceList = bidPriceMapper.selectList(queryWrapper);
        //创建一个评审指标列表
        List<EvaluationFactor> evaluationFactorList = new ArrayList<EvaluationFactor>();
        //获得指标的最大序号
        Integer sortNoValue = evaluationFactorMapper.getMaxEvaluationFactorSortNo(tenderId, EvaluationFactorType.AUDIT_PRICE, null != parentFactor ? parentFactor.getId() : null);
        int sortNo = null != sortNoValue ? sortNoValue.intValue() : 0; 
        if (null != bidPriceList && bidPriceList.size() > 0) {
            for (BidPrice bidPrice : bidPriceList) {
            	if(BidPriceEnum.IS_PRICE_ITEM.equals(bidPrice.getBidPriceType())) {
            		++sortNo;
            		EvaluationFactor evaluationFactor = new EvaluationFactor();
            		evaluationFactor.setFactorName(bidPrice.getBidPriceName());
            		evaluationFactor.setTenderId(tenderId);
            		evaluationFactor.setBidPriceCode(bidPrice.getBidPriceCode());
            		evaluationFactor.setAuditType(EvaluationFactorType.AUDIT_PRICE);
            		evaluationFactor.setObjective(EvaluationFactorType.OBJECTIVE_NO);// 设置为非客观项
            		evaluationFactor.setSortNo(sortNo);
            		evaluationFactor.setBidPriceCode(bidPrice.getBidPriceCode());
            		if (null != parentFactor) {
            			evaluationFactor.setParentId(parentFactor.getId());
            			evaluationFactor.setFactorFinalWeight(parentFactor.getFactorFinalWeight());
            		}
            		evaluationFactor.setId(IDUtils.getID(evaluationFactor.getId()));
            		evaluationFactorList.add(evaluationFactor);
            	}
            }
        } else {
            EvaluationFactor evaluationFactor = new EvaluationFactor();
            evaluationFactor.setFactorName("投标总报价");
            evaluationFactor.setTenderId(tenderId);
            evaluationFactor.setAuditType(EvaluationFactorType.AUDIT_PRICE);
            evaluationFactor.setBidPriceCode(BidPrice.BID_PRICE_TOTAL_CODE);// 设置为分项报价
            evaluationFactor.setObjective(EvaluationFactorType.OBJECTIVE_NO);// 设置为非客观项
            evaluationFactor.setSortNo(sortNo);
            if (null != parentFactor) {
                evaluationFactor.setParentId(parentFactor.getId());
                evaluationFactor.setFactorFinalWeight(parentFactor.getFactorFinalWeight());
            }
            evaluationFactor.setId(IDUtils.getID(evaluationFactor.getId()));
            evaluationFactorList.add(evaluationFactor);
        }
        
        if (null != parentFactor) {
            parentFactor.setFactorType(EvaluationFactorType.FACTOR_DEFAULT);
            parentFactor.setBidPriceCode(null);
            EvaluationFactor evaluationFactorTemp = new EvaluationFactor();
            evaluationFactorTemp.setId(parentFactor.getId());
            evaluationFactorTemp.setFactorType(parentFactor.getFactorType());
            evaluationFactorTemp.setBidPriceCode(parentFactor.getBidPriceCode());
            evaluationFactorMapper.updateById(evaluationFactorTemp);
        }
        return evaluationFactorMapper.insertByBatch(evaluationFactorList);
    }
    
    /**
     * @Description: 通过id列表查找评审指标
     * @author mazc
     * @param lsit
     * @return
     */
    @Override
    public List<EvaluationFactor> selectBatchIds(List<String> lsit) {
        return evaluationFactorMapper.selectBatchIds(lsit);
    }
    
    /**
     * @Description: 获取项目下指定类型的指标ids
     * @author mazc
     * @param tenderId
     * @param auditType
     * @return
     */
    @Override
    public List<Object> getFactorIdList(String tenderId, String auditType) {
        QueryWrapper<EvaluationFactor> wrapper = new QueryWrapper<EvaluationFactor>();
        wrapper.select("FACTOR_ID").lambda().eq(EvaluationFactor::getTenderId, tenderId);
        if (StringUtils.isNotEmpty(auditType)) {
            wrapper.lambda().eq(EvaluationFactor::getAuditType, auditType);
        }
        return evaluationFactorMapper.selectObjs(wrapper);
    }
    
    /**
     * @Description: 获得价格指标
     * @author mazc
     * @param tenderId
     * @return
     */
    @Override
    public EvaluationFactor getPriceEvaluationFactor(String tenderId) {
        if (StringUtils.isEmpty(tenderId)) {
            return null;
        }
        QueryWrapper<EvaluationFactor> wrapper = new QueryWrapper<EvaluationFactor>();
        wrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId);
        wrapper.lambda().eq(EvaluationFactor::getAuditType, EvaluationFactorType.AUDIT_PRICE);
        return evaluationFactorMapper.selectOne(wrapper);
    }
    
    /** 
     * @Description: 交换两个指标的排序
     * @author chenxw
     * @param sourceId
     * @param targetId 
     */
    @Override
    public void changeEvaluationFactorSortNo(String sourceId, String targetId) {
        evaluationFactorMapper.changeEvaluationFactorSortNo(sourceId, targetId);
    }
    
    /**
     * @Description: 获得指标的最大序号
     * @author chenxw
     * @param tenderId
     * @param auditType
     * @param parentId
     * @return
     */
    @Override
    public int getMaxEvaluationFactorSortNo(String tenderId, String auditType, String parentId) {
    	Integer integer = evaluationFactorMapper.getMaxEvaluationFactorSortNo(tenderId, auditType, parentId);
        return null != integer ? integer.intValue() : 0 ;
    }
    
    /**
     * @Description: 获得指标的数量
     * @author chenxw
     * @param tenderId
     * @param auditType
     * @return
     */
    @Override
    public int getEvaluationFactorCount(String tenderId, String auditType) {
    	Integer integer = evaluationFactorMapper.getFactorCount(tenderId, auditType);
    	return null != integer ? integer.intValue() : 0 ;
    }
    
    /** 
     * @Description: 导入指标
     * @author chenxw
     * @param tenderId
     * @param complianceFactorList
     * @param detailFactorList
     * @param priceFactorList
     * @param billFactorList
     * @return 
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean importEvaluationFactor(
                String tenderId, 
                List<EvaluationFactor> complianceFactorList, 
                List<EvaluationFactor> detailFactorList, 
                List<EvaluationFactor> priceFactorList, 
                List<EvaluationFactor> billFactorList
            ) {
        List<EvaluationFactor> list = new ArrayList<EvaluationFactor>();
        if (null != complianceFactorList && complianceFactorList.size() > 0) {
            evaluationFactorMapper.delEvaluationFactorByType(tenderId, EvaluationFactorEnum.AUDIT_COMPLIANCE);
            // 组装上下级关系
//          complianceFactorList = initChildFactor(complianceFactorList);
            for (EvaluationFactor factor : complianceFactorList) {
                factor.setId(IDUtils.getId());
                factor.setParentId(null);
                factor.setTenderId(tenderId);
                // 递归设置新的上下级关系
                list.addAll(cycleInsertEvaluationFactor(tenderId, factor));
            }
        }
        if (null != detailFactorList && detailFactorList.size() > 0) {
            evaluationFactorMapper.delEvaluationFactorByType(tenderId, EvaluationFactorEnum.AUDIT_SCORE);
            evaluationFactorMapper.delEvaluationFactorByType(tenderId, EvaluationFactorEnum.AUDIT_CREDIT);
            // 组装上下级关系
//          detailFactorList = initChildFactor(detailFactorList);
            for (EvaluationFactor factor : detailFactorList) {
                if (EvaluationFactorEnum.FACTOR_FLOAT.equals(factor.getFactorType())) {
                    continue;
                }
                factor.setId(IDUtils.getId());
                factor.setParentId(null);
                factor.setTenderId(tenderId);
                // 递归设置新的上下级关系
                list.addAll(cycleInsertEvaluationFactor(tenderId, factor));
            }
        }
        if (null != priceFactorList && priceFactorList.size() > 0) {
            evaluationFactorMapper.delEvaluationFactorByType(tenderId, EvaluationFactorEnum.AUDIT_PRICE);
            int index = 0;
            for (EvaluationFactor factor : priceFactorList) {
                factor.setId(IDUtils.getId());
                factor.setParentId(null);
                factor.setTenderId(tenderId);
                if (index++ != 0) {
                    factor.setBidPriceCode(null);
                }
                // 递归设置新的上下级关系
                list.addAll(cycleInsertEvaluationFactor(tenderId, factor));
            }
        }
        if (null != billFactorList && billFactorList.size() > 0) {
            evaluationFactorMapper.delEvaluationFactorByType(tenderId, EvaluationFactorEnum.AUDIT_BILL);
            for (EvaluationFactor factor : billFactorList) {
                factor.setId(IDUtils.getId());
                factor.setParentId(null);
                factor.setTenderId(tenderId);
                // 递归设置新的上下级关系
                list.addAll(cycleInsertEvaluationFactor(tenderId, factor));
            }
        }
        
        if (list.size() > 0) {
            // 批量导入指标
            int count = evaluationFactorMapper.insertByBatch(list);
            if (count > 0) {
                // 刷新指标的权重
                List<EvaluationFactor> initWeightScoreFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_SCORE);
                if (initWeightScoreFactorList.size() > 0) {
                    count = batchUpdateProperty(initWeightScoreFactorList, new String[] { "factorWeight", "factorFinalWeight" });
                }
                // 刷新指标的权重
                List<EvaluationFactor> initWeightCreditFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_CREDIT);
                if (initWeightCreditFactorList.size() > 0) {
                    count = batchUpdateProperty(initWeightCreditFactorList, new String[] { "factorWeight", "factorFinalWeight" });
                }
                // 刷新指标的权重
                List<EvaluationFactor> initWeightPriceFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_PRICE);
                if (initWeightPriceFactorList.size() > 0) {
                    count = batchUpdateProperty(initWeightPriceFactorList, new String[] { "factorWeight", "factorFinalWeight" });
                }
                // 刷新指标的权重
                List<EvaluationFactor> initWeightBillFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_BILL);
                if (initWeightBillFactorList.size() > 0) {
                    count = batchUpdateProperty(initWeightBillFactorList, new String[] { "factorWeight", "factorFinalWeight" });
                }
            }
        }
        return true;
    }
    
    /**
     * @Description: 递归插入子指标和浮动指标
     * @author guoyr
     * @param tenderId
     * @param factor
     */
    public List<EvaluationFactor> cycleInsertEvaluationFactor(String tenderId, EvaluationFactor factor){
        List<EvaluationFactor> list = new ArrayList<EvaluationFactor>();
        if(null != factor.getContainFactor() && factor.getFactorType().equals(EvaluationFactorEnum.FACTOR_BASE)){
            EvaluationFactor containFactor = factor.getContainFactor();
            containFactor.setContainId(factor.getId());
            containFactor.setId(IDUtils.getId());
            containFactor.setParentId(factor.getParentId());
            containFactor.setTenderId(tenderId);
            containFactor.setSortNo(factor.getSortNo() + 1);
            factor.setContainId(containFactor.getId());
            list.add(containFactor);
        }
        
        if(null != factor.getChildFactorList() && factor.getChildFactorList().size() > 0){
            for(EvaluationFactor childFactor : factor.getChildFactorList()){
                if(EvaluationFactorEnum.FACTOR_FLOAT.equals(childFactor.getFactorType())){
                    continue;
                }
                childFactor.setId(IDUtils.getId());
                childFactor.setParentId(factor.getId());
                childFactor.setTenderId(tenderId);
                list.addAll(cycleInsertEvaluationFactor(tenderId, childFactor));
            }
            
        }
        if(null == factor.getScoreType() || factor.getScoreType().length() == 0){
            factor.setScoreType(EvaluationFactorEnum.SCORE_TYPE_DIRECT);
        }
        list.add(factor);
        return list;
    }
    
    /**
     * @Description: 获得可为评标步骤分配置的指标
     * @author chenxw
     * @param tenderId
     * @param auditType
     * @param stepId
     * @return
     */
    @Override
    public List<EvaluationFactor> getFactorListOfEvaluationStep(String tenderId, String auditType, String stepId) {
        String stepType = "";
        // 如果是符合性评审指标，则评标步骤类型为符合性评标步骤
        if (auditType.equals(EvaluationFactorEnum.AUDIT_COMPLIANCE)) {
            stepType = EvaluationStepEnum.COMPLIANCE;
            // 如果是打分评审指标，则评标步骤类型为详细评标步骤
        } else if (auditType.equals(EvaluationFactorEnum.AUDIT_SCORE)) {
            stepType = EvaluationStepEnum.DETAIL;
            // 如果是价格评审指标，则评标步骤类型为价格评标步骤
        } else if (auditType.equals(EvaluationFactorEnum.AUDIT_PRICE)) {
            stepType = EvaluationStepEnum.PRICE;
        }
        return initChildFactor(evaluationFactorMapper.getFactorListOfEvaluationStep(tenderId, auditType, stepType, stepId), null);
    }
    
    /** 
     * @Description: 获得指定评标步骤的评审指标树
     * @author chenxw
     * @param tenderId
     * @param evaluationStep
     * @return 
     */
    @Override
    public List<EvaluationFactor> getEvaluationStepFactorTreeList(String tenderId, EvaluationStep evaluationStep) {
        return getExpertEvaluationStepFactorTreeList(tenderId, evaluationStep, null);
    }
    
    /** 
     * @Description: 获得专家在指定评标步骤的评审指标树
     * @author chenxw
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @return 
     */
    @Override
    public List<EvaluationFactor> getExpertEvaluationStepFactorTreeList(String tenderId, EvaluationStep evaluationStep, String userId) {
        List<EvaluationFactor> evaluationFactorList = evaluationFactorMapper.getEvaluationStepFactorList(tenderId, evaluationStep.getId());
        List<EvaluationFactor> list = new ArrayList<EvaluationFactor>();
        if (null != evaluationFactorList && evaluationFactorList.size() > 0) {
            List<String> containsFactorList = null;
            if (null != userId) {
                boolean evalByGroup = evaluationStep.getEvaluationParamByKey(EvaluationStepEnum.PARAM_EVAL_BY_GROUP, Boolean.class);
                if (evalByGroup) {
                    containsFactorList = evaluationFactorMapper.getEvalGroupEvaluationList(tenderId, userId);
                }
            }
            
            evaluationFactorList = initChildFactor(evaluationFactorList, containsFactorList);
            for (EvaluationFactor topFactor : evaluationFactorList) {
                if (null != topFactor.getChildFactorList() && topFactor.getChildFactorList().size() == 1) {
                    list.addAll(topFactor.getChildFactorList());
                } else {
                    list.add(topFactor);
                }
            }
        }
        return list;
    }
    
    /** 
     * @Description: 保存下载的评审指标
     * @author chenxw
     * @param tenderId
     * @param evaluationFactorList
     * @return 
     */
    @Override
    public int saveDownloadEvaluationFactor(String tenderId, List<EvaluationFactor> ObeEvaluationFactorList) {
        
        delAllEvaluationFactor(tenderId);
        
        int count = evaluationFactorMapper.insertByBatch(ObeEvaluationFactorList);
        if (count > 0) {
            // 刷新指标的权重
            List<EvaluationFactor> initWeightScoreFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_SCORE);
            if (initWeightScoreFactorList.size() > 0) {
                count = batchUpdateProperty(initWeightScoreFactorList, new String[] { "factorWeight", "factorFinalWeight" });
            }
            // 刷新指标的权重
            List<EvaluationFactor> initWeightCreditFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_CREDIT);
            if (initWeightCreditFactorList.size() > 0) {
                count = batchUpdateProperty(initWeightCreditFactorList, new String[] { "factorWeight", "factorFinalWeight" });
            }
            // 刷新指标的权重
            List<EvaluationFactor> initWeightPriceFactorList = updateEvaluationFactorWeight(tenderId, EvaluationFactorEnum.AUDIT_PRICE);
            if (initWeightPriceFactorList.size() > 0) {
                count = batchUpdateProperty(initWeightPriceFactorList, new String[] { "factorWeight", "factorFinalWeight" });
            }
            // 保存评审内容列表
//            ObeEvaluationFactorList.stream().flatMap(t->Optional.ofNullable(t.getEvalContent()));
            List<ObeEvaluationContent> evaluationContentList = ObeEvaluationFactorList
                    .stream()
                    .flatMap(t -> Optional.ofNullable(t.getEvaluationContentList()).orElseGet(ArrayList::new).stream())
                    .collect(Collectors.toList());
            if (!evaluationContentList.isEmpty()) {
                count = evaluationContentMapper.insertByBatch(evaluationContentList);
            }
            
        }
        return count;
    }
    
    /**
     * @Description: 批量更新指定列
     * @author chenxw
     * @param evaluationFactorList
     * @param propers
     * @return 
     */
    @Override
    public Integer batchUpdateProperty(List<EvaluationFactor> evaluationFactorList, String[] propers) {
        List<EvaluationFactor> list = new ArrayList<EvaluationFactor>();
        for (EvaluationFactor evaluationFactor : evaluationFactorList) {
            list.add(ReflecUtils.reflex(evaluationFactor, propers, EvaluationFactor.class));
        }
        return evaluationFactorMapper.updateEvaluationFactorBatchList(list);
    }

    /** 
     * @Description: 获得指定评标步骤的评审指标(未对指标信息封装树节点，现在主要用于生成报表使用)
     * @author chenxw
     * @param tenderId
     * @param evaluationStep
     * @param removeTopOnly
     * @return 
     */
    @Override
    public List<EvaluationFactor> getEvaluationStepFactorList(String tenderId, EvaluationStep evaluationStep, boolean removeTopOnly) {
        return getExpertEvaluationStepFactorList(tenderId, evaluationStep, null, removeTopOnly);
    }
    
    /** 
     * @Description: 获得专家在指定评标步骤的评审指标(未对指标信息封装树节点，现在主要用于生成报表使用)
     * @author chenxw
     * @param tenderId
     * @param evaluationStep
     * @param userId
     * @param removeTopOnly
     * @return 
     */
    @Override
    public List<EvaluationFactor> getExpertEvaluationStepFactorList(String tenderId, EvaluationStep evaluationStep, String userId, boolean removeTopOnly) {
        List<EvaluationFactor> evalFactorList = evaluationFactorMapper.getEvaluationStepFactorList(tenderId, evaluationStep.getId());
        if(null != evalFactorList && evalFactorList.size() > 0){
            
            List<String> containsFactorList = null;
            if(null != userId){
                boolean evalByGroup = evaluationStep.getEvaluationParamByKey(EvaluationStepEnum.PARAM_EVAL_BY_GROUP, Boolean.class);
                if(evalByGroup){
                    containsFactorList = evaluationFactorMapper.getEvalGroupEvaluationList(tenderId, userId);
                }
            }
            return setEvalFactorRelation(evalFactorList, containsFactorList, removeTopOnly);
        }
        return evalFactorList;
    }
    
    /**
     * @Description: 设置指标父子关系（用于生成报表使用）
     * @author chenxw
     * @param evalFactorList
     * @param containsFactorList
     * @param removeTopOnly
     * @return 
     */
    private List<EvaluationFactor> setEvalFactorRelation(List<EvaluationFactor> evalFactorList, List<String> containsFactorList, boolean removeTopOnly){
        List<EvaluationFactor> newEvaluationFactorList = new ArrayList<EvaluationFactor>();
        if(null == evalFactorList || evalFactorList.size() < 1){
            return newEvaluationFactorList;
        }
        EvaluationFactor topFactor = null;
        int topCount = 0;
        for(EvaluationFactor parentFactor : evalFactorList){
            if(ObjectUtils.equals(parentFactor.getFactorType(), EvaluationFactorEnum.FACTOR_FLOAT)){
                continue;
            }
            boolean isParentFactor = false;
            int childCount = 0;
            for(EvaluationFactor subFactor : evalFactorList){
                // 如果存在包含指标，即存在浮动指标
                if(StringUtils.isNotEmpty(parentFactor.getContainId()) && subFactor.getId().equals(parentFactor.getContainId())){
                    parentFactor.setContainFactor(subFactor);
                    subFactor.setContainFactor(parentFactor);
                }
                if(StringUtils.isNotEmpty(subFactor.getParentId()) && parentFactor.getId().equals(subFactor.getParentId())){
                    isParentFactor = true;
                    if(null != containsFactorList && !containsFactorList.contains(subFactor.getId())){
                        continue;
                    }
                    newEvaluationFactorList.add(subFactor);
                    parentFactor.addChild(subFactor);
                    subFactor.setParentFactor(parentFactor);
                    childCount ++;
                }
            }
            
            // 如果是顶级指标
            if(StringUtils.isEmpty(parentFactor.getParentId())){
                // 如果没有子节点，且不是父级指标
                if(childCount == 0){
                    if(isParentFactor){
                        continue;
                    }else {
                        if(null != containsFactorList && !containsFactorList.contains(parentFactor.getId())){
                            continue;
                        }
                    }
                }
                
//               如果父顶级指标只有一个子节点，则删除父指标
//              if(isParentFactor && removeTopOnly && childCount == 1){
//                  List<EvaluationFactor> childFactorList = parentFactor.getChildFactorList();
//                  for(EvaluationFactor childFactor : childFactorList){
//                      childFactor.setParentId(null);
//                      childFactor.setParentFactor(null);
//                  }
//                  continue;
//              }
                topFactor = parentFactor;
                parentFactor.setSortNo(topCount++);
                newEvaluationFactorList.add(parentFactor);
                parentFactor.setChildCount(childCount);
            }
        }
        
        // 父指标个数为1并且有子节点，删除父指标，否则不删除。
        if(removeTopOnly && topCount == 1 && null != topFactor){
            List<EvaluationFactor> childFactorList = topFactor.getChildFactorList();
            if(null != childFactorList && childFactorList.size() > 0){
                for(EvaluationFactor childFactor : childFactorList){
                    childFactor.setParentId(null);
                    childFactor.setParentFactor(null);
                }
                newEvaluationFactorList.remove(topFactor);
            }
        }
        
        return newEvaluationFactorList;
    }
    
    /** 
     * @Description: 获得父级评审指标分值
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @return 
     */
    @Override
    public BigDecimal getParentFactorScore(String tenderId, String stepId) {
        return evaluationFactorMapper.getEvalNodeParenFactorScore(tenderId, stepId);
    }
    
    @Override
    public List<EvaluationFactor> getEvaluationFactorResultList(String tenderId, String userId, String supplierId, String stepId, String auditType) {
        return initChildFactor(evaluationFactorMapper.getEvaluationFactorResultList(tenderId, userId, supplierId, stepId, auditType), null);
    }
    
    /** 
     * @Description: 获得指定指标类形的权重
     * @author chenxw
     * @param tenderId
     * @param auditType
     * @return 
     */
    @Override
    public BigDecimal getWeightByAuditType(String tenderId, String auditType) {
        return evaluationFactorMapper.getWeightByAuditType(tenderId, auditType);
    }
    
    /** 
     * @Description: 根据清单类型获取清单指标
     * @author chenxw
     * @param tenderId
     * @param billType
     * @return 
     */
    @Override
    public EvaluationFactor getBillFactoryByType(String tenderId, String billType) {
        QueryWrapper<EvaluationFactor> wrapper = new QueryWrapper<EvaluationFactor>();
        wrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId)
                        .eq(EvaluationFactor::getFactorType, billType);
        return getOne(wrapper);
    }
    
    /** 
     * @Description: 根据步骤id获取权重
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @return 
     */
    @Override
    public BigDecimal getWeightByStepId(String tenderId, String stepId) {
        return evaluationFactorMapper.getWeightByStepId(tenderId, stepId);
    }
    
    /** 
     * @Description: 根据步骤id获取得分
     * @author chenxw
     * @param tenderId
     * @param stepId
     * @return 
     */
    @Override
    public BigDecimal getScoreByStepId(String tenderId, String stepId, boolean weightScore) {
        return evaluationFactorMapper.getScoreByStepId(tenderId, stepId, weightScore);
    }
    
    /** 
     * @Description: 获得指定指标类形的得分
     * @author chenxw
     * @param tenderId
     * @param auditType
     * @param weightScore
     * @return 
     */
    @Override
    public BigDecimal getScoreByAuditType(String tenderId, String auditType, boolean weightScore) {
        return evaluationFactorMapper.getScoreByAuditType(tenderId, auditType, weightScore);
    }

    /**
     * @Description: 根据项目id获取指标数量
     * @author chenxw
     * @param tenderId
     * @return 
     */
    @Override
    public int getFactorCountByTenderId(String tenderId) {
        QueryWrapper<EvaluationFactor> queryWrapper = new QueryWrapper<EvaluationFactor>();
        queryWrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId);
        return count(queryWrapper);
    }
    
    /** 
     * @Description: 根据项目id获取指标列表
     * @author chenxw
     * @param tenderId
     * @return 
     */
    @Override
    public List<EvaluationFactor> getFactorListByTenderId(String tenderId) {
        QueryWrapper<EvaluationFactor> queryWrapper = new QueryWrapper<EvaluationFactor>();
        queryWrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId);
        return list(queryWrapper);
    }
    
    /**
     * @Description: 根据指标编号获取指标
     * @author chenxw
     * @param tenderId
     * @param factorCode
     * @return 
     */
    @Override
    public EvaluationFactor getFactorByFactorCode(String tenderId, String factorCode) {
        QueryWrapper<EvaluationFactor> wrapper = new QueryWrapper<EvaluationFactor>();
        wrapper.lambda().eq(EvaluationFactor::getTenderId, tenderId)
                        .eq(EvaluationFactor::getFactorCode, factorCode);
        return getOne(wrapper);
    }

    /**
     * @Description: 保存商务指标
     * @author chenxw
     * @param tenderProject
     * @param businessFactor
     * @return
     */
    @Override
    public boolean saveBusinessFactor(TenderProject tenderProject, EvaluationFactor businessFactor) {
        boolean isSuccess = false;
        int successCount;
        if (null == businessFactor.getId()) {
            businessFactor.setId(IDUtils.getId());
            successCount = evaluationFactorMapper.insert(businessFactor);
        } else {
            successCount = evaluationFactorMapper.updateAssignProperty(businessFactor, new String[]{"factorWeight", "computerParams"});
        }
        if (successCount > 0) {
            isSuccess = true;
            tenderProjectMapper.updateAssignProperty(tenderProject, new String[]{"bidPriceLimit"});
        }
        return isSuccess;
    }

	@Override
	public boolean updateAssignProperty(EvaluationFactor evaluationStep, String[] attributes) {
		return evaluationFactorMapper.updateAssignProperty(evaluationStep, attributes) > 0;
	}

}
