package com.gx.obe.server.management.evaluation.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.management.evaluation.dao.EvaluationReportMapper;
import com.gx.obe.server.management.evaluation.entity.EvaluationReport;
import com.gx.obe.server.management.evaluation.service.EvaluationReportService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
@Transactional
public class EvaluationReportServiceImpl extends ServiceImpl<EvaluationReportMapper, EvaluationReport> implements EvaluationReportService {

	@Autowired
	private EvaluationReportMapper evaluationReportMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeEvaluationReportList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch (List<EvaluationReport> ObeEvaluationReportList){
		return evaluationReportMapper.updateBatchList(ObeEvaluationReportList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeEvaluationReportList 
	 */
	 @Override
	 @Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch (List<EvaluationReport> ObeEvaluationReportList){
		return evaluationReportMapper.insertByBatch(ObeEvaluationReportList);
	}

	/** 
	 * @Description: 删除专家的评标报表
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param stepId
	 * @return 
	 */
	@Override
	public boolean deleteUserEvaluationReport(String tenderId, String userId, String stepId) {
		return evaluationReportMapper.deleteUserEvaluationReport(tenderId, userId, stepId) > 0;
	}

	/** 
	 * @Description: 获得专家的评标报表
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param stepId
	 * @return 
	 */
	@Override
	public EvaluationReport getUserEvaluationReport(String tenderId, String userId, String stepId) {
		return evaluationReportMapper.getUserEvaluationReport(tenderId, userId, stepId);
	}

	/** 
	 * @Description: 
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param evaluationStepId
	 * @param evaluationReport
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean insertEvaluationReport(String tenderId, String userId, String evaluationStepId,EvaluationReport evaluationReport) {
		evaluationReportMapper.deleteUserEvaluationReport(tenderId, userId, evaluationStepId);
		int size = evaluationReportMapper.insert(evaluationReport);
		if(size > 0) {
			return true;
		}
		return false;
	}

	@Override
	public List<EvaluationReport> getEvaluationReportListByTenderId(String tenderId) {
		QueryWrapper<EvaluationReport> queryWrapper = new QueryWrapper<EvaluationReport>();
		queryWrapper.lambda().eq(EvaluationReport::getTenderId, tenderId);
		return evaluationReportMapper.selectList(queryWrapper);
	}
	

}
