package com.gx.obe.server.management.expert.controller;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.management.expert.entity.ExpertEvaluationResult;
import com.gx.obe.server.management.expert.service.ExpertEvaluationResultService;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
/** 
 * @Description: 
 * @author mazc 
 */
@Api(tags = "")
@RestController
@AllArgsConstructor
@RequestMapping("/expertEvaluationResult")
public class ExpertEvaluationResultController extends BaseController<ExpertEvaluationResultService,ExpertEvaluationResult>{
	@Autowired
	public ExpertEvaluationResultService expertEvaluationResultServices;
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertEvaluationResultList 
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<ExpertEvaluationResult> ObeExpertEvaluationResultList) {
		return expertEvaluationResultServices.insertByBatch(ObeExpertEvaluationResultList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param ObeExpertEvaluationResultList 
	 */
	@PostMapping("/updateByBatch")
	public int updateByBatch(@RequestBody List<ExpertEvaluationResult> ObeExpertEvaluationResultList) {
		return expertEvaluationResultServices.updateByBatch(ObeExpertEvaluationResultList);
	}
	
	
	/**
	 * @Description: 获得评标专家对指定评标步骤对投标人的评审记录
	 * @author mazc
	 * @return 
	 */
	@GetMapping("/getStepAllExpertEvaluationResultList")
	public List<ExpertEvaluationResult> getStepAllExpertEvaluationResultList(String tenderId, String stepId){
		return expertEvaluationResultServices.getStepAllExpertEvaluationResultList(tenderId, stepId);
	}
	
	/**
	 * @Description: 删除评标专家对指定评标步骤的所有评审记录
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return
	 */
	@GetMapping("/deleteExpertEvaluationResult")
	public int deleteExpertEvaluationResult(String tenderId, String stepId, String userId) {
		return expertEvaluationResultServices.deleteExpertEvaluationResult(tenderId,stepId,userId);
	}
	
	/**
	 * @Description: 获得供应商在指定评标步骤的所有专家的评标结果
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @return 
	 */
	@GetMapping("/getSupplierStepAllExpertEvaluationResultList")
	public List<ExpertEvaluationResult> getSupplierStepAllExpertEvaluationResultList(String tenderId, String stepId, String supplierId){
		return expertEvaluationResultServices.getSupplierStepAllExpertEvaluationResultList(tenderId,stepId,supplierId);
	}
	
	/**
	 * @Description: 按供应商分组获得每家供应商的评分汇总 
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	@GetMapping("/getSupplierSumEvaluationResultList")
	public List<Map<String, Object>> getSupplierSumEvaluationResultList(String tenderId, String stepId, String userId){
		return expertEvaluationResultServices.getSupplierSumEvaluationResultList(tenderId,stepId,userId);
	}
	
	/**
	 * @Description: 获得供应商在指定评标步骤的平均得分
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @param removeMaxAndMin
	 * @return 
	 */
	@GetMapping("/getSupplierStepAvgResult")
	public List<BigDecimal> getSupplierStepAvgResult(String tenderId, String stepId, String supplierId, boolean removeMaxAndMin){
		return expertEvaluationResultServices.getSupplierStepAvgResult(tenderId,stepId,supplierId,removeMaxAndMin);
	}

	/**
	 * @Description: 获得供应商在指定评标步骤的平均得分
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @param removeMaxAndMin
	 * @return 
	 */
	@GetMapping("/getSupplierStepAvgResultByFactor")
	public List<BigDecimal> getSupplierStepAvgResultByFactor(String tenderId, String stepId, String supplierId, boolean removeMaxAndMin){
		return expertEvaluationResultServices.getSupplierStepAvgResultByFactor(tenderId,stepId,supplierId,removeMaxAndMin);
	}
	
	/**
	 * @Description: 删除专家评审结果
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId 
	 * @return 
	 */
	@GetMapping("/deleteExpertEvaluationAndFactorResult")
	public boolean deleteExpertEvaluationAndFactorResult(String tenderId, String stepId, String userId) {
		return expertEvaluationResultServices.deleteExpertEvaluationAndFactorResult(tenderId,stepId,userId);
	}
	
	
	
	
	@GetMapping("/getSupplierAllEvaluationResultList")
	public List<ExpertEvaluationResult> getSupplierAllEvaluationResultList(String tenderId,String supplierId){
		return expertEvaluationResultServices.getSupplierAllEvaluationResultList(tenderId, supplierId);
	}
	@GetMapping("/getAllExpertEvaluationResultList")
	public List<ExpertEvaluationResult> getAllExpertEvaluationResultList(String tenderId ){
		return expertEvaluationResultServices.getAllExpertEvaluationResultList(tenderId);
	}
	@GetMapping("/getExpertSupplierEvaluationResultList")
	public List<ExpertEvaluationResult> getExpertSupplierEvaluationResultList(String tenderId,String supplierId ,String userId ){
		return expertEvaluationResultServices.getExpertSupplierEvaluationResultList(tenderId, supplierId, userId);
	}
	 
	@GetMapping("/getExpertEvaluationResult")
	public ExpertEvaluationResult getExpertEvaluationResult(String tenderId, String stepId, String supplierId, String userId){
		return expertEvaluationResultServices.getExpertEvaluationResult(tenderId, stepId, supplierId, userId);
	}
	@GetMapping("/getSupplierStepAllExpertEvaluationAccessmentList")
	public	List<Object> getSupplierStepAllExpertEvaluationAccessmentList(String tenderId,String supplierId ,String stepId ){
		return expertEvaluationResultServices.getSupplierStepAllExpertEvaluationAccessmentList(tenderId, supplierId, stepId);
	}
	@GetMapping("/getExpertStepAllSupplierEvaluationResultList")
	public List<ExpertEvaluationResult> getExpertEvaluationResult(String tenderId, String stepId, String userId){ 
		return expertEvaluationResultServices.getExpertEvaluationResult(tenderId, stepId, userId);
	}
	
	
	
	
	
}