package com.gx.obe.server.management.expert.dao;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.gx.obe.server.management.expert.entity.ExpertEvaluationResult;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import java.util.List;
import java.util.Map;
/** 
 * @Description: 
 * @author mazc 
 */
 @Mapper
public interface ExpertEvaluationResultMapper extends BaseMapper<ExpertEvaluationResult> {

	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertEvaluationResultList 
	 */
	Integer updateBatchList (List<ExpertEvaluationResult> ObeExpertEvaluationResultList);
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param ObeExpertEvaluationResultList 
	 */
	Integer insertByBatch (@Param("ObeExpertEvaluationResultList")List<ExpertEvaluationResult> ObeExpertEvaluationResultList);

	/**
	 * @Description:  获得评标专家对指定评标步骤对投标人的评审记录
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	List<ExpertEvaluationResult> getStepAllExpertEvaluationResultList(@Param("tenderId")String tenderId, @Param("stepId")String stepId);
	
	
	/**
	 * @Description: 删除评标专家对指定评标步骤的所有评审记录
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	Integer deleteExpertEvaluationResult(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("userId")String userId);
	
	
	/**
	 * @Description: 获得供应商的价格得分结果
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	List<Map<String, Object>> findPriceEvaluationResultList(@Param("tenderId") String tenderId);
	
	
	/**
	 * @Description: 获得投标人在当前评标步骤中评审不通过的个数
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @return 
	 */
	Integer getSupplierDisAccessmentCount(@Param("tenderId")String tenderId, @Param("stepId")String stepId,  @Param("supplierId")String supplierId);
	
	
	/**
	 * @Description: 获得供应商在指定评标步骤的平均得分
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @param removeMaxAndMin
	 * @return 
	 */
	Map<String, Object> getSupplierStepAvgResult(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("supplierId")String supplierId, @Param("removeMaxAndMin")boolean removeMaxAndMin);
	
    /**
     * @Description: 获得供应商在指定评标步骤的所有专家的评标结果
     * @author mazc
     * @param tenderId
     * @param stepId
     * @param supplierId
     * @return 
     */
    List<ExpertEvaluationResult> getSupplierStepAllExpertEvaluationResultList(@Param("tenderId")String tenderId, @Param("stepId")String stepId,@Param("supplierId") String supplierId);
    
	/**
	 * @Description:  获得供应商在指定评标步骤的平均得分
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param supplierId
	 * @param removeMaxAndMin
	 * @return 
	 */
	Map<String, Object> getSupplierStepAvgResultByFactor(@Param("tenderId")String tenderId, @Param("stepId")String stepId, @Param("supplierId")String supplierId, @Param("removeMaxAndMin")boolean removeMaxAndMin);
	
	
	/**
	 * @Description: 删除一个或多个用户数据
	 * @author mazc
	 * @param tenderId
	 * @param stepId
	 * @param userIdList
	 * @return 
	 */
	Integer deleteEvaluationResultAndUserId(@Param("tenderId") String tenderId,@Param("stepId")String stepId,@Param("userIdList")List<String> userIdList);
	
	
	/**
	 * @Description:重新开始评审、 删除专家对投标人的评标结果
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	Integer deleteRestartEvaluation(@Param("tenderId") String tenderId);

	/**
	 * @Description: 获得供应商在指定评标步骤的指定专家的评标结果
	 * @param tenderId
	 * @param userId
	 * @param stepId
	 * @param supplierId
	 * @return
	 */
	List<ExpertEvaluationResult> getSupplierExpertEvaluationResultList(@Param("tenderId") String tenderId,@Param("stepId")  String stepId,@Param("supplierId")  String supplierId);

	/**
	 * @Description: 按供应商分组获得每家供应商的评分汇总 
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId
	 * @param userId
	 * @return 
	 */
	List<Map<String, Object>> getSupplierSumEvaluationResultList(@Param("tenderId") String tenderId,@Param("stepId")  String stepId, @Param("userId") String userId);
	
	/**
	 * @Description: 删除评审结果记录
	 * @author wangxiang
	 * @param tenderId
	 * @param stepId 
	 */
	 Integer deleteExpertEvaluationResult(@Param("tenderId")String tenderId, @Param("stepId") String stepId);
	 
	 
	 List<ExpertEvaluationResult> getExpertEvaluationResultList(@Param("tenderId")String tenderId,@Param("stepId") String stepId,@Param("userId") String userId);
	
}