package com.gx.obe.server.management.fileload.controller;

import static com.gx.obe.server.management.fileload.utill.FileUtils.generatePath;
import static com.gx.obe.server.management.fileload.utill.FileUtils.merge;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.common.vo.Result;
import com.gx.obe.server.management.fileload.entity.FileChunk;
import com.gx.obe.server.management.fileload.entity.ParameterInfo;
import com.gx.obe.server.management.fileload.utill.FileMd5Util;

import lombok.extern.slf4j.Slf4j;
/** 
 * @Description: 
 * @author mazc 
 */
@Slf4j
@RestController
@RequestMapping("/uploader")
public class UploadController {
	
	private static final org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(UploadController.class);
	
    @Value("${upload.folder}")
    private String uploadFolder;
    
      
    @PostMapping("/chunk")
    public Result<String> uploadChunk(FileChunk chunk) {
        MultipartFile file = chunk.getFile();
        try {
        	byte[] bytes = file.getBytes();
            Path path = Paths.get(generatePath(uploadFolder, chunk));
            //文件写入指定路径
            Files.write(path, bytes);       
            Result<String> result = new Result<String>();
            result.setCode(Result.SUCCESS);
            result.setData("success");
            result.setMsg("上传成功");
            return result;
        } catch (Exception e) {
        	e.printStackTrace();
            log.error(e.getMessage());
            Result<String> result = new Result<String>();
            result.setCode(Result.FAIL);
            result.setData("fail");
            result.setMsg("上传异常");
            return result;
        }
    
    }
    
    @PostMapping("/checkFile")
    public boolean checkChunk(ParameterInfo fileInfo) {
    	Path path = Paths.get(generatePath(uploadFolder, fileInfo));
    	File file = new File(path.toString());
    	if(file.exists()) {
    		return true;
    	}
    	return false;
    }
    
    @PostMapping("/checkFileMD5")
    public boolean checkChunkMD5(FileChunk fileChunk) {
    	Path path = Paths.get(generatePath(uploadFolder, fileChunk));
    	File file = new File(path.toString());
    	if(file.exists()) {
     		if(MD5check(file,fileChunk.getFileChunkMD5())) {
    			return true;    			
    		}
    	}
    	return false;
    }  

    @PostMapping("/chunk/mergeFile")
    public Result<String> mergeFile(ParameterInfo fileInfo) {
        String filename = fileInfo.getFilename();
        StringBuilder sb = new StringBuilder();
        sb.append(uploadFolder).append("/").append(fileInfo.getTenderId().trim());
        
        if(StringUtils.isNotEmpty(fileInfo.getSupplierId()) ) {
        	sb.append("/").append(fileInfo.getSupplierId().trim());
        }        
        if(StringUtils.isNotEmpty(fileInfo.getFileType())) {
        	sb.append("/").append(fileInfo.getFileType().trim());
        }
        String file = sb.toString() + "/" + filename;
        String folder = sb.toString() ;
        try {
			merge(file, folder, filename);
		} catch (IOException e) {
		    log.error(e.getMessage());
		    Result<String> result = new Result<String>();
		    result.setCode(Result.FAIL);
            result.setData("fail");
            result.setMsg("上传异常");
            return result;
		}
        Result<String> result = new Result<String>();
        result.setCode(Result.SUCCESS);
        result.setData("success");
        result.setMsg("上传成功!");
        if(!MD5check(new File(file),fileInfo.getFileMD5())) {
        	 result.setMsg("上传成功,文件MD5校验失败!");
        }
        return result;
    }
    
    private boolean MD5check(File file ,String fileMD5){
    	String md5 = FileMd5Util.getFileMD5(file);    	
    	return md5.equals(md5);
    }

}
