package com.gx.obe.server.management.im.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.im.dao.MessageMapper;
import com.gx.obe.server.management.im.entity.Message;
import com.gx.obe.server.management.im.service.MessageService;

/** 
 * @Description: 
 * @author mazc 
 */
@Service
public class MessageServiceImpl extends ServiceImpl<MessageMapper, Message> implements MessageService {

	@Autowired
	private MessageMapper messageMapper;
	
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param MessageList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer updateByBatch(List<Message> MessageList) {
		return messageMapper.updateBatchList(MessageList);
	}
	
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param MessageList 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Integer insertByBatch(List<Message> MessageList) {
		return messageMapper.insertByBatch(MessageList);
	}
	
	/**
     * @Description: 指定字段修改
     * @author chenxw
     * @param Message
	 * @param attributes
     */
	@Override
    public boolean updateAssignProperty(Message Message, String[] attributes) {
		return messageMapper.updateAssignProperty(Message, attributes) > 0;
	}
    
    /**
     * @Description: 批量指定字段修改
     * @author chenxw
     * @param MessageList
	 * @param attributes
     */
	@Override
    public Integer batchUpdateProperty(List<Message> MessageList, String[] attributes) {
		return messageMapper.batchUpdateProperty(MessageList, attributes);
	}
	
	/** 
	 * @Description: 批量添加或跟新
	 * @author chenxw
	 * @param MessageList
	 * @param attributes
	 * @return 
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<Message> MessageList, String[] attributes) {
        List<Message> insertMessageList = new ArrayList<>();
        List<Message> updateMessageList = new ArrayList<>();
        for (Message Message : MessageList) {
            if (StringUtils.isEmpty(Message.getId())) {
                Message.setId(IDUtils.getId());
                insertMessageList.add(Message);
            } else {
                updateMessageList.add(Message);
            }
        }
        int count = 0;
        if (insertMessageList.size() > 0) {
            count += messageMapper.insertByBatch(insertMessageList);
        }
        if (updateMessageList.size() > 0) {
            count += messageMapper.batchUpdateProperty(updateMessageList, attributes);
        }
        return count;
    }

	/** 
	 * @Description: 
	 * @author mazc
	 * @param id
	 * @return 
	 */
	@Override
	public List<Message> getMessageByReceiveIdlist(String id) {
		QueryWrapper<Message> queryWrapper = new QueryWrapper<Message>();
		queryWrapper.lambda().select(Message::getSendId,Message::getSendId).eq(Message::getReceiveId, id);
		return messageMapper.selectList(queryWrapper);
	}

	@Override
	public List<Message> getMessageByReceiveIdAndSendId(String myId, String receiveId) {
		QueryWrapper<Message> queryWrapper = new QueryWrapper<Message>();
		queryWrapper.lambda().eq(Message::getReceiveId, myId).eq(Message::getSendId, receiveId);
		return messageMapper.selectList(queryWrapper);
	}

	@Override
	public List<Message> getMessageByGroupIdAndSendTimeAsc(String groupId) {
		QueryWrapper<Message> queryWrapper = new QueryWrapper<Message>();
		queryWrapper.lambda().eq(Message::getGroupId, groupId).orderByAsc(Message::getSendTime);
		return messageMapper.selectList(queryWrapper);
	}

	@Override
	public List<Message> getMessageListByReceiveIdAndSendIdAndSendTiemAsc(String myId, String uid) {
		 QueryWrapper<Message> queryWrapper = new QueryWrapper<Message>();
         queryWrapper.lambda().eq(Message::getReceiveId, myId).eq(Message::getSendId, uid).orderByDesc(Message::getSendTime);
         return messageMapper.selectList(queryWrapper);
	}

	@Override
	public boolean saveUpdate(Message message) {
		if(null == message.getId()) {
			message.setId(IDUtils.getId());
			return  messageMapper.insert(message) > 0;
		}else {
			return messageMapper.updateById(message) > 0;
		}
	}

}
