package com.gx.obe.server.management.project.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.gx.obe.server.common.utils.*;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.management.project.entity.ProjectRule;
import com.gx.obe.server.management.project.service.ProjectRuleService;
import com.gx.obe.server.management.vo.BatchUpdatePropertyVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;
import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

 /**  
 * 
 * @author 
 * @Title: ProjectRuleController  
 * @Package com.gx.obe.server.management.user.controller   
 * @Description:   
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-04-30   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Api(tags = "项目规则管理")
@RestController
@AllArgsConstructor
@RequestMapping("/projectRule")
public class ProjectRuleController {
	
	@Autowired
	private ProjectRuleService projectRuleService;
	
	@GetMapping("/getProjectRule")
	public ProjectRule findByTenderID(String tenderId) {
		QueryWrapper<ProjectRule> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda()
				.eq(ProjectRule::getTenderId, tenderId)
				.orderByAsc(ProjectRule::getBidOpeningStartTime);
		List<ProjectRule> list = projectRuleService.list(queryWrapper);
		if (CollectionUtils.isNotNull(list)) return list.get(0);
		return null;
	}

	@PostMapping("/insert")
	public ProjectRule createSave(@RequestBody ProjectRule entity){
		return projectRuleService.save(entity) ? entity : null ;
	}
	
	@PostMapping("/updateAssignProperty")
	public boolean  updateAssignProperty(@RequestBody UpdateAssignPropertyVo<ProjectRule> vo) {
		return projectRuleService.updateAssignProperty(vo.getEntity(),vo.getAttributes());
	}
	
	@PostMapping("/updateAssignPropertyByProjectId")
 	public boolean  updateAssignPropertyByProjectId(@RequestBody UpdateAssignPropertyVo<ProjectRule> vo) {
		return projectRuleService.updateAssignPropertyByProjectId(vo.getProjectId(), vo.getEntity(), vo.getAttributes());
	}
	
	@PostMapping("/batchUpdateProperty")
	public boolean batchUpdateProperty(@RequestBody BatchUpdatePropertyVo<ProjectRule> vo) {
		return projectRuleService.batchUpdateProperty(vo.getEntityList(), vo.getAttributes()) > 0;
	}
	
	
	@PostMapping("/updateRuleList")
	public boolean  updateProjectRuleInfo(@RequestBody RequestBodyEntity<ProjectRule> requestBodyEntity) {
		LambdaQueryWrapper<ProjectRule> query = Wrappers.lambdaQuery();
		query.in(ProjectRule::getTenderId,requestBodyEntity.getList());
		return projectRuleService.update(requestBodyEntity.getEntity(),query);
 	}
	
	@GetMapping("/isAnonymous")
	public boolean getAnonymous(String tenderId){
		return projectRuleService.isAnonymous(tenderId);
	}
	
	/**
	 * @Description: 保存或更新
	 * @author mazc
	 * @return 
	 */
	@PostMapping("/saveOrUpdate")
	public boolean saveOrUpdate(@RequestBody ProjectRule projectRule){
		if(StringUtils.isEmpty(projectRule.getId())) {
			projectRule.setId(IDUtils.getId());
			return projectRuleService.save(projectRule);
		}else {
			return projectRuleService.updateById(projectRule);
		}
	}
	
	@PostMapping("/updateRule")
	public boolean updateRule(@RequestBody ProjectRule projectRule){
		return projectRuleService.updateById(projectRule);
	}
	
	/**
	 * @Description: 获得项目的开标时间
	 * @author mazc
	 * @param tenderId
	 * @param bidopeningStage
	 * @return 
	 */
	@GetMapping("/getBidOpeningTime")
	public String getBidOpeningTime(String tenderId, String bidopeningStage){
		Date openingTime = projectRuleService.getBidOpeningTime(tenderId, bidopeningStage);
		if(null != openingTime) {
			return DateUtils.getTimestampFormat(openingTime);
		}
		return null;
	}
	
	/**
	 * @Description: 获得项目的开标时间（所有标段的时间列表）
	 * @author mazc
	 * @param projectId
	 * @param bidopeningStage
	 * @return 
	 */
	@GetMapping("/getProjectBidOpeningTime")
	public List<String> getProjectBidOpeningTime(String projectId, String bidopeningStage){
		List<Date> openingTime = projectRuleService.getProjectBidOpeningTime(projectId, bidopeningStage);
		List<String> openingTimeList = new ArrayList<>();
		if(null != openingTime) {
			for(Date time : openingTime) {
				openingTimeList.add(DateUtils.getTimestampFormat(time));
			}
		}
		return openingTimeList;
	}
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param tenderProjectRuleEntity
	 * @return 
	 */
	@PostMapping("/saveOrUpdateReturn")
	public ProjectRule saveOrUpdateReturn(@RequestBody ProjectRule tenderProjectRuleEntity){
		return projectRuleService.saveOrUpdateReturn(tenderProjectRuleEntity);
	}
	
	@PostMapping("/updateBidOpeningTimeOrOpeningStartTime")
	@Deprecated
	public int updateBidOpeningTimeOrOpeningStartTime(@RequestBody RequestBodyEntity<Date> bodyEntity){
		List<Object> tenderIdList = bodyEntity.getList();
		Date date=  bodyEntity.getEntity();
		String dbColumnName=(String)bodyEntity.getParam("dbColumnName");
		return projectRuleService.updateBidOpeningTimeOrOpeningStartTime(tenderIdList,date,dbColumnName);
	}
	
	/**
	 * @Description: 是否到达某个时间
	 * @author mazc
	 * @param tenderId
	 * @param dbColumnName 项目规则表中所对应的时间的字段名称
	 * @return 
	 */
	@GetMapping("/isStartTime")
	public boolean isStartTime(String tenderId, String dbColumnName){
		QueryWrapper<ProjectRule> queryWrapper = new QueryWrapper<>();
		queryWrapper.select(dbColumnName).lambda().eq(ProjectRule::getTenderId, tenderId);
		Map<String, Object> map = projectRuleService.getMap(queryWrapper);
		return NowTimeUtils.getNowDate().after((Date)map.get(dbColumnName));
	}
	/**
	 * @Description: 是否到达某个时间
	 * @author mazc
	 * @param tenderId
	 * @param evalMethod
	 * @param evalMethodName 项目规则表中所对应的时间的字段名称
	 * @return
	 */
	@GetMapping("/updateEvalMethod")
	public boolean updateEvalMethod(String tenderId, String evalMethod, String evalMethodName){
		return projectRuleService.updateEvalMethod(tenderId, evalMethod, evalMethodName);
	}
}