package com.gx.obe.server.management.project.controller;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gx.obe.server.common.base.BaseController;
import com.gx.obe.server.common.json.EntityUtill;
import com.gx.obe.server.common.log.OperationType;
import com.gx.obe.server.common.log.SysLogAnnotation;
import com.gx.obe.server.common.utils.DateUtils;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.common.vo.RequestBodyEntity;
import com.gx.obe.server.common.vo.Result;
import com.gx.obe.server.config.error.ResultBody;
import com.gx.obe.server.management.project.entity.ProjectRule;
import com.gx.obe.server.management.project.entity.TenderProject;
import com.gx.obe.server.management.project.service.TenderProjectService;
import com.gx.obe.server.management.vo.BatchUpdatePropertyVo;
import com.gx.obe.server.management.vo.ImportBidOpenResultVo;
import com.gx.obe.server.management.vo.SaveDownLoadProjectVo;
import com.gx.obe.server.management.vo.TenderPackOpeningInfo;
import com.gx.obe.server.management.vo.TenderPackPriceFactorVo;
import com.gx.obe.server.management.vo.TenderProjectVo;
import com.gx.obe.server.management.vo.UpdateAssignPropertyVo;

import io.swagger.annotations.Api;
import lombok.AllArgsConstructor;
 
 
 /**  
 * 
 * @author 
 * @Title: TenderProjectController  
 * @Package com.gx.obe.server.management.user.controllers   
 * @Description:   
 * @author: XiaoMa Pro - myzhichao@163.com
 * @date:   2019-04-29   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Api(tags = "项目管理")
@RestController
@AllArgsConstructor
@RequestMapping("/tenderProject")
public class TenderProjectController  extends BaseController<TenderProjectService,TenderProject>{
	@Autowired
	private TenderProjectService tenderProjectService ;
	
	
	@PostMapping("/insertTenderProject")
	@SysLogAnnotation(detail="插入项目",level = 0,operationType = OperationType.INSERT)
	public TenderProject createSave(@RequestBody TenderProject entity){
		return tenderProjectService.save(entity) ? entity : null ;
	}
	
	
	@PostMapping("/updateAssignProperty")
//	@SysLogAnnotation(detail="指定字段修改项目信息",level = 0,operationType = OperationType.UPDATE)
    public boolean updateAssignProperty(@RequestBody UpdateAssignPropertyVo<TenderProject> updateAssignPropertyVo) {
        TenderProject tenderProject = updateAssignPropertyVo.getEntity();
        String[] attributes = updateAssignPropertyVo.getAttributes();
        return tenderProjectService.updateAssignProperty(tenderProject, attributes);
    }
	
	@PostMapping("/updateByParendId")
//	@SysLogAnnotation(detail="修改项目信息",level = 0,operationType = OperationType.UPDATE)
	public int updateByParendId(@RequestBody RequestBodyEntity<TenderProject> requestBodyEntity) {
		TenderProject tenderProject = requestBodyEntity.getEntity();
		String parendId = (String) requestBodyEntity.getParam("parendId");
		return tenderProjectService.updateByParendId(tenderProject, parendId);
	}
	
	@PostMapping("/update")
	@SysLogAnnotation(detail="修改项目信息",level = 0,operationType = OperationType.UPDATE)
	public boolean update(@RequestBody TenderProject entity) {
		QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(TenderProject::getId, entity.getId());
		return tenderProjectService.update(entity,queryWrapper);
	}
	
	
	@PostMapping("/updateTenderType")
	@SysLogAnnotation(detail="变更项目类型",level = 5,operationType = OperationType.UPDATE)
	public void updateTenderType (String tenderType, String parentId ,String useStatus){
		TenderProject tenderProject = new TenderProject();
		tenderProject.setTenderType(tenderType);
		UpdateWrapper<TenderProject> updateWrapper = new UpdateWrapper<>();
		updateWrapper.eq(TenderProject.TENDER_PARENT_ID,parentId).eq(TenderProject.USE_STATUS, useStatus);
		tenderProjectService.update(tenderProject, updateWrapper);
	}
	
	@GetMapping("/getSubPackagePageList")
	 public IPage<TenderProject> findTenderPack(int pageNo , int pageSize,String tenderId) {
		 Page<TenderProject> page = new Page<>(pageNo, pageSize);
		 return tenderProjectService.findTenderPack(page, tenderId);
	 }
	
	/**
	 * @Description: 获得可唱标的包件
	 * @author mazc
	 * @param parentId
	 * @param pageNo
	 * @param pageSize
	 * @return 
	 */
	@GetMapping("/getSignBidTenderPackagePageList")
	public IPage<TenderProject> getSignBidTenderPackagePageList(String tenderId ,int pageNo, int pageSize){
		 Page<TenderProject> page = new Page<>(pageNo, pageSize);
		 return tenderProjectService.getSignBidTenderPackagePageList(page, tenderId);
	}
	
	
	/**
	 * @Description: 按分页查询评标项目
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@GetMapping("/getEvaluationProjectPageList")
	 public IPage<TenderProjectVo> getEvaluationProjectPageList(String userId, String userType, String evaluationStatus, String searchKey, String startTime, String endTime, int pageNo, int pageSize) {
		
		 Page<TenderProjectVo> page = new Page<>(pageNo, pageSize);
	        return tenderProjectService.getEvaluationProjectPageList(page, userId, userType, evaluationStatus, searchKey, DateUtils.toDate(startTime), DateUtils.toDate(endTime));
	 }
//	@PostMapping("/getEvaluationProjectPageList")
//	public IPage<TenderProjectVo> getEvaluationProjectPageList(@RequestBody RequestBodyEntity<AuthUserEntity> requestBodyEntity) {
//		AuthUserEntity user = requestBodyEntity.getEntity(); 
//		String useSatus = (String) requestBodyEntity.getParam("useSatus"); 
//		String searchKey = (String) requestBodyEntity.getParam("searchKey"); 
//		int pageNo = (int) requestBodyEntity.getParam("pageNo");
//		int pageSize = (int) requestBodyEntity.getParam("pageSize");
//		Page<TenderProjectVo> page = new Page<>(pageNo,pageSize);
//		return tenderProjectService.getEvaluationProjectPageList(page, user.getId(), null != user.getAuthRole()? user.getAuthRole().getRoleType() : UserEnum.ROLE_TYPE_EXPERT, useSatus,searchKey);
//	}
	
	/**
	 * @Description: 获得开标项目分页列表
	 * @author guoyr
	 * @param page
	 * @param userId
	 * @param userType
	 * @param openingStatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @return 
	 */
	@GetMapping("/getOpeningProjectPageList")
	public IPage<TenderProjectVo> getOpeningProjectPageList(String userId, String userType, String openingStatus, String searchKey, String startTime, String endTime, int pageNo, int pageSize) {
		Page<TenderProjectVo> page = new Page<>(pageNo, pageSize);
		return tenderProjectService.getOpeningProjectPageList(page, userId, userType, openingStatus, searchKey, DateUtils.toDate(startTime), DateUtils.toDate(endTime));
	}
	
	/**
	 * @Description: 获取开标项目的标段分页列表
	 * @author guoyr
	 * @param page
	 * @param parentId
	 * @param userType
	 * @param openingStatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @return 
	 */
	@GetMapping("/getOpeningTenderPackPageList")
	public IPage<TenderProjectVo> getOpeningTenderPackPageList(String parentId, String userType, String openingStatus, String searchKey, String startTime, String endTime, int pageNo, int pageSize) {
		Page<TenderProjectVo> page = new Page<>(pageNo, pageSize);
		return tenderProjectService.getOpeningTenderPackPageList(page, parentId, userType, openingStatus, searchKey, DateUtils.toDate(startTime), DateUtils.toDate(endTime));
	}
	/**
	 * @Description: 获取评标项目的标段分页列表
	 * @author guoyr
	 * @param page
	 * @param parentId
	 * @param userType
	 * @param evaluationStatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @return 
	 */
	@GetMapping("/getTenderPackPageList")
	public IPage<TenderProjectVo> getTenderPackPageList(String parentId, String userType, String userId, String openingStatus, String evaluationStatus, String searchKey, String startTime, String endTime, int pageNo, int pageSize) {
		Page<TenderProjectVo> page = new Page<>(pageNo, pageSize);
		return tenderProjectService.getTenderPackPageList(page, parentId, userType, userId, openingStatus, evaluationStatus, searchKey, DateUtils.toDate(startTime), DateUtils.toDate(endTime));
	}
	/**
	 * @Description: 获得我项目分页列表（项目经理）
	 * @param userId
	 * @param userType
	 * @param useSatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/getMyProjectPageList")
    public IPage<TenderProjectVo> getMyProjectPageList(String userId, String userType, String evaluationStatus, String searchKey, String startTime, String endTime, int pageNo, int pageSize) {
        Page<TenderProjectVo> page = new Page<>(pageNo, pageSize);
        return tenderProjectService.getMyProjectPageList(page, userId, userType, evaluationStatus, searchKey, DateUtils.toDate(startTime), DateUtils.toDate(endTime));
    }
	/**
	 * @Description: 获得我项目分页列表（项目经理）
	 * @param userId
	 * @param userType
	 * @param useSatus
	 * @param searchKey
	 * @param startTime
	 * @param endTime
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@GetMapping("/updateEvaluationComplateStatus")
	public boolean updateEvaluationComplateStatus(String tenderId, String parentId, String archiveStatus, String  date) {
		return tenderProjectService.updateEvaluationComplateStatus(tenderId, parentId, archiveStatus, DateUtils.toDate(date));
	}
	
	 /**   
	 * @Title: getSuperviseProjectPage   
	 * @Description: 按分页查询监督人监督项目
	 * @param: @param userId
	 * @param: @param useSatus
	 * @param: @param searchKey
	 * @param: @param pageNo
	 * @param: @param pageSize
	 * @param: @return      
	 * @return: IPage<TenderProjectVo>      
	 * @throws   
	 */  
	@GetMapping("/getSuperviseProjectPageList")
	 public IPage<TenderProjectVo> getSuperviseProjectPage(String userId, String evaluationStatus, String searchKey, String startTime, String endTime, int pageNo, int pageSize) {
		 Page<TenderProjectVo> page = new Page<>(pageNo,pageSize);
		 return tenderProjectService.selectSuperviseProjectPage(page, userId, evaluationStatus, searchKey, DateUtils.toDate(startTime), DateUtils.toDate(endTime));				 
	 }
	
	/**
	 * @Description: 按分页查询评标项目的分包
	 * @author mazc
	 * @param userId
	 * @param projectId
	 * @param useSatus
	 * @param pageNo
	 * @param pageSize
	 * @return 
	 */
	@GetMapping("/getExpertEvaluationTenderPackPageList")
	 public IPage<TenderProjectVo> getExpertEvaluationTenderPackPageList(String userId, String projectId, String evaluationStatus, int pageNo,
			 int pageSize){
		 Page<TenderProjectVo> page = new Page<>(pageNo,pageSize);
		 return tenderProjectService.getExpertEvaluationTenderPackPageList(page, userId, projectId, evaluationStatus);
	 }
	
	
	 /**   
	 * @Title: geUserInfo   
	 * @Description: 通过项目ID查询项目详细信息
	 * @param: @param TENDER_ID
	 * @param: @return      
	 * @return: TenderProject      
	 * @throws   
	 */  
	@GetMapping("/getTenderProjectInfo")
	public TenderProject geUserInfo(String tenderId) {
		return tenderProjectService.getById(tenderId);
	}
	
	
	 
	
	@GetMapping("/isUnique")
	public boolean isUniqueTenderProject(String id, String useStatus, String tenderNO, String phaseType, String parentId) {
		return tenderProjectService.isUniqueTenderProject(id, useStatus, tenderNO, phaseType, parentId);
	}
	
	@GetMapping("/getSubPackIdList")
	public List<String> getTenderIdList(String parentId) {
		return tenderProjectService.getTenderIdList(parentId);	
	}
	
	 /**   
	 * @Title: deleteTenderProjectId   
	 * @Description: 删除项目、删除id是同时删除下面的子项目
	 * @param: @param id
	 * @param: @return      
	 * @return: boolean      
	 * @throws   
	 */  
	@GetMapping("/deleteProject")
	@SysLogAnnotation(detail="删除项目",level = 5,operationType = OperationType.DELETE)
	public boolean deleteTenderProjectId(String id) {
		LambdaQueryWrapper<TenderProject> query = Wrappers.lambdaQuery();
		query.in(TenderProject::getId,id).or().in(TenderProject::getParentId, id);
		
		List<TenderProject> tenderList = tenderProjectService.list(query);
		if(null != tenderList) {
			for(TenderProject tenderProject : tenderList) {
				tenderProjectService.deleteDetailByTenderId(tenderProject.getId());
			}
		}else {
			return false;
		}
		return true;
	}
	
	
	/**
	 * @Description: 验证项目是否已经进入指定的评审环节
	 * @author mazc
	 * @param tenderId
	 * @param stepType
	 * @return 
	 */
	@GetMapping("/isInEvaluation")
	public boolean isInEvaluation(String tenderId, String stepType) {
		return tenderProjectService.isInEvaluation(tenderId, stepType);
	}
	
	/**
	 * @Description: 获取标段价格指标信息
	 * @author mazc
	 * @param tenderParentId
	 * @param auditType
	 * @return 
	 */
	@GetMapping("/getTenderPackPriceFactorList")
	public List<TenderPackPriceFactorVo> selectTenderPackPriceFactor(String tenderParentId,String auditType) {
		return tenderProjectService.selectTenderPackPriceFactor(tenderParentId, auditType);
	}
	
	
	/**
	 * @Description: 下载  项目更新操作
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@PostMapping("/updateDownLoadProject")
	@SysLogAnnotation(detail="下载 项目更新操作",level = 0,operationType = OperationType.UPDATE)
    public boolean updateDownLoadProject(@RequestBody RequestBodyEntity<Object> requestBodyEntity) {
        @SuppressWarnings("unchecked")
        TenderProject tenderProject = EntityUtill.toBean((Map<String, Object>) requestBodyEntity.getParam("tenderProjectEntity"), TenderProject.class);
        @SuppressWarnings("unchecked")
        ProjectRule projectRule = EntityUtill.toBean((Map<String, Object>) requestBodyEntity.getParam("tenderProjectRuleEntity"), ProjectRule.class);
        return tenderProjectService.updateDownLoadProject(tenderProject, projectRule);
    }
	
	
	
	/**
	 * @Description: 通过Tenderid 获取开标状态
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/getBidOpeningStatusById")
    public String findBidOpeningStatusById(String tenderId) {
        QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<>();
        queryWrapper.select(TenderProject.BID_OPENING_STATUS).lambda().eq(TenderProject::getId, tenderId);
        return tenderProjectService.getObj(queryWrapper, String::valueOf);
    }
	
	/**
	 * @Description: 获取开标不足三家供应商的标段
	 * @author mazc
	 * @param parentId
	 * @return 
	 */
	@GetMapping("/getUnEnoughSupplierList")
	public List<TenderProject> findUnEnoughSupplierList(String parentId){
		return tenderProjectService.findUnEnoughSupplierList(parentId);
	}
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param tenderId
	 * @param columns
	 * @return 
	 */
	@GetMapping("/getPackageByTenderIdOrderbyAsc")
	public List<TenderProject> findPackageByTenderId(String tenderId, String[] columns){
		QueryWrapper<TenderProject> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(TenderProject::getId, tenderId);
		queryWrapper.orderByAsc(columns);
		return tenderProjectService.list(queryWrapper);
	}
	
	/**
	 * @Description: 保存或更新
	 * @author mazc
	 * @return 
	 */
	@PostMapping("/saveOrUpdate")
	@SysLogAnnotation(detail="保存或更新项目信息",level = 0,operationType = OperationType.UPDATE)
	public boolean saveOrUpdate(@RequestBody TenderProject tenderProjectEntity){
		if(StringUtils.isEmpty(tenderProjectEntity.getId())) {
			tenderProjectEntity.setId(IDUtils.getId());
			return tenderProjectService.save(tenderProjectEntity);
		}else {
			return tenderProjectService.updateById(tenderProjectEntity);
		}
	}
	
	/**
	 * @Description: 获得专家的评审项目列表
	 * @author mazc
	 * @param userId
	 * @param searchKey
	 * @param macName
	 * @return 
	 */
	@GetMapping("/getExpertTenderProjectList")
	public List<TenderProject> getExpertTenderProjectList(String userId, String searchKey, String macName){
		return tenderProjectService.getExpertTenderProjectList(userId, searchKey, macName);
	}
	
	
	/**
	 * @Description: 查找和当前品目相同的相同分包
	 * @author mazc
	 * @param parentId
	 * @param purcategoryNames
	 * @param id
	 * @return 
	 */
	@GetMapping("/getPurcategoryNamesAndParentId")
	public List<TenderProject> getPurcategoryNamesAndParentId(String parentId,String purcategoryNames,String id){
		return tenderProjectService.getPurcategoryNamesAndParentId(parentId, purcategoryNames, id);
	}
	
	/**
	 * @Description: 修改开标状态
	 * @author mazc
	 * @param tenderId
	 * @param parentId
	 * @param bidOpeningStatus
	 * @return 
	 */
	@GetMapping("/updateBidOpeningStatus")
	@SysLogAnnotation(detail="修改开标状态",level = 0,operationType = OperationType.UPDATE)
	public int updateBidopeningStatus(String tenderId, String parentId, String bidOpeningStatus) {
		return tenderProjectService.updateBidOpeningStatus(tenderId, parentId, bidOpeningStatus);
	}
	
	/**
	 * @Description: 修改开标解密状态
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@PostMapping("/updateBidOpeningDecodeStatus")
	@SysLogAnnotation(detail="修改开标解密状态",level = 0,operationType = OperationType.UPDATE)
	public int updateBidOpeningStatus(@RequestBody RequestBodyEntity<TenderProject> requestBodyEntity){
		TenderProject tenderProject = requestBodyEntity.getEntity();
		String bidopeningStage = (String)requestBodyEntity.getParam("bidopeningStage");
		String status = (String)requestBodyEntity.getParam("status");
		return tenderProjectService.updateBidOpeningStatus(tenderProject, bidopeningStage, status);
	}
	
	
	/**
	 * @Description: 获得项目下的所有包件的开标信息
	 * @author mazc
	 * @param projectId
	 * @param bidopeningStage
	 * @return 
	 */
	@GetMapping("/getTenderPackOpeningInfoList")
	public List<TenderPackOpeningInfo> getTenderPackOpeningInfoList(String projectId, String bidopeningStage){
		return tenderProjectService.getTenderPackOpeningInfoList(projectId, bidopeningStage);
	}
	
	/**
	 * @Description:  获得包件的开标信息
	 * @author mazc
	 * @param tenderId
	 * @param bidopeningStage
	 * @return 
	 */
	@GetMapping("/getTenderPackOpeningInfo")
	public TenderPackOpeningInfo getTenderPackOpeningInfo(String tenderId, String bidopeningStage){
		return tenderProjectService.getTenderPackOpeningInfo(tenderId, bidopeningStage);
	}
	/**
	 * @Description: 批量插入
	 * @author mazc
	 * @param tenderProjectList
	 */
	@PostMapping("/insertByBatch")
	public int insertByBatch(@RequestBody List<TenderProject> tenderProjectList) {
		return tenderProjectService.insertByBatch(tenderProjectList);
	}
	/**
	 * @Description: 批量更新
	 * @author mazc
	 * @param tenderProjectList
	 */
	@PostMapping("/batchUpdate")
	public int updateByBatch(@RequestBody List<TenderProject> tenderProjectList) {
		return tenderProjectService.updateByBatch(tenderProjectList);
	}
	
	@PostMapping("/batchUpdateProperty")
	@SysLogAnnotation(detail="批量指定字段修改项目信息",level = 0,operationType = OperationType.UPDATE)
	public boolean batchUpdateProperty(@RequestBody BatchUpdatePropertyVo<TenderProject> vo) {
		return tenderProjectService.batchUpdateProperty(vo.getEntityList(), vo.getAttributes()) > 0;
	}
	
	/**
	 * @Description: 获得项目或标包的开标信息
	 * @author mazc
	 * @param requestBodyEntity
	 * @return 
	 */
	@PostMapping("/getTenderProjectOpeningInfo")
	public TenderPackOpeningInfo getTenderProjectOpeningInfo(@RequestBody RequestBodyEntity<TenderProject> requestBodyEntity){
		TenderProject tenderProject = requestBodyEntity.getEntity();
		String bidopeningStage = (String) requestBodyEntity.getParam("bidopeningStage");		
		return tenderProjectService.getTenderProjectOpeningInfo(tenderProject, bidopeningStage);
	}
	
	
	/**
	 * @Description: 获得包件的开标信息
	 * @author mazc
	 * @param projectId
	 * @return 
	 */
	@GetMapping("/getTenderPackBidOpeningInfoList")
	public List<TenderProject> getTenderPackBidOpeningInfoList(String projectId) {
		return tenderProjectService.getTenderPackBidOpeningInfoList(projectId);
	}
	
	/**
	 * @Description: 获得分包项目
	 * @author mazc
	 * @param parentId
	 * @return 
	 */
	@GetMapping("/getSubPackageList")
	public List<TenderProject> getSubPackageList(String parentId){
		return tenderProjectService.getSubPackageList(parentId);
	}
	
	/** 
	 * @Description: 是否获取了最新的投标信息
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/isGetLatestBidinfo")
	public boolean isGetLatestBidinfo(String tenderId) {
		return tenderProjectService.isGetLatestBidinfo(tenderId);
	}
	
	/** 
	 * @Description: 是否完成开标
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/isBidOpeningComplate")
	public boolean isBidOpeningComplate(String tenderId) {
		return tenderProjectService.isBidOpeningComplate(tenderId);
	}
	
	/**
	 * @Description: 评标是否结束
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@GetMapping("/isEvaluationComplate")
	public boolean isEvaluationComplate (String tenderId) {
		return tenderProjectService.isEvaluationComplate(tenderId);
	}
	
	/**
	 * @Description:  通过 tenderNo、phaseType、tenderNumber 查询项目信息
	 * @author mazc
	 * @param tenderNo
	 * @param phaseType
	 * @param tenderNumber
	 * @return 
	 */
	@GetMapping("/getTenderProjectByPhaseTypeTenderNumber")
	public TenderProject getTenderProjectByPhaseTypeTenderNumber(String tenderNo ,String phaseType,String tenderNumber) {
		return tenderProjectService.getTenderProjectByPhaseTypeTenderNumber(tenderNo, phaseType, tenderNumber);
	}
	
	/**
	 * @Description:  根据项目tenderNo和父ID，获得项目ID。
	 * @author mazc
	 * @param tenderNo
	 * @param parentId
	 * @return 
	 */
	@GetMapping("/getTenderIdListByTenderNoAndparentId")
	public List<String> getTenderIdListByTenderNoAndparentId(String tenderNo,String parentId){
		return tenderProjectService.queryTenderIdListByTenderNoAndparentId(tenderNo, parentId);
	}
	
	/**
	 * @Description: 导入开标结果
	 * @author mazc
	 * @param importBidOpenResultVo
	 * @return 
	 */
	@PostMapping("/importBidOpenResult")
	@SysLogAnnotation(detail="导入开标结果",level = 0,operationType = OperationType.UPDATE)
	public boolean importBidOpenResult(@RequestBody ImportBidOpenResultVo importBidOpenResultVo) {
		return 	tenderProjectService.importBidOpenResult(importBidOpenResultVo.getTenderProject(), importBidOpenResultVo.getSupplierList(),
				importBidOpenResultVo.getBidPriceList(),importBidOpenResultVo.getBidOpeningStage());
	}
	
	/**
	 * @Description: 下载项目操作保存方法
	 * @author mazc
	 * @param saveDownLoadProjectVo
	 * @return 
	 */
	@PostMapping("/saveDownLoadProject")
	@SysLogAnnotation(detail="保存下载项目",level = 0,operationType = OperationType.INSERT)
	public Result<TenderProject> saveDownLoadProject(@RequestBody SaveDownLoadProjectVo saveDownLoadProjectVo) {
		TenderProject tenderProject = tenderProjectService.saveDownLoadProject(saveDownLoadProjectVo.getTenderProject(), saveDownLoadProjectVo.getChildTenderProject(),
				saveDownLoadProjectVo.getParentRule(), saveDownLoadProjectVo.getChildRule());
		if(null == tenderProject) {
			return Result.failed(null, "下载项目失败");
		}
		return Result.ok(tenderProject);
	}

	/**
	 * @Description: 删除开标结果
	 * @author mazc
	 * @param id
	 * @return 
	 */
	@GetMapping("/deleteBidOpeningResult")
	@SysLogAnnotation(detail="删除开标结果",level = 0,operationType = OperationType.DELETE)
	public int deleteBidOpeningResult(String id) {
		return tenderProjectService.deleteBidOpeningResult(id);
	}
	
	@GetMapping("/getTenderProjectList")
	public List<TenderProject> getTenderProjectList(String userId){
		return tenderProjectService.getTenderProjectList(userId);
	}
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param projectId
	 * @param withoutTenderId
	 * @return 
	 */
	@GetMapping("/getTenderPackEvaluationInfoList")
	public List<Map<String, Object>> getTenderPackEvaluationInfoList(String projectId, String withoutTenderId){
		return tenderProjectService.getTenderPackEvaluationInfoList(projectId, withoutTenderId);
	}
	
	
	/**
	 * @Description: 获得可生成报表的包件信息
	 * @author mazc
	 * @param projectId
	 * @param withoutTenderId
	 * @return 
	 */
	@GetMapping("/getReportTenderPackInfoList")
	public List<Map<String, Object>> getReportTenderPackInfoList(String projectId, String withoutTenderId){
		return tenderProjectService.getReportTenderPackInfoList(projectId, withoutTenderId);
	}
	
	
	/**
	 * @Description: 获得当前项目下所有标段专家信息
	 * @author mazc
	 * @param projectId
	 * @param withoutTenderId
	 * @return 
	 */
	@GetMapping("/getTenderPackExpertInfoList")
	public List<Map<String, Object>> getTenderPackExpertInfoList(String projectId, String withoutTenderId){
		return tenderProjectService.getTenderPackExpertInfoList(projectId,withoutTenderId);
	}
	
	/**
	 * @Description: 查询所有有评审专家的项目
	 * @author mazc
	 * @return 
	 */
	@GetMapping("/getTenderProjectByExpert")
	public List<TenderProject> getTenderProjectByExpert() {
		return tenderProjectService.getTenderProjectByExpert();
	}
	
	/**
	 * @Description: 重新开始评审
	 * @author mazc
	 * @param tenderId
	 * @param cleanData 
	 */
	@GetMapping("/restartEvaluation")
	@SysLogAnnotation(detail="设置项目重新评审",level = 0,operationType = OperationType.DELETE)
	public void restartEvaluation(String tenderId, boolean cleanData){
		tenderProjectService.restartEvaluation(tenderId, cleanData);
	}
	
	@GetMapping("/updateServerContent")
	public int updateServerContent(String serverId ,String serverAccount,String serverPassword,String parentId){
		return tenderProjectService.updateServerContent(serverId, serverAccount, serverPassword, parentId);
	}
	
	/**
	 * @Description 验证是否存在
	 * @author mazc
	 * @param tenderNo
	 * @param parentId
	 * @param phaseType
	 * @param tenderNumber
	 * @return 
	 */
	@GetMapping("/valProjectExist")
	public TenderProject valProjectExist(String tenderNo, String parentId, String phaseType, String tenderNumber) {
		return tenderProjectService.valProjectExist(tenderNo, parentId, phaseType, tenderNumber);
	}
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param tenderId
	 * @param status
	 * @return 
	 */
	@GetMapping("/updateEvaluateStatus")
	@SysLogAnnotation(detail="修改评审进度状态",level = 0,operationType = OperationType.DELETE)
	public int updateEvaluateStatus(String tenderId, String status) {
		return tenderProjectService.updateEvaluateStatus(tenderId, status);
	}
	
	/**
	 * @Description: 验证项目是否已经完成指定阶段的开标
	 * @param tenderId
	 * @param bidopeningStage
	 * @return
	 */
	@GetMapping("/isComplateProjectOpeningByStage")
	public boolean isComplateProjectOpeningByStage(String projectId, String bidopeningStage) {
		return tenderProjectService.isComplateProjectOpeningByStage(projectId, bidopeningStage);
	}
	
	/**
	 * @Description: 
	 * @author mazc
	 * @param tenderCode	项目编号
	 * @param packCode	包件编号
	 * @param stageType 阶段类型
	 * @param tenderNumber 项目流水号
	 * @param stageId 阶段编号
	 * @return 
	 */
	@PostMapping("/deleteProjectInfo")
	@SysLogAnnotation(detail="外部删除项目",level = 5,operationType = OperationType.DELETE)
	public ResultBody deleteProjectInfo(String tenderCode,String packCode,String stageType ,String tenderNumber,String stageId) {
		return tenderProjectService.deleteProjectInfo( tenderCode, packCode, stageType , tenderNumber, stageId);
	}

	 /**
	  * @Description: 评标是否结束
	  * @author mazc
	  * @param tenderId
	  * @return
	  */
	 @GetMapping("/isUploadEvalResult")
	 public boolean isUploadEvalResutl (String tenderId) {
		 return tenderProjectService.isUploadEvalResult(tenderId);
	 }

	 @PostMapping({"/copyTenderProject"})
	 public String copyTenderProject(@RequestParam("tenderNo") String tenderNo, @RequestParam("stageId") String stageId, @RequestParam("tenderNonumber") String tenderNonumber, @RequestParam("username") String username, @RequestParam("password") String password, @RequestParam("copyStatus") String copyStatus) {
		 return this.tenderProjectService.copyProject(tenderNo, stageId, tenderNonumber, username, password, copyStatus);
	 }

}