package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeCashSheetMapper;
import com.gx.obe.server.management.struct_new.entity.ObeCashSheet;
import com.gx.obe.server.management.struct_new.service.ObeCashSheetService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeCashSheetServiceImpl extends ServiceImpl<ObeCashSheetMapper, ObeCashSheet> implements ObeCashSheetService {

    @Autowired
    private ObeCashSheetMapper obeCashSheetMapper;

    /**
     * @param ObeCashSheetList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeCashSheet> ObeCashSheetList) {
        return obeCashSheetMapper.updateBatchList(ObeCashSheetList);
    }

    /**
     * @param ObeCashSheetList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeCashSheet> ObeCashSheetList) {
        return obeCashSheetMapper.insertByBatch(ObeCashSheetList);
    }

    /**
     * @param ObeCashSheet
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeCashSheet ObeCashSheet, String[] attributes) {
        return obeCashSheetMapper.updateAssignProperty(ObeCashSheet, attributes) > 0;
    }

    /**
     * @param ObeCashSheetList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeCashSheet> ObeCashSheetList, String[] attributes) {
        return obeCashSheetMapper.batchUpdateProperty(ObeCashSheetList, attributes);
    }

    /**
     * @param ObeCashSheetList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeCashSheet> ObeCashSheetList, String[] attributes) {
        List<ObeCashSheet> insertObeCashSheetList = new ArrayList<>();
        List<ObeCashSheet> updateObeCashSheetList = new ArrayList<>();
        for (ObeCashSheet ObeCashSheet : ObeCashSheetList) {
            if (StringUtils.isEmpty(ObeCashSheet.getId())) {
                ObeCashSheet.setId(IDUtils.getId());
                insertObeCashSheetList.add(ObeCashSheet);
            } else {
                updateObeCashSheetList.add(ObeCashSheet);
            }
        }
        int count = 0;
        if (insertObeCashSheetList.size() > 0) {
            count += obeCashSheetMapper.insertByBatch(insertObeCashSheetList);
        }
        if (updateObeCashSheetList.size() > 0) {
            count += obeCashSheetMapper.batchUpdateProperty(updateObeCashSheetList, attributes);
        }
        return count;
    }

}
