package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.CollectionUtils;
import com.gx.obe.server.common.utils.ExceptionUtil;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.*;
import com.gx.obe.server.management.struct_new.entity.*;
import com.gx.obe.server.management.struct_new.service.ObeEvaluationContentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author chenxw
 * @Description:
 */
@Service
@Transactional
public class ObeEvaluationContentServiceImpl extends ServiceImpl<ObeEvaluationContentMapper, ObeEvaluationContent> implements ObeEvaluationContentService {

    @Autowired
    private ObeEvaluationContentMapper obeEvaluationContentMapper;

    @Autowired
    private ObeTemplateTableMapper obeTemplateTableMapper;

    @Autowired
    private ObeModelDataMapper obeModelDataMapper;

    @Autowired
    private ObeBusinessLicenseMapper obeBusinessLicenseMapper;

    @Autowired
    private ObeFinanceMapper obeFinanceMapper;

    @Autowired
    private ObeBalanceSheetMapper obeBalanceSheetMapper;

    @Autowired
    private ObeProfitlossSheetMapper obeProfitlossSheetMapper;

    @Autowired
    private ObeCashSheetMapper obeCashSheetMapper;

    @Autowired
    private ObePerformanceMapper obePerformanceMapper;

    @Autowired
    private ObeBidderBasicInfoMapper obeBidderBasicInfoMapper;

    @Autowired
    private ObeQualificationMapper obeQualificationMapper;

    @Autowired
    private ObeProjectLeaderMapper obeProjectLeaderMapper;

    @Autowired
    private ObeCertificateMapper obeCertificateMapper;

    @Autowired
    private ObeTemplateDataItemMapper obeTemplateDataItemMapper;

    @Autowired
    private ObeAttachmentFileMapper obeAttachmentFileMapper;

    @Autowired
    private ObeWorkExperienceMapper obeWorkExperienceMapper;

    /**
     * @param ObeEvaluationContentList
     * @Description: 批量更新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeEvaluationContent> ObeEvaluationContentList) {
        return obeEvaluationContentMapper.updateBatchList(ObeEvaluationContentList);
    }

    /**
     * @param ObeEvaluationContentList
     * @Description: 批量插入
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeEvaluationContent> ObeEvaluationContentList) {
        return obeEvaluationContentMapper.insertByBatch(ObeEvaluationContentList);
    }

    /**
     * @param ObeEvaluationContent
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeEvaluationContent ObeEvaluationContent, String[] attributes) {
        return obeEvaluationContentMapper.updateAssignProperty(ObeEvaluationContent, attributes) > 0;
    }

    /**
     * @param ObeEvaluationContentList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeEvaluationContent> ObeEvaluationContentList, String[] attributes) {
        return obeEvaluationContentMapper.batchUpdateProperty(ObeEvaluationContentList, attributes);
    }

    /**
     * @param ObeEvaluationContentList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeEvaluationContent> ObeEvaluationContentList, String[] attributes) {
        List<ObeEvaluationContent> insertObeEvaluationContentList = new ArrayList<>();
        List<ObeEvaluationContent> updateObeEvaluationContentList = new ArrayList<>();
        for (ObeEvaluationContent ObeEvaluationContent : ObeEvaluationContentList) {
            if (StringUtils.isEmpty(ObeEvaluationContent.getId())) {
                ObeEvaluationContent.setId(IDUtils.getId());
                insertObeEvaluationContentList.add(ObeEvaluationContent);
            } else {
                updateObeEvaluationContentList.add(ObeEvaluationContent);
            }
        }
        int count = 0;
        if (insertObeEvaluationContentList.size() > 0) {
            count += obeEvaluationContentMapper.insertByBatch(insertObeEvaluationContentList);
        }
        if (updateObeEvaluationContentList.size() > 0) {
            count += obeEvaluationContentMapper.batchUpdateProperty(updateObeEvaluationContentList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @return
     * @Description: 获取评审内容
     * @author chenxw
     */
    @Override
    public List<ObeEvaluationContent> getEvaluationContentList(String tenderId) {
        QueryWrapper<ObeEvaluationContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeEvaluationContent::getTenderId, tenderId);
        return obeEvaluationContentMapper.selectList(queryWrapper);
    }

    /**
     * @param tenderId
     * @param factorCode
     * @return
     * @Description: 根据指标编号获取评审内容列表
     * @author chenxw
     */
    @Override
    public List<ObeEvaluationContent> getEvaluationContentListByFactorCode(String tenderId, String factorCode) {
        QueryWrapper<ObeEvaluationContent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeEvaluationContent::getTenderId, tenderId);
        queryWrapper.lambda().eq(ObeEvaluationContent::getFactorCode, factorCode);
        return obeEvaluationContentMapper.selectList(queryWrapper);
    }

    /**
     * @param structDateInfo
     * @Description: 保存结构化信息数据
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveStructDateInfo(StructDateInfo structDateInfo) {
        List<ObeTemplateTable> templateTableList = structDateInfo.getTemplateTableList();
        if (CollectionUtils.isNotNull(templateTableList))
            ExceptionUtil.equal(obeTemplateTableMapper.insertByBatch(templateTableList), templateTableList.size());

        List<ObeModelData> modelDataList = structDateInfo.getModelDataList();
        if (CollectionUtils.isNotNull(modelDataList))
            ExceptionUtil.equal(obeModelDataMapper.insertByBatch(modelDataList), modelDataList.size());

        List<ObeBusinessLicense> businessLicenseList = structDateInfo.getBusinessLicenseList();
        if (CollectionUtils.isNotNull(businessLicenseList))
            ExceptionUtil.equal(obeBusinessLicenseMapper.insertByBatch(businessLicenseList), businessLicenseList.size());

        List<ObeFinance> financeList = structDateInfo.getFinanceList();
        if (CollectionUtils.isNotNull(financeList)) {
            ExceptionUtil.equal(obeFinanceMapper.insertByBatch(financeList), financeList.size());
            insertByBatch(financeList, ObeFinance::getBalanceSheet, obeBalanceSheetMapper::insertByBatch);
            insertByBatch(financeList, ObeFinance::getProfitlossSheet, obeProfitlossSheetMapper::insertByBatch);
            insertByBatch(financeList, ObeFinance::getCashSheet, obeCashSheetMapper::insertByBatch);
        }

        List<ObePerformance> performanceList = structDateInfo.getPerformanceList();
        if (CollectionUtils.isNotNull(performanceList))
            ExceptionUtil.equal(obePerformanceMapper.insertByBatch(performanceList), performanceList.size());

        List<ObeBidderBasicInfo> bidderBasicInfoList = structDateInfo.getBidderBasicInfoList();
        if (CollectionUtils.isNotNull(bidderBasicInfoList))
            ExceptionUtil.equal(obeBidderBasicInfoMapper.insertByBatch(bidderBasicInfoList), bidderBasicInfoList.size());

        List<ObeQualification> qualificationList = structDateInfo.getQualificationList();
        if (CollectionUtils.isNotNull(qualificationList))
            ExceptionUtil.equal(obeQualificationMapper.insertByBatch(qualificationList), qualificationList.size());

        List<ObeProjectLeader> projectLeaderList = structDateInfo.getProjectLeaderList();
        if (CollectionUtils.isNotNull(projectLeaderList)) {
            ExceptionUtil.equal(obeProjectLeaderMapper.insertByBatch(projectLeaderList), projectLeaderList.size());
            insertListByBatch(projectLeaderList, ObeProjectLeader::getCertificateList, obeCertificateMapper::insertByBatch);
            insertListByBatch(projectLeaderList, ObeProjectLeader::getWorkExperienceList, obeWorkExperienceMapper::insertByBatch);
        }

        List<ObeTemplateDataItem> templateDataItemList = structDateInfo.getTemplateDataItemList();
        if (CollectionUtils.isNotNull(templateDataItemList))
            ExceptionUtil.equal(obeTemplateDataItemMapper.insertByBatch(templateDataItemList), templateDataItemList.size());

        List<ObeAttachmentFile> attachmentFileList = structDateInfo.getAttachmentFileList();
        if (CollectionUtils.isNotNull(attachmentFileList))
            ExceptionUtil.equal(obeAttachmentFileMapper.insertByBatch(attachmentFileList), attachmentFileList.size());
        return true;
    }

    private <T, R> void insertByBatch(List<T> list, Function<T, R> trFunction, Function<List<R>, Integer> listIntegerFunction) {
        List<R> rList = list.stream().map(trFunction).filter(Objects::nonNull).collect(Collectors.toList());
        if (!rList.isEmpty())
            ExceptionUtil.equal(listIntegerFunction.apply(rList), rList.size());
    }

    private <T, R> void insertListByBatch(List<T> list, Function<T, List<R>> tListFunction, Function<List<R>, Integer> listIntegerFunction) {
        List<R> rList = list.stream().map(tListFunction).filter(Objects::nonNull).flatMap(List::stream).collect(Collectors.toList());
        if (!rList.isEmpty())
            ExceptionUtil.equal(listIntegerFunction.apply(rList), rList.size());
    }

    /**
     * @param tenderId
     * @return
     * @Description: 清空结构化信息数据
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteStructDateInfo(String tenderId) {
        QueryWrapper<ObeTemplateTable> templateTableQueryWrapper = new QueryWrapper<>();
        templateTableQueryWrapper.lambda().eq(ObeTemplateTable::getTenderId, tenderId);
        obeTemplateTableMapper.delete(templateTableQueryWrapper);

        QueryWrapper<ObeModelData> modelDataQueryWrapper = new QueryWrapper<>();
        modelDataQueryWrapper.lambda().eq(ObeModelData::getTenderId, tenderId);
        obeModelDataMapper.delete(modelDataQueryWrapper);

        QueryWrapper<ObeBusinessLicense> businessLicenseQueryWrapper = new QueryWrapper<>();
        businessLicenseQueryWrapper.lambda().eq(ObeBusinessLicense::getTenderId, tenderId);
        obeBusinessLicenseMapper.delete(businessLicenseQueryWrapper);

        QueryWrapper<ObeFinance> financeQueryWrapper = new QueryWrapper<>();
        financeQueryWrapper.lambda().eq(ObeFinance::getTenderId, tenderId);
        obeFinanceMapper.delete(financeQueryWrapper);

        QueryWrapper<ObeBalanceSheet> balanceSheetQueryWrapper = new QueryWrapper<>();
        balanceSheetQueryWrapper.lambda().eq(ObeBalanceSheet::getTenderId, tenderId);
        obeBalanceSheetMapper.delete(balanceSheetQueryWrapper);

        QueryWrapper<ObeProfitlossSheet> profitlossSheetQueryWrapper = new QueryWrapper<>();
        profitlossSheetQueryWrapper.lambda().eq(ObeProfitlossSheet::getTenderId, tenderId);
        obeProfitlossSheetMapper.delete(profitlossSheetQueryWrapper);

        QueryWrapper<ObeCashSheet> cashSheetQueryWrapper = new QueryWrapper<>();
        cashSheetQueryWrapper.lambda().eq(ObeCashSheet::getTenderId, tenderId);
        obeCashSheetMapper.delete(cashSheetQueryWrapper);

        QueryWrapper<ObePerformance> performanceQueryWrapper = new QueryWrapper<>();
        performanceQueryWrapper.lambda().eq(ObePerformance::getTenderId, tenderId);
        obePerformanceMapper.delete(performanceQueryWrapper);

        QueryWrapper<ObeBidderBasicInfo> bidderBasicInfoQueryWrapper = new QueryWrapper<>();
        bidderBasicInfoQueryWrapper.lambda().eq(ObeBidderBasicInfo::getTenderId, tenderId);
        obeBidderBasicInfoMapper.delete(bidderBasicInfoQueryWrapper);

        QueryWrapper<ObeQualification> qualificationQueryWrapper = new QueryWrapper<>();
        qualificationQueryWrapper.lambda().eq(ObeQualification::getTenderId, tenderId);
        obeQualificationMapper.delete(qualificationQueryWrapper);

        QueryWrapper<ObeProjectLeader> projectLeaderQueryWrapper = new QueryWrapper<>();
        projectLeaderQueryWrapper.lambda().eq(ObeProjectLeader::getTenderId, tenderId);
        obeProjectLeaderMapper.delete(projectLeaderQueryWrapper);

        QueryWrapper<ObeTemplateDataItem> templateDataItemQueryWrapper = new QueryWrapper<>();
        templateDataItemQueryWrapper.lambda().eq(ObeTemplateDataItem::getTenderId, tenderId);
        obeTemplateDataItemMapper.delete(templateDataItemQueryWrapper);

        QueryWrapper<ObeAttachmentFile> attachmentFileQueryWrapper = new QueryWrapper<>();
        attachmentFileQueryWrapper.lambda().eq(ObeAttachmentFile::getTenderId, tenderId);
        obeAttachmentFileMapper.delete(attachmentFileQueryWrapper);

        QueryWrapper<ObeCertificate> certificateQueryWrapper = new QueryWrapper<>();
        certificateQueryWrapper.lambda().eq(ObeCertificate::getTenderId, tenderId);
        obeCertificateMapper.delete(certificateQueryWrapper);

        QueryWrapper<ObeWorkExperience> workExperienceQueryWrapper = new QueryWrapper<>();
        workExperienceQueryWrapper.lambda().eq(ObeWorkExperience::getTenderId, tenderId);
        obeWorkExperienceMapper.delete(workExperienceQueryWrapper);
        return true;
    }

    /**
     * @param structDateInfo
     * @Description: 保存结构化信息数据
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteOrSaveStructDateInfo(String tenderId, StructDateInfo structDateInfo) {
        deleteStructDateInfo(tenderId);
        saveStructDateInfo(structDateInfo);
        return true;
    }

}
