package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeModelDataMapper;
import com.gx.obe.server.management.struct_new.entity.ObeModelData;
import com.gx.obe.server.management.struct_new.service.ObeModelDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeModelDataServiceImpl extends ServiceImpl<ObeModelDataMapper, ObeModelData> implements ObeModelDataService {

    @Autowired
    private ObeModelDataMapper obeModelDataMapper;

    /**
     * @param ObeModelDataList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeModelData> ObeModelDataList) {
        return obeModelDataMapper.updateBatchList(ObeModelDataList);
    }

    /**
     * @param ObeModelDataList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeModelData> ObeModelDataList) {
        return obeModelDataMapper.insertByBatch(ObeModelDataList);
    }

    /**
     * @param ObeModelData
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeModelData ObeModelData, String[] attributes) {
        return obeModelDataMapper.updateAssignProperty(ObeModelData, attributes) > 0;
    }

    /**
     * @param ObeModelDataList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeModelData> ObeModelDataList, String[] attributes) {
        return obeModelDataMapper.batchUpdateProperty(ObeModelDataList, attributes);
    }

    /**
     * @param ObeModelDataList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeModelData> ObeModelDataList, String[] attributes) {
        List<ObeModelData> insertObeModelDataList = new ArrayList<>();
        List<ObeModelData> updateObeModelDataList = new ArrayList<>();
        for (ObeModelData ObeModelData : ObeModelDataList) {
            if (StringUtils.isEmpty(ObeModelData.getId())) {
                ObeModelData.setId(IDUtils.getId());
                insertObeModelDataList.add(ObeModelData);
            } else {
                updateObeModelDataList.add(ObeModelData);
            }
        }
        int count = 0;
        if (insertObeModelDataList.size() > 0) {
            count += obeModelDataMapper.insertByBatch(insertObeModelDataList);
        }
        if (updateObeModelDataList.size() > 0) {
            count += obeModelDataMapper.batchUpdateProperty(updateObeModelDataList, attributes);
        }
        return count;
    }

    /**
     * @Description: 获取对象结构化数据
     * @author chenxw
     * @param tenderId
     * @param supplierId
     * @param relChapterType
     * @param dataCode
     * @return
     */
    public ObeModelData getModelData(String tenderId, String supplierId, String relChapterType, String dataCode) {
        QueryWrapper<ObeModelData> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeModelData::getTenderId, tenderId);
        queryWrapper.lambda().eq(ObeModelData::getSupplierId, supplierId);
        queryWrapper.lambda().eq(ObeModelData::getRelChapterType, relChapterType);
        queryWrapper.lambda().eq(ObeModelData::getDataCode, dataCode);
        return obeModelDataMapper.selectOne(queryWrapper);
    }

}
