package com.gx.obe.server.management.struct_new.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.struct_new.dao.ObeProjectLeaderMapper;
import com.gx.obe.server.management.struct_new.entity.ObeProjectLeader;
import com.gx.obe.server.management.struct_new.service.ObeCertificateService;
import com.gx.obe.server.management.struct_new.service.ObeProjectLeaderService;
import com.gx.obe.server.management.struct_new.service.ObeWorkExperienceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeProjectLeaderServiceImpl extends ServiceImpl<ObeProjectLeaderMapper, ObeProjectLeader> implements ObeProjectLeaderService {

    @Autowired
    private ObeProjectLeaderMapper obeProjectLeaderMapper;
    @Autowired
    private ObeCertificateService obeCertificateService;
    @Autowired
    private ObeWorkExperienceService obeWorkExperienceService;

    /**
     * @param ObeProjectLeaderList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeProjectLeader> ObeProjectLeaderList) {
        return obeProjectLeaderMapper.updateBatchList(ObeProjectLeaderList);
    }

    /**
     * @param ObeProjectLeaderList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeProjectLeader> ObeProjectLeaderList) {
        return obeProjectLeaderMapper.insertByBatch(ObeProjectLeaderList);
    }

    /**
     * @param ObeProjectLeader
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeProjectLeader ObeProjectLeader, String[] attributes) {
        return obeProjectLeaderMapper.updateAssignProperty(ObeProjectLeader, attributes) > 0;
    }

    /**
     * @param ObeProjectLeaderList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeProjectLeader> ObeProjectLeaderList, String[] attributes) {
        return obeProjectLeaderMapper.batchUpdateProperty(ObeProjectLeaderList, attributes);
    }

    /**
     * @param ObeProjectLeaderList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeProjectLeader> ObeProjectLeaderList, String[] attributes) {
        List<ObeProjectLeader> insertObeProjectLeaderList = new ArrayList<>();
        List<ObeProjectLeader> updateObeProjectLeaderList = new ArrayList<>();
        for (ObeProjectLeader ObeProjectLeader : ObeProjectLeaderList) {
            if (StringUtils.isEmpty(ObeProjectLeader.getId())) {
                ObeProjectLeader.setId(IDUtils.getId());
                insertObeProjectLeaderList.add(ObeProjectLeader);
            } else {
                updateObeProjectLeaderList.add(ObeProjectLeader);
            }
        }
        int count = 0;
        if (insertObeProjectLeaderList.size() > 0) {
            count += obeProjectLeaderMapper.insertByBatch(insertObeProjectLeaderList);
        }
        if (updateObeProjectLeaderList.size() > 0) {
            count += obeProjectLeaderMapper.batchUpdateProperty(updateObeProjectLeaderList, attributes);
        }
        return count;
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取项目负责人列表
     * @author chenxw
     */
    public List<ObeProjectLeader> getProjectLeaderList(String tenderId, String modelDataId) {
        QueryWrapper<ObeProjectLeader> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeProjectLeader::getModelDataId, modelDataId);
        return obeProjectLeaderMapper.selectList(queryWrapper);
    }

    /**
     * @param tenderId
     * @param modelDataId
     * @return
     * @Description: 获取项目负责人列表
     * @author chenxw
     */
    public List<ObeProjectLeader> getProjectLeaderListAll(String tenderId, String modelDataId) {
        QueryWrapper<ObeProjectLeader> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ObeProjectLeader::getModelDataId, modelDataId);
        List<ObeProjectLeader> obeProjectLeaders = obeProjectLeaderMapper.selectList(queryWrapper);
        obeProjectLeaders.forEach(t -> {
            t.setCertificateList(obeCertificateService.getCertificateList(t.getId()));
            t.setWorkExperienceList(obeWorkExperienceService.getWorkExperienceList(t.getId()));
        });
        return obeProjectLeaders;
    }

}
