package com.gx.obe.server.management.table.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.table.dao.ObeBidPriceTableItemMapper;
import com.gx.obe.server.management.table.entity.ObeBidPriceTableItem;
import com.gx.obe.server.management.table.service.ObeBidPriceTableItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeBidPriceTableItemServiceImpl extends ServiceImpl<ObeBidPriceTableItemMapper, ObeBidPriceTableItem> implements ObeBidPriceTableItemService {

    @Autowired
    private ObeBidPriceTableItemMapper obeBidPriceTableItemMapper;

    /**
     * @param ObeBidPriceTableItemList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeBidPriceTableItem> ObeBidPriceTableItemList) {
        return obeBidPriceTableItemMapper.updateBatchList(ObeBidPriceTableItemList);
    }

    /**
     * @param ObeBidPriceTableItemList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeBidPriceTableItem> ObeBidPriceTableItemList) {
        return obeBidPriceTableItemMapper.insertByBatch(ObeBidPriceTableItemList);
    }

    /**
     * @param ObeBidPriceTableItem
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeBidPriceTableItem ObeBidPriceTableItem, String[] attributes) {
        return obeBidPriceTableItemMapper.updateAssignProperty(ObeBidPriceTableItem, attributes) > 0;
    }

    /**
     * @param ObeBidPriceTableItemList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeBidPriceTableItem> ObeBidPriceTableItemList, String[] attributes) {
        return obeBidPriceTableItemMapper.batchUpdateProperty(ObeBidPriceTableItemList, attributes);
    }

    /**
     * @param ObeBidPriceTableItemList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeBidPriceTableItem> ObeBidPriceTableItemList, String[] attributes) {
        List<ObeBidPriceTableItem> insertObeBidPriceTableItemList = new ArrayList<>();
        List<ObeBidPriceTableItem> updateObeBidPriceTableItemList = new ArrayList<>();
        for (ObeBidPriceTableItem ObeBidPriceTableItem : ObeBidPriceTableItemList) {
            if (StringUtils.isEmpty(ObeBidPriceTableItem.getId())) {
                ObeBidPriceTableItem.setId(IDUtils.getId());
                insertObeBidPriceTableItemList.add(ObeBidPriceTableItem);
            } else {
                updateObeBidPriceTableItemList.add(ObeBidPriceTableItem);
            }
        }
        int count = 0;
        if (insertObeBidPriceTableItemList.size() > 0) {
            count += obeBidPriceTableItemMapper.insertByBatch(insertObeBidPriceTableItemList);
        }
        if (updateObeBidPriceTableItemList.size() > 0) {
            count += obeBidPriceTableItemMapper.batchUpdateProperty(updateObeBidPriceTableItemList, attributes);
        }
        return count;
    }

    /**
     * @param priceTableId
     * @return
     * @Description: 根据报价表id获取
     * @author chenxw
     */
    public List<ObeBidPriceTableItem> getByPriceTableId(String priceTableId) {
        QueryWrapper<ObeBidPriceTableItem> obeBidPriceTableItemQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableItemQueryWrapper.lambda()
                .eq(ObeBidPriceTableItem::getPriceTableId, priceTableId)
                .orderByAsc(ObeBidPriceTableItem::getSortNo);
        return obeBidPriceTableItemMapper.selectList(obeBidPriceTableItemQueryWrapper);
    }

}
