package com.gx.obe.server.management.table.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.table.dao.ObeBidPriceTableTitleMapper;
import com.gx.obe.server.management.table.entity.ObeBidPriceTableTitle;
import com.gx.obe.server.management.table.service.ObeBidPriceTableTitleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author mazc
 * @Description:
 */
@Service
@Transactional
public class ObeBidPriceTableTitleServiceImpl extends ServiceImpl<ObeBidPriceTableTitleMapper, ObeBidPriceTableTitle> implements ObeBidPriceTableTitleService {

    @Autowired
    private ObeBidPriceTableTitleMapper obeBidPriceTableTitleMapper;

    /**
     * @param ObeBidPriceTableTitleList
     * @Description: 批量更新
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer updateByBatch(List<ObeBidPriceTableTitle> ObeBidPriceTableTitleList) {
        return obeBidPriceTableTitleMapper.updateBatchList(ObeBidPriceTableTitleList);
    }

    /**
     * @param ObeBidPriceTableTitleList
     * @Description: 批量插入
     * @author mazc
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer insertByBatch(List<ObeBidPriceTableTitle> ObeBidPriceTableTitleList) {
        return obeBidPriceTableTitleMapper.insertByBatch(ObeBidPriceTableTitleList);
    }

    /**
     * @param ObeBidPriceTableTitle
     * @param attributes
     * @Description: 指定字段修改
     * @author chenxw
     */
    @Override
    public boolean updateAssignProperty(ObeBidPriceTableTitle ObeBidPriceTableTitle, String[] attributes) {
        return obeBidPriceTableTitleMapper.updateAssignProperty(ObeBidPriceTableTitle, attributes) > 0;
    }

    /**
     * @param ObeBidPriceTableTitleList
     * @param attributes
     * @Description: 批量指定字段修改
     * @author chenxw
     */
    @Override
    public Integer batchUpdateProperty(List<ObeBidPriceTableTitle> ObeBidPriceTableTitleList, String[] attributes) {
        return obeBidPriceTableTitleMapper.batchUpdateProperty(ObeBidPriceTableTitleList, attributes);
    }

    /**
     * @param ObeBidPriceTableTitleList
     * @param attributes
     * @return
     * @Description: 批量添加或跟新
     * @author chenxw
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer batchSaveOrUpdate(List<ObeBidPriceTableTitle> ObeBidPriceTableTitleList, String[] attributes) {
        List<ObeBidPriceTableTitle> insertObeBidPriceTableTitleList = new ArrayList<>();
        List<ObeBidPriceTableTitle> updateObeBidPriceTableTitleList = new ArrayList<>();
        for (ObeBidPriceTableTitle ObeBidPriceTableTitle : ObeBidPriceTableTitleList) {
            if (StringUtils.isEmpty(ObeBidPriceTableTitle.getId())) {
                ObeBidPriceTableTitle.setId(IDUtils.getId());
                insertObeBidPriceTableTitleList.add(ObeBidPriceTableTitle);
            } else {
                updateObeBidPriceTableTitleList.add(ObeBidPriceTableTitle);
            }
        }
        int count = 0;
        if (insertObeBidPriceTableTitleList.size() > 0) {
            count += obeBidPriceTableTitleMapper.insertByBatch(insertObeBidPriceTableTitleList);
        }
        if (updateObeBidPriceTableTitleList.size() > 0) {
            count += obeBidPriceTableTitleMapper.batchUpdateProperty(updateObeBidPriceTableTitleList, attributes);
        }
        return count;
    }

    /**
     * @param priceTableId
     * @return
     * @Description: 根据报价表id获取
     * @author chenxw
     */
    public List<ObeBidPriceTableTitle> getByPriceTableId(String priceTableId) {
        QueryWrapper<ObeBidPriceTableTitle> obeBidPriceTableQueryWrapper = new QueryWrapper<>();
        obeBidPriceTableQueryWrapper.lambda()
                .eq(ObeBidPriceTableTitle::getPriceTableId, priceTableId)
                .orderByAsc(ObeBidPriceTableTitle::getSortNo);
        return obeBidPriceTableTitleMapper.selectList(obeBidPriceTableQueryWrapper);
    }

}
