package com.gx.obe.server.management.user.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gx.obe.server.common.utils.IDUtils;
import com.gx.obe.server.common.utils.StringUtils;
import com.gx.obe.server.management.evaluation.entity.EvaluationStep;
import com.gx.obe.server.management.user.dao.AuthUserMapper;
import com.gx.obe.server.management.user.entity.AuthUserEntity;
import com.gx.obe.server.management.user.service.AuthUserService;
  /**  
 * 
 * @author 
 * @Title: AuthUserService
 * @Package com.gx.obe.server.management.user.service  
 * @Description:     服务实现类
 * @author: XiaoMa Pro
 * @date:   2019-04-25   
 * @version V1.0 
 * @Copyright: 2019 www.msypro.com Inc. All rights reserved. 
 *  
 */
@Service
public class AuthUserServiceImpl extends ServiceImpl<AuthUserMapper, AuthUserEntity> implements AuthUserService {
	
	@Autowired
	private AuthUserMapper authUserMapper;
	
	@Override
	public boolean valLoginAuthority(String userId, String terminal) {
		return authUserMapper.valLoginAuthority(userId, terminal) > -1;
	}

	@Override
	@Transactional(rollbackFor = Exception.class) 
	public AuthUserEntity caLogin(String caName, String certId) {
		AuthUserEntity  user = null;
		if(null != certId){
			user = authUserMapper.findCertId(certId);
		}
		if(null != user){
			return user;
		}
		QueryWrapper<AuthUserEntity>  queryWrapper = new QueryWrapper<AuthUserEntity>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserName, caName);
		List<AuthUserEntity> userList = authUserMapper.selectList(queryWrapper);
		if(null != userList && userList.size() > 0){
			// 如果只有一个专家
			if(userList.size() == 1){
				return userList.get(0);
			}else {
				// TODO 如果存在多个
				return userList.get(0);
			}
			
		}else {
			return null;
		}
	}
	/**
	 * @Description: 获得评标步骤的评标专家
	 * @author mazc
	 * @param tenderId
	 * @param evaluationStep
	 * @return 
	 */
	@Override
	public List<AuthUserEntity>findEvaluationStepUserList(String tenderId, EvaluationStep evaluationStep){
		return authUserMapper.findEvaluationStepUserList(tenderId, evaluationStep);
	}

	/** 
	 * @Description:  查询项目所对用的用户信息  
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	@Override
	public AuthUserEntity findUserInfoByTenderId(String tenderId) {
		return authUserMapper.findUserInfoByTenderId(tenderId);
	}

	@Override
	public List<AuthUserEntity> getUserAllByStatus(String useStatus , String userType) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<AuthUserEntity>();
		queryWrapper.select(AuthUserEntity.ID,AuthUserEntity.USER_NAME,AuthUserEntity.USER_ACCOUNT,AuthUserEntity.USER_IDCARD,
				AuthUserEntity.GENDER,AuthUserEntity.USER_TYPE).lambda().eq(AuthUserEntity::getUseStatus, useStatus);
		if(StringUtils.isNotEmpty(userType)) {
			queryWrapper.lambda().eq(AuthUserEntity::getUserType, userType);
		}
		queryWrapper.lambda().orderByDesc(AuthUserEntity::getCreateTime);
		return authUserMapper.selectList(queryWrapper);
	}

	/** 
	 * @Description: 通过用户名查询用户信息
	 * @author mazc
	 * @param userName
	 * @return 
	 */
	@Override
	public AuthUserEntity getUserByName(String userName) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<AuthUserEntity>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserName, userName);
		return authUserMapper.selectOne(queryWrapper);
	}

	/** 
	 * @Description: 
	 * @author mazc
	 * @param authUserEntity
	 * @return 
	 */
	@Override
	public AuthUserEntity saveOrUpdateReturn(AuthUserEntity authUserEntity) {
		if(null != authUserEntity && null != authUserEntity.getId() && authUserEntity.getId().length() > 0) {
			authUserMapper.updateById(authUserEntity);
			return authUserMapper.selectById(authUserEntity.getId());
		}else {
			authUserEntity.setId(IDUtils.getId());
			authUserMapper.insert(authUserEntity);
			return authUserEntity;
		}
	}

	/** 
	 * @Description: 通过用户Useraccount获取用户信息
	 * @author mazc
	 * @param userAccount
	 * @return 
	 */
	@Override
	public AuthUserEntity getUserByUserAccount(String userAccount) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<AuthUserEntity>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserAccount, userAccount);
		return authUserMapper.selectOne(queryWrapper);
	}

	@Override
	public List<AuthUserEntity> getExpertByProjectId(String projectId) {
		return authUserMapper.getExpertByProjectId(projectId);
	}

	/** 
	 * @Description: 获取当前系统临时用户数量
	 * @author mazc
	 * @return 
	 */
	@Override
	public int getTempUserNum() {
 		return authUserMapper.getTempUserNum();
	}

	/** 
	 * @Description: 通过账号或者身份证验证用户是否存在
	 * @author mazc
	 * @param userAccont
	 * @param idCard
	 * @return 
	 */
	@Override
	public AuthUserEntity getUserByIdOrAccont(String userAccont, String idCard) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<AuthUserEntity>();
		if(null != userAccont  && userAccont.length() > 0){
			queryWrapper.lambda().eq(AuthUserEntity::getUserAccount, userAccont);
			return authUserMapper.selectOne(queryWrapper);
		}else if(null != idCard  && idCard.length() > 0){
			queryWrapper.lambda().eq(AuthUserEntity::getUserIdcard, idCard);
			return authUserMapper.selectOne(queryWrapper);
		}else {
			return null;
		}
	}

	/** 
	 * @Description: 根据用户名和用户类型，获得用户列表。
	 * @author mazc
	 * @param userType
	 * @param userName
	 * @return 
	 */
	@Override
	public List<AuthUserEntity> getUserListByUserName(String userType, String userName) {
		return authUserMapper.getUserListByUserName(userType, userName);
	}

	@Override
	public int updateUsestatus(String id, String useStatus) {
		return authUserMapper.updateUsestatus(id, useStatus);
	}

	@Override
	public AuthUserEntity getUserByIdCard(String idCard) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		queryWrapper.lambda().eq(AuthUserEntity::getUserIdcard, idCard);
		return authUserMapper.selectOne(queryWrapper);
 	}

	@Override
	public List<AuthUserEntity> getUserAllByUserAccount(String userAccount) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<AuthUserEntity>();
		queryWrapper.lambda().like(AuthUserEntity::getUserAccount, userAccount);
 		return authUserMapper.selectList(queryWrapper);
	}

	@Override
	public List<AuthUserEntity> getUserAllByUserTypeUserNameOrUserAccount(String userAccount, String userName,String userType) {
		QueryWrapper<AuthUserEntity> queryWrapper = new QueryWrapper<>();
		if(StringUtils.isNotEmpty(userAccount)){
			queryWrapper.lambda().like(AuthUserEntity::getUserAccount, userAccount);
		}
		if(StringUtils.isNotEmpty(userName)){
			queryWrapper.lambda().like(AuthUserEntity::getUserName, userName);
		}
		if(StringUtils.isNotEmpty(userType)){
			queryWrapper.lambda().eq(AuthUserEntity::getUserType, userType);
		}
		queryWrapper.lambda().orderByDesc(AuthUserEntity::getCreateTime);
 		return authUserMapper.selectList(queryWrapper);
	}

	@Override
	public IPage<AuthUserEntity> getUserPageList(Page<AuthUserEntity> page, String account, String userName,
			String roleType) {
		return authUserMapper.getUserPageList(page, account, userName, roleType);
	}
	
	/**
     * @Description: 更新分配属性
     * @author chenxw
     * @param authUserEntity
     * @param attributes
     * @return 
     */
	@Override
    public boolean updateAssignProperty(AuthUserEntity authUserEntity, String[] attributes) {
        return authUserMapper.updateAssignProperty(authUserEntity, attributes) > 0;
    }

	@Override
	public List<AuthUserEntity> getGroupItemUserList(String id) {
		return authUserMapper.getGroupItemUserList(id);
	}

	  @Override
	  public String updatePhone(String id, String oldPhone, String newPhone) {
		  String returnFlag = "{\n\"suc\":\"0\"\n}";
		  if (oldPhone != null && newPhone != null) {
			  LambdaUpdateWrapper<AuthUserEntity> wrapper =new LambdaUpdateWrapper<>();
			  wrapper.set(AuthUserEntity::getPhone,newPhone);
			  wrapper.set(AuthUserEntity::getUserAccount,newPhone);
			  wrapper.eq(AuthUserEntity::getUserAccount,oldPhone);
			  this.update(wrapper);
			  //this.authUserMapper.updatePhone(id, oldPhone, newPhone);
			  returnFlag = "{\n\"suc\":\"1\"\n}";
		  }

		  return returnFlag;
	  }

  }
